package gui;
import uchicago.src.sim.analysis.DataRecorder;
import uchicago.src.sim.analysis.OpenSequenceGraph;
import uchicago.src.sim.analysis.PlotModel;
import uchicago.src.sim.analysis.Sequence;
import uchicago.src.sim.engine.BasicAction;
import uchicago.src.sim.engine.Schedule;
import uchicago.src.sim.engine.SimModelImpl;
import uchicago.src.sim.gui.AbstractGraphLayout;
import uchicago.src.sim.gui.CircularGraphLayout;
import uchicago.src.sim.gui.DisplaySurface;
import uchicago.src.sim.gui.Network2DDisplay;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;

import units.*;
import hivaids.*;
import infastructure.*;
import clubs.*;
import economics.*;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;
import org.cfpm.ruth.modifiedCloud.Market;
import org.cfpm.ruth.modifiedCloud.FarmingWorld;

import cern.jet.random.Normal;
import cern.jet.random.engine.MersenneTwister;

//### for Jess
import jess.*;

import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

public class Model extends SimModelImpl {   
	
	//### for Jess
	public Rete engine;
	
    private int initialSteps = 0;     
    private Schedule schedule;
    private BasicAction initialAction;    
       
    //the following is understood from the UNRAVEL 
    private int minHouseholdSize;
    private int maxHouseholdSize;
    private int initNumHouseholds;
    //e.g. 4 means the number of clusters are total houses / 4 
    private int clusterProportion;         
    protected ArrayList<Agent> agentList = new ArrayList<Agent>();       
    private ArrayList<Household> householdList = new ArrayList<Household>();
    private ArrayList<HouseholdCluster> clusterList = new ArrayList<HouseholdCluster>();    
     
     
    private HashMap <Integer, Agent> agentMap = new HashMap<Integer, Agent>();
    private HashMap<Integer, Household> houseMap = new HashMap<Integer, Household>();
    private HashMap<Integer, HouseholdCluster> clusterMap = new HashMap<Integer, HouseholdCluster>();
    private HashMap<Integer, Agent> aidsPatients = new HashMap<Integer, Agent>();
	
	private ArrayList<FuneralClub> funeralClubs = new ArrayList<FuneralClub>();
	private ArrayList<SavingsClub> savingsClubs = new ArrayList<SavingsClub>();
	
	private int initialFuneralClubs;
	
    private FinanceAgency agency;
    private int lastAgentID = 0;
    private int lastHouseID = 0;
    
    private AidsSpread aidSpread = new AidsSpread();    
   
    private int NUM_RECORDS = 1000;
    
    private ModelGUI gui;
    
    //here begins the display
    private DisplaySurface surface;
    private AbstractGraphLayout graphLayout;
    Network2DDisplay displayNetwork;  
    
    private ArrayList<Message> inbox = new ArrayList<Message>();
    private ArrayList<Message> outbox = new ArrayList<Message>();
    
    public int agentsDead = 0;    
    private Sigmoid sig = new Sigmoid();
    
    //do we want this to be included in our simulation
    private boolean isFuneralClub = true;
    //do we want this to be included in our simulation
    private boolean isHIV = true;
    
    //SEI CLOUD
    private FarmingWorld world = new FarmingWorld(); //###
    private Market market = new Market();
    private boolean isFarm = true;    
    //proportion of households owning a land parcels ... for growing
    private double proportionFarm; 
    
//  for collection of data
    private OpenSequenceGraph[] plotGraph;
    private boolean showPlot = true;           
//    private String charts[] = {"Adult deaths with and without HIV/AIDS", "Households dissolved", 
//    		"Number of Savings Clubs", "Household Wealth Impact", "Eligible Labour", "Orphans", 
//    		"Number of Adult Males", "Funeral Club Membership"};
    private String charts[] = {};    		
    //some global variables   
    int deathHIV = 0, deathWithoutHIV = 0, numHHdissolved = 0, numSavClub = 0, numHHaccom = 0, fcMembers = 0;    
    public double hhWealth = 0; int numAdultMales = 0;
    public int numOrphans=0;
    int eligibleLabFarm = 0, eligibleLabWoFarm = 0, numHH = 0;      
    // A DataRecorder allows data generated by a simulation to be written to a file
	private DataRecorder recorder;	
	//testing: for health expenses: set as public
	public boolean healthExpense = true;	
	//my normal object
	Normal myNormal = new Normal( Settings.MEAN_LIFE_EXPECTANCY, Settings.SD_LIFE_EXPECTANCY, new  MersenneTwister());
    
	private HashMap<Integer,HashMap<Integer,byte[]>> stokvelNetwork = new HashMap<Integer, HashMap<Integer,byte[]>>();
	
	public Model() {		
    }

    public void buildModel() { 
    	createHouseholds();
    	createHouseholdClusters();
    	agency = new FinanceAgency(this);
		createFuneralClubs();
		for (int i=0; i<charts.length; i++) {
	        if (showPlot) {
	            makePlot(i);	        
	        }	        			
		}
//		recorder = new DataRecorder("impact_data.txt", this);
//		recorder.addObjectDataSource("", new DataSource() {
//			public Object execute() {
//				return new Double(hhWealth);
//			}
//		});
    }
		
	public void createFuneralClubs() {
		for (int i=0; i<this.initialFuneralClubs; i++) {		
			FuneralClub funeralClub = new FuneralClub(i);
			initializeFuneralClub(funeralClub);
			this.funeralClubs.add(funeralClub);
		}
	}
	
	/*
	 * populate it with some members for a kick-start ...*  
	 */
	public void initializeFuneralClub(FuneralClub club) {
		SimUtilities.shuffle(this.householdList);
		int initialSize = Random.uniform.nextIntFromTo(3,5);		
		for (Household household : this.householdList) {			 
			if (!household.isMemberFuneralClub() && (initialSize>0)) {
				club.addMember(household);
				household.setMemberFuneralClub(true);
				household.setFuneralClub(club);
			}
			initialSize--;
		}		
	}
    
    public void createHouseholds () {
    	int size = 0, agentCount = 0;
    	for (int i=0; i<this.initNumHouseholds; i++) {    	
    		size = Random.uniform.nextIntFromTo(minHouseholdSize, maxHouseholdSize);
    		Household household = new Household(i, this);
    		lastHouseID = i;
//    		System.out.println("Created Household: " + household.getId());    		
    		agentCount = createHouseholdMembers(household, size, agentCount);
        	/*now set head, grants (if any) */
        	household.initialize();
        	if (Math.random() < this.proportionFarm) {
        		household.setFarm(true);
        	}
        	else {
//        		int wealth = Random.uniform.nextIntFromTo(2000, Settings.STARTING_WEALTH);
        		household.addWealth(Settings.STARTING_WEALTH);
//        		household.addWealth((double)wealth);
        	}
    		householdList.add(household);
    		houseMap.put(new Integer(household.getId()), household);    
			//### for Jess: put household as shadow fact into memory
			try {
				engine.definstance("household", household, true);
			}
			catch (JessException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();							
			}
    	}    		    
    	this.lastAgentID = agentCount;
    }
    
    /* 
     * this is to match the 'anectodal comments: . [in reality there would be a few hh  possibly 30% with no seniors, 
     * there would also be some with no children 5%, and some with seniors and no adults but still children, 5%]
     * this is just to improve towards it ...
     * another (possible) bug is that the adults are not related to seniors at the start ... this needs to be
     * taken care of ... 
     */
    public int createHouseholdMembers(Household household, int size, int curCount) { 
    	//this is a speical case ... one female and one male
    	Agent husband = new Agent(curCount, Settings.GENDER.MALE, Settings.AGE.ADULT, Settings.HEALTH_STATUS.WELL, household.getId(), this);    		   	
    	curCount++; household.addAgent(husband);    	   	
    	Agent wife = new Agent(curCount, Settings.GENDER.FEMALE, Settings.AGE.ADULT, Settings.HEALTH_STATUS.WELL, household.getId(), this);    	       	
    	curCount++; household.addAgent(wife); 
    	//minus husband and wife
    	size -= 2;   	    	
    	household.addCouple(husband, wife);
    	
    	Agent agent; 
    	int gender = -1, ageGroup = -1, numSeniors = Random.uniform.nextIntFromTo(0, 2);
    	
    	while ((numSeniors > 0) && (size > 0)) {
    		gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE : Settings.GENDER.MALE;
    		agent = new Agent(curCount, gender, Settings.AGE.SENIOR, Settings.HEALTH_STATUS.OLD_AGE_SICKNESS, household.getId(), this);    		
    		curCount++; household.addAgent(agent);
    		//Problem: still spouse id is missing...needs to be set
    		if (size == 2) {
    			agent.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);    			
    		}
    		else {
    			agent.setMaritalStatus(Settings.MARITAL_STATUS.WIDOWED);
    		}
    		size--; numSeniors--;
    	}
    	
    	int numOtherAdults = Random.uniform.nextIntFromTo(0, 2);    	
    	
    	while ((numOtherAdults > 0) && (size > 0)) {
    		gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE : Settings.GENDER.MALE;
    		agent = new Agent(curCount, gender, Settings.AGE.ADULT, Settings.HEALTH_STATUS.WELL, household.getId(), this);
    		curCount++; household.addAgent(agent);
    		size--; numOtherAdults--;
    	}
    	
    	for (int i=0; i<size; i++) {    						    		    		
    		gender = Math.random() < 0.50 ? Settings.GENDER.FEMALE : Settings.GENDER.MALE;
    		ageGroup = Math.random() < 0.0 ? Settings.AGE.ADULT : Settings.AGE.CHILD; 
    		agent = new Agent(curCount, gender, ageGroup, Settings.HEALTH_STATUS.WELL, household.getId(), this);
    		if (ageGroup == Settings.AGE.CHILD) {
    			agent.setFatherID(husband.getId());
        		agent.setMotherID(wife.getId());	
    		}    		  
    		household.addAgent(agent);
    		curCount++;    		
    	}    	        	    	

    	//this is just an idea to assign the seniors as parents of adults ... which could be
    	//it can be optimized a lot of times ...   	
    	for (Agent senior : household.getSeniors()) {
    		int seniorAge = senior.getAge(), seniorID = senior.getId(), 
    		seniorGender = senior.getGender();
    		for (Agent adult : household.getAdults()) {
    			if (seniorAge - adult.getAge() > 15) {   				    				
    				if (seniorGender == Settings.GENDER.FEMALE){
    					adult.setMotherID(seniorID);
    				}
    				else {
    					adult.setFatherID(seniorID);
    				}    				    				
    			}
    		}
    	}    	 
    	return curCount;
    }
       
    /**
     * this function is called when child agent, if any, is to be created during the simulation run 
     */
    public void createChildAgent(int _id, Couple couple, Household household) {
    	int gender = Math.random() > 0.5 ? Settings.GENDER.FEMALE :Settings.GENDER.MALE;
    	Agent agent = new Agent(_id, gender, Settings.HEALTH_STATUS.WELL, household.getId(), this);    	
    	Agent father = couple.getMale();
    	Agent mother = couple.getFemale();
    	agent.setFatherID(father.getId());
    	agent.setMotherID(mother.getId());
    	/*
    	 * from avert.org: "Without treatment, around 15-30% of babies born to HIV positive women will become 
    	 * infected with HIV during pregnancy and delivery."
    	 * here the assumption is that their is no nevirapine or Antiretroviral drugs are available ... which 
    	 * can reduce the risk to as low (perhaps ideally) to a below 2%
    	 * we're making it 30% chance of getting HIV from mother; mother to child transmission (MTCT) 
    	 */    	
    	if ((mother.getHealthStatus() == Settings.HEALTH_STATUS.HIV) 
    			&& (Math.random() <= 0.30)){
    		agent.setHealthStatus(Settings.HEALTH_STATUS.HIV);
    		agent.setInfectedTick((int)getTickCount());
    	}
    	household.addAgent(agent);	
		this.lastAgentID = _id;	
    }    
   
    public void createHouseholdClusters(){
    	SimUtilities.shuffle(householdList);    	
    	int totalHouses = householdList.size();
    	int num = totalHouses/clusterProportion, count=0;
    	ArrayList<Household> tempHouses = new ArrayList<Household>(householdList);    	
    	for (int i=0; i<num; i++) {
    		HouseholdCluster cluster = new HouseholdCluster(i, this);    		
    		while ((count < clusterProportion) && (!tempHouses.isEmpty())) {
    			Household household = (Household) tempHouses.get(0);
    			cluster.addHousehold(household);
    			System.out.println("Household cluster : " + cluster.getId() + " gets household: " + household.getId());    			
    			tempHouses.remove(0);
    			count++;
    		}
    		cluster.upadateMembers();
    		clusterList.add(cluster);
    		clusterMap.put(new Integer(cluster.getId()), cluster);    		
    		count=0;
    	}    	
    } 
   
    public void buildDisplay() {
    	int xSize = 1024, ySize = 768;
        graphLayout = new CircularGraphLayout(gui.getHouseNodeList(), xSize, ySize);        
        //graphLayout = new FruchGraphLayout(gui.getHouseNodeList(), xSize, ySize);
        displayNetwork = new Network2DDisplay(graphLayout);
        graphLayout.updateLayout();        
        surface.addDisplayableProbeable(displayNetwork, "Social links of Households");
        surface.addZoomable(displayNetwork);
        surface.setBackground(java.awt.Color.white);
        addSimEventListener(surface);       
    }                

    public void initialAction() {
    	SimUtilities.shuffle(this.agentList);
    	for (Agent agent : this.agentList) {
    		agent.initializeFriends();
    	}
    }    
  
    public void mainAction() {  	
    	numHH = householdList.size();
    	
    	ArrayList<Agent> dead = new ArrayList<Agent>();		
		SimUtilities.shuffle(agentList);			
		int currentTick = (int) this.getTickCount();	
		for(Agent agent : agentList) {    			
			agent.step(currentTick);    			
			if (!agent.getIsAlive()) {
				dead.add(agent);
				agentsDead++;
			}	
		}    		

		//the incidence is currently being set to annual ... am still fiddling with it 
		if ((currentTick % 12 == 0) && (isHIV)) {
			hivIncidence(dead, currentTick);
		}		 
	
		for (Agent agent: dead) {
			if (agent.getHealthStatus() == Settings.HEALTH_STATUS.HIV) {
				deathHIV++;
			}
			else {
				deathWithoutHIV++;
			}
			purgeAgent(agent);			
		}
		
		//allocate grant
		agency.step();		
		//System.out.println("Grant allocated.. tick count: " + getTickCount());			
			    		    		    		
		ArrayList<Household> noGuardian = new ArrayList<Household>();
		
		for (Household household : this.householdList) {			
			household.preStep();				
			if (household.getNoGuardians()) {
				noGuardian.add(household);
			}
			/*now create some agents
			 * right now only max. one birth is possible for a household
			 * currently its 10% chance for a child to born 
			 */   	
			
			if (household.getCoupleList().size() > 1) {
				int t = 0; t++;				
			}
			for (Couple couple : household.getCoupleList()) {
				if ((Math.random() <= Settings.BIRTH_PROBABILITY) && couple.birthPossible(currentTick)) {						
					this.lastAgentID++;
					createChildAgent(this.lastAgentID, couple, household);    				
				}	
			}			
		}    		
		
		numHHdissolved = noGuardian.size();
		
		for (Household household : noGuardian) {
			purgeHousehold(household);
		}		
		
		//### the agriculture module: needs to be updated before household.step() is called
		this.world.step();

		for (Household household : this.householdList) {			
			household.step();
			hhWealth += household.getWealth();
//			eligibleLabFarm += household.getCurrentMembersOnLabor();			
			numAdultMales += household.getCurrentAdultMales();
				
			if (household.isFarm()) {
				eligibleLabFarm += household.getCurrentMembersOnLabor();
			}
			else {
				eligibleLabWoFarm += household.getCurrentMembersOnLabor();
			}
		}
		
		//TODO: marriages occur ... but not nocessarily create new  households
		//occurs annually ... changed the order ... 
		if ((Math.random() <= Settings.MARRIAGE_PROBABILITY) && ((int) getTickCount()%12 == 0)) {		
			marriage();		
		}   	
    	
//		postOffice();  
		//### replaced by Jess stuff		
		try {
			//engine.executeCommand("(facts)");
			engine.run();
		}
		catch (JessException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();			
		}
   	
		//### try and write network file!
		if (getTickCount() <= 1) this.outputStokvelNetwork("/data/caves/modelling/stokvelNetwork.vna");
		
		for (FuneralClub club : this.funeralClubs) {
			club.step();
			fcMembers += club.getMembers().size();
		}
		
		for (SavingsClub club : this.savingsClubs) {
			if (club.isViable()) {
				numSavClub++;
				club.step();				
			}			
		}
		
		for (int i=0; i<charts.length; i++) {
			plotGraph[i].step();			
		}
		
//		recorder.record();		
//    	this.world.updateWorld();    	/#### moved further up (world.step())
    	surface.updateDisplay();
    	schedule.scheduleActionAt(NUM_RECORDS-1, this, "pause", Schedule.LAST);    	
    	refreshVariables();
    }
    
    /*
     * HIV/AIDS incidence method
     */
    public void hivIncidence(ArrayList<Agent> dead, int currentTick) {    	
		int count = 0;
		int proportionPatients = (int) aidSpread.returnIncidence();
		SimUtilities.shuffle(agentList);
		for (Agent agent : this.agentList) {
			if ((count <= proportionPatients)
					&& (agent.getHealthStatus() != Settings.HEALTH_STATUS.HIV)
					&& (agent.getAgeGroup() != Settings.AGE.CHILD)
					&& (!dead.contains(agent))) {    					
				agent.setHealthStatus(Settings.HEALTH_STATUS.HIV);
				agent.setInfectedTick(currentTick);
				//sets the current month as the agent's infected month					
				this.aidsPatients.put(new Integer(agent.getId()), agent);
				count++;        			
				if ((agent.getMaritalStatus() == Settings.MARITAL_STATUS.MARRIED)
						&& (agent.getSpouseID() != -1) 
						&& (this.agentList.contains(agent.getSpouseID()))) {
					//had an arrayOutofBound error ...
					Agent spouse = (Agent) this.agentList.get(agent.getSpouseID());
//						if (spouse != null) {
//							spouse.setHealthStatus(Settings.HEALTH_STATUS.HIV);
//						}	
					spouse.setHealthStatus(Settings.HEALTH_STATUS.HIV);						
				}															
			}				
		}    	
    }
    
    /**
     * this is a crude implementation ... i think, a lot better can be done ...
     * currently ... marriage, if possible, takes place at every year ... 
     */
    public void marriage() {
    	ArrayList<Household> temp = new ArrayList<Household>(householdList);
    	SimUtilities.shuffle(temp);
    	Agent husband = null, wife = null;    
    	boolean flagHusband = false, flagWife = false;    	

    	//this looks buggy ... this is random matching ... with nuclear-incest restriction only    	    	
    	for (Household household : temp) {
    		if (flagHusband && flagWife) {
    			break;
    		}    		
    		for (Agent adult : household.getAdults()) {
    			if ((adult.getMaritalStatus() == Settings.MARITAL_STATUS.SINGLE)
						&& (adult.getAge() <= Settings.MARRIAGE_AGE_LIMIT)) {
    				if (adult.getGender() == Settings.GENDER.FEMALE) {
    					if (wife == null) {
        					wife = adult; 
        					flagWife = true;    						
    					}
    				}
    				else if (adult.getGender() == Settings.GENDER.MALE){
    					if (husband == null) {
    						husband = adult;
    						flagHusband = true;
    					}
    				}
    				break;
    			}
    		}
    	}  	   	
   	
    	//returns true if the new household created successfully ... else false
    	if (!flagHusband || !flagWife) {
    		return;
    	}
    	
//    	createNewHousehold(husband, wife);
    	
    	//so we set the marriage ...    	    	
    	if (!husband.isYoungestSon()) {
    		createNewHousehold(husband, wife);	
    	}
    	else {
    		remainInSameHousehold(husband, wife);
    	}    	    	
    	//testing: added this new method ... 
    	checkSpouseHIV(wife, husband);   	    	
    }   
    
    public void remainInSameHousehold(Agent husband, Agent wife) {
    	Household husbandHousehold = (Household) houseMap.get(new Integer(husband.getHouseHoldID()));
    	Household wifeHousehold = (Household) houseMap.get(new Integer(wife.getHouseHoldID()));
    	wifeHousehold.removeAgent(wife);
    	husbandHousehold.addAgent(wife);
    	husbandHousehold.addCouple(husband, wife);    	
    	//problematic ... 
    	husbandHousehold.addWealth(dowry(husbandHousehold, wifeHousehold));
    }
               
    public void createNewHousehold(Agent husband, Agent wife) {   	   
    	Household husbandHousehold = (Household) houseMap.get(new Integer(husband.getHouseHoldID()));
    	Household wifeHousehold = (Household) houseMap.get(new Integer(wife.getHouseHoldID()));
    	int clusterID = husbandHousehold.getClusterID();
    	husbandHousehold.removeAgent(husband);
    	wifeHousehold.removeAgent(wife);   	    	 	
       	lastHouseID++; 
    	Household household = new Household(lastHouseID, this);
		System.out.println("Created NEW Household: " + household.getId());    	
    	household.addAgent(wife); household.addAgent(husband);     	
    	//initialize households 
    	household.initialize();
    	household.addCouple(husband, wife);
    	householdList.add(household);
    	this.houseMap.put(new Integer(household.getId()), household);    	    	
    	//the new household belongs to the cluster of the husband's cluster
    	household.setClusterID(clusterID);	    	    	
    	HouseholdCluster cluster = (HouseholdCluster) clusterMap.get(new Integer(clusterID));
    	cluster.addHousehold(household);
    	cluster.upadateMembers();    	    	
    	//now add to the gui - network ... and inherit all the links of all      	
    	this.gui.inheritLinks(household, husbandHousehold, wifeHousehold);
    	this.graphLayout.updateLayout();   	    	
    	household.addWealth(dowry(husbandHousehold, wifeHousehold));
    }

    /*
	 *  we want new households to get the money from within the model
	 *  dowry: actually, it's the bride's household that pays the amount ..
	 *   
	 */
    public double dowry(Household husbandHousehold, Household wifeHousehold) {    	
    	double amt = 1000;
    	double wealth = 0;
    	if ((husbandHousehold.getWealth() - amt) > Settings.FUNERAL_COST) {    		
    		 husbandHousehold.deductWealth((double)amt);
    		 wealth += amt;  
    	}
    	
    	if ((wifeHousehold.getWealth() - amt)> Settings.FUNERAL_COST) {
    		wifeHousehold.deductWealth((double)amt);
    		wealth += amt; 
    	}    	
    	return wealth;
    }
        
    //SJA: testing .. .added this new method
    public void checkSpouseHIV(Agent wife, Agent husband) {
    	if ((wife.getHealthStatus() == Settings.HEALTH_STATUS.HIV)
    			&& (husband.getHealthStatus() != Settings.HEALTH_STATUS.HIV)) {
    		husband.setHealthStatus(Settings.HEALTH_STATUS.HIV);
    		husband.setInfectedTick((int) getTickCount());
    	}
    	else if ((husband.getHealthStatus() == Settings.HEALTH_STATUS.HIV)
    			&& (wife.getHealthStatus() != Settings.HEALTH_STATUS.HIV)) {
    		wife.setHealthStatus(Settings.HEALTH_STATUS.HIV);
    		wife.setInfectedTick((int) getTickCount());    		
    	}
    }
        
    public void purgeHousehold(Household household) {
    	//SJA: testing 
    	if (household.getMembers().size() <= 0) {
    		removeHousehold(household);
    		return;
    	}
    	   	
		HouseholdCluster cluster = (HouseholdCluster) clusterMap.get(new Integer(household.getClusterID()));
		Household accommodator = null;	
		if ((cluster != null) && (household != null)) {
			accommodator = cluster.accomodate(household);
		}	
		//accommodate ... the neighbour (or friend?) accommodates ..		
		//couldn't find an extended-family household
		if (accommodator == null) {									
			accommodator = (Household) household.accommodateNeighbour();
			if (accommodator != null) {
				cluster.removeHousehold(household);
				accommodator.accomodate(household);
				cluster = (HouseholdCluster) clusterMap.get(new Integer(accommodator.getClusterID()));								
				cluster.upadateMembers(); 
			}			
			else {
				numHHaccom++;
			}
		}
		
		HouseholdNode node = (HouseholdNode) this.gui.getHouseholdNode(household);
		node.setColor(Color.BLACK);
	
		if (accommodator != null) {			
			//retrieve households connections ... if any?
			HouseholdNode accommodatorNode = (HouseholdNode) this.gui.getHouseholdNode(accommodator);			
			accommodatorNode.setColor(Color.BLUE);
			//this is an undirected relation ... so any would do ... 
			accommodatorNode.constructEdges(node.getToNodes());
		}		
		removeHousehold(household);
    }
    
    public void removeHousehold(Household household) {
		//remove from the gui - node and network ...		
    	HouseholdCluster cluster = (HouseholdCluster) clusterMap.get(new Integer(household.getClusterID()));
    	cluster.removeHousehold(household);
		this.gui.removeHoushold(household);	
		householdList.remove(household);		
		houseMap.remove(household);
		surface.updateDisplay();
		household.die(); //###
    }
    
    public void purgeAgent(Agent agent) {
    	if (this.aidsPatients.containsValue(agent)) {
    		this.aidsPatients.remove(agent);
    	}
    	Household household = (Household) houseMap.get(new Integer(agent.getHouseHoldID()));    	
    	/*this gets more complex when an adult dies because we then might have to 
    	 * move the household to another if there's no more adult left ... */    	    	
    	household.removeAgent(agent);
    	//CAUTION: THE AGESTILL EXISTS IN agentMAP:HASHMAP ... 
    	this.agentList.remove(agent);
    	agent.setIsAlive(false);
    }    
   
//    public void postOffice() {
///*    	this can be improved as currently only household heads are processing the messages
//    	but we leave it to the entire list, as we intend to have further agents' interactions
//    	and not just those between the household heads. 
//*/    	for (Agent agent : this.agentList) {
//    		//agent.processOutbox();
//    		if (!agent.getOutbox().isEmpty()) {
//    			this.outbox.addAll(agent.getOutbox());
//    			agent.getOutbox().clear();
//    		}
//    	}    	    	
//    	do {
//    		ArrayList<Agent> receivers = new ArrayList<Agent>();
//    		for (Message message : outbox) {
//    			Agent receiver = (Agent) message.getReceiver();
//    			receiver.getInbox().add(message);
//    			receivers.add(receiver);
//    		}
//    		this.outbox.clear();
//    		for (Agent agent : receivers) {
//    			agent.processInbox();
//    		}    		
//    	} while (!this.outbox.isEmpty());    	
//    	System.out.println("....");
//    	
//    	for (Household household : this.householdList) {
//    		Agent agent = household.getHead();
//    		if ((agent != null) && (agent.getAcceptances().contains(agent))){
//    			agent.createStokvel();
//    		}    		
//    	}
//    }
  	   
    public void removeInitialAction() {
        schedule.removeAction(initialAction);
    }

    protected void buildSchedule() {
        initialAction = schedule.scheduleActionAt(0, this, "initialAction");
        schedule.scheduleActionAt(initialSteps, this, "removeInitialAction",
                Schedule.LAST);
		schedule.scheduleActionBeginning(initialSteps + 1, this, "mainAction");		
    }

    public void begin() {
		 //#### changes for JESS
		 this.engine = new Rete();
		 initialiseRules();
		 
        buildModel();
        gui = new ModelGUI(this);
        buildDisplay();
        buildSchedule();        
        surface.display();

        for (int i=0; i<charts.length; i++) {
            if (showPlot) {
            	plotGraph[i].display();
            }       	        	
        }
    }
	 
	 //###
	 private void initialiseRules(){
		 // decision rules regarding savings clubs
		 String initStokvel = "(defrule initiate-stokvel " +
		 			"(agent (houseHoldID ?h)(innovator TRUE) (committed FALSE) (numStokvelTries 0) (head TRUE) (gender " + Settings.GENDER.FEMALE + ")(memberSavings FALSE) (OBJECT ?agent))" +
				   "(household (id ?h) (wealth ?w&:(>= ?w " + Settings.STOKVEL_AMOUNT + "))) " +
				   "=> " +
			      "(call ?agent sendInvitation)" +
			      ")";
		 			// sendInvitation() determines all female household heads in social neighbours and friends
		 			// and invites them = asserts message facts

		String acceptInvite = "(defrule accept-stokvel-invitation " +
			    	"(agent (id ?me) (houseHoldID ?h) (committed FALSE) (memberSavings FALSE) (OBJECT ?agent)) " +
			    	"(household (id ?h) (wealth ?w&:(>= ?w " + Settings.STOKVEL_AMOUNT + "))) " +
					"?m <- (message (receiver ?me) (sender ?sender) (type stokvel-invitation)) " +
//					"?m <- (message (receiver ?me) (sender ?sender&:(eq ?sender (decide-which ?me ?agent))) (type stokvel-invitation)) " +
					"(test (eq ?sender (decide-which ?me ?agent))) " + 
					"=> " +
			    	"(printout t \"ACCEPT fired for agent \t\" ?me \"\t and sender \t\" ?sender crlf) " +
			    	"(assert (message (receiver ?sender) (sender ?me) (type accept-stokvel))) " +
			    	"(retract ?m)" +
			    	"(call ?agent setCommitted TRUE) " +
			    	"(call ?agent updateStokvelNetwork ?sender " + Agent.ACCEPT + ") " +
			    	")";

		String rejectInvite = "(defrule reject-stokvel-invitation " +
					"?m <- (message (receiver ?me) (sender ?sender) (type stokvel-invitation)) " +
					"(or (agent (id ?me) (committed TRUE) (OBJECT ?agent)) " +
					"    (agent (id ?me) (memberSavings TRUE) (OBJECT ?agent)) " +
			    	"	  (and (agent (id ?me) (houseHoldID ?h) (committed FALSE) (memberSavings FALSE) (OBJECT ?agent)) " +
			    	"			 (household (id ?h) (wealth ?w&:(< ?w " + Settings.STOKVEL_AMOUNT + "))) " +
			    	"    ))" +
			    	"=> " +
			    	"(printout t \"REJECT fired for agent \t\" ?me \"\t and sender \t\" ?sender crlf) " +
			    	"(assert (message (receiver ?sender) (sender ?me) (type reject-stokvel)))" +
			    	"(retract ?m)" +
			    	"(call ?agent updateStokvelNetwork ?sender " + Agent.REJECT + ") " +
			    	")";


		String addInvitee = "(defrule add-invitee " +
					"(agent (id ?me) (innovator TRUE) (OBJECT ?innovator)) " +
					"?m <- (message (receiver ?me) (sender ?agent) (type accept-stokvel)) " +
					"=> " +
					"(call ?innovator handleAcceptance ?agent) " +
					"(retract ?m)" +
					")";
			// handleAcceptance checks if every invitee has answered and if there are enough accepts
			// then creates the stokvel; if not disbands the so far collected acceptances

		String removeInvitee = "(defrule remove-invitee " +
			    	"(agent (id ?me) (innovator TRUE) (OBJECT ?innovator)) " +
			    	"?m <- (message (receiver ?me) (sender ?agent) (type reject-stokvel)) " +
					"=> " +
					"(call ?innovator handleRejection ?agent) " +
					"(retract ?m)" +
					")";
		// handleRejection does the same
		
		//queries
		String qFindInvitations = "(defquery find-stokvel-invitations " +
					"(declare (variables ?agent)) " +
					"(message (receiver ?agent) (sender ?sender) (type stokvel-invitation)) " +
					")";
		
		// functions
		String fDecideWhichStokvel = "(deffunction decide-which (?me ?myObj) " +
					"(bind ?result (run-query* find-stokvel-invitations ?me)) " +
					"(return (call ?myObj decideWhichStokvel ?result)) " +
					")";
		 
		try {
			// define templates
			engine.executeCommand("(deftemplate message (slot receiver) (slot sender) (slot type) (slot contents))");
			// define classes used in facts and rules
			engine.executeCommand("(import units.*)");
			engine.defclass("agent", "Agent", null);
			engine.defclass("household", "Household", null);
			// define queries
			engine.executeCommand(qFindInvitations);
			// define functions
			engine.executeCommand(fDecideWhichStokvel);
			// define rules
			engine.executeCommand(initStokvel);
			engine.executeCommand(acceptInvite);
			engine.executeCommand(rejectInvite);
			engine.executeCommand(addInvitee);
			engine.executeCommand(removeInvitee);
			// set conflict resolution strategy to FIFO
			engine.executeCommand("(set-strategy breadth)");
			// set output
//			engine.watchAll();
			engine.executeCommand("(facts)");
			engine.executeCommand("(rules)");
//			engine.watch(WatchConstants.ACTIVATIONS);
		}
		catch(JessException e){
			System.out.println(e.getMessage());
			e.printStackTrace();			
		}	 
	 }
	 
	 public void updateStokvelNetwork(int fromAgent, int toAgent, int what){
		 Integer from = new Integer(fromAgent);
		 Integer to = new Integer(toAgent);
		 HashMap<Integer,byte[]> fromMap;
		 byte[] toRecord;
		 if (this.stokvelNetwork.containsKey(from)){
			 // already a record in there for fromAgent
			 fromMap = this.stokvelNetwork.get(from);
			 if (fromMap.containsKey(to)){
				 // already a record for toAgent
				 toRecord = fromMap.get(to);
			 }
			 else {
				 // create new record for toAgent
				 toRecord = new byte[5];
			 }
		 }
		 else {
			 // create new record for fromAgent and toAgent
			 fromMap = new HashMap<Integer, byte[]>();
			 toRecord = new byte[5];
		 }
		 // now put it all in
		 toRecord[what] = 1;
		 fromMap.put(to, toRecord);
		 stokvelNetwork.put(from, fromMap);
	 }
	 
	 private void outputStokvelNetwork(String fileName){
		 if (this.stokvelNetwork.isEmpty()) return;
		 
		 try {
			 // output the current state to a file in .vna format (for NetDraw)
			 File outFile = new File(fileName);
			 PrintWriter out = new PrintWriter(new FileOutputStream(outFile));
			 Integer[] agents = this.stokvelNetwork.keySet().toArray(new Integer[this.stokvelNetwork.size()]);
			 
			 //## just for debugging: printout the whole HashMap
//			 System.out.println("\nFROM\tTO\tinvite\tinvite2\taccept\treject\tstokvel");
//			 for (int i=0; i<agents.length; i++){
//				 HashMap<Integer,byte[]> fromMap = stokvelNetwork.get(agents[i]);
//				 Iterator toAgents = fromMap.keySet().iterator();
//				 while(toAgents.hasNext()){
//					 Integer to = (Integer)toAgents.next();
//					 byte[] record = fromMap.get(to);
//					 System.out.print(agents[i].intValue() + "\t" + to.intValue() + "\t");
//					 for (int j=0; j<record.length; j++){
//						 System.out.print(record[j] + "\t");
//					 }
//					 System.out.println();
//				 }
//			 }
			 
			 // first the list of nodes
			 out.println("*Node data");
			 out.println("\"ID\", \"Innovator\"");
			 for (int i = 0; i < agents.length; i++) {
				 Agent a = this.agentMap.get(agents[i]);
				 out.println("\"" + agents[i].intValue() + "\" " + (a.isInnovator()? 1 : 0));
			 }
			 
			 // then all the edges
			 out.println("*Tie data");
			 out.println("FROM TO Distance Accept Reject Stokvel");
			 for (int i = 0; i < agents.length; i++) {
				 HashMap<Integer, byte[]> fromMap = this.stokvelNetwork.get(agents[i]);
				 Iterator<Integer> toAgents = fromMap.keySet().iterator();
				 while (toAgents.hasNext()) {
					 Integer to = toAgents.next();
					 byte[] toRecord = fromMap.get(to);
					 out.print("\"" + agents[i].intValue() + "\" \"" + to.intValue() + "\" ");
					 // distance
					 if (toRecord[Agent.INVITE] == 1) out.print(1);
					 else if (toRecord[Agent.INVITE2] == 1) out.print(2);
					 else {
						 // look up other pair
						 byte[] other = stokvelNetwork.get(to).get(agents[i]);
						 out.print((other[Agent.INVITE] == 1? 1 : 2));
					 }
					 // accept and reject
					 out.print(" " + toRecord[Agent.ACCEPT] + " " + toRecord[Agent.REJECT]);
					 // stokvel
					 out.println(" " + toRecord[Agent.STOKVEL]);
				 }
			 }
			 
			 // close the output stream
			 out.flush();
			 out.close();		
		 }
		 catch(IOException e){
			 System.err.println(e.getMessage());
			 System.err.println(e.getStackTrace());
		 }
		 
	 }

    public void setup() {
    	schedule = null;
        System.gc();
        schedule = new Schedule();
        agentList = new ArrayList<Agent>();
        householdList = new ArrayList<Household>();
        clusterList = new ArrayList<HouseholdCluster>();
        surface = null;
        System.gc();        
        surface = new DisplaySurface(this, "Household Network Surface");
        registerDisplaySurface("Main Display", surface);             
        plotGraph = new OpenSequenceGraph[charts.length];        
        for (int i=0; i<charts.length; i++) {
            if (plotGraph[i] != null) {
            	plotGraph[i].dispose();
            }            
            plotGraph[i] = null;        	
        }
        
        Random.createUniform();
		//parameters		
		initNumHouseholds = 100;
//		isFarm = true;
		isFuneralClub = true;
		proportionFarm = Settings.PROPORTION_FARMS;
		initialFuneralClubs = 5;
		clusterProportion = 10;
		minHouseholdSize = 5;
	    maxHouseholdSize = 7;
    }   

    public String[] getInitParam() {
        String[] params = {"isFuneralClub, initNumHouseholds, minHouseholdSize," +
        		"maxHouseholdSize, proportionFarm, initialFuneralClubs, clusterProportion"};
        return params;		
    }

    public Schedule getSchedule() {
        return schedule;
    }

    public String getName() {
        return "SA_HIV/AIDS";
    }

    public static void main(String[] args) {
        uchicago.src.sim.engine.SimInit init = new uchicago.src.sim.engine.SimInit();
        Model model = new Model();
        init.loadModel(model, null, false);
    }

	public int getMaxHouseholdSize() {
		return maxHouseholdSize;
	}

	public void setMaxHouseholdSize(int maxHouseholdSize) {
		this.maxHouseholdSize = maxHouseholdSize;
	}

	public int getMinHouseholdSize() {
		return minHouseholdSize;
	}

	public void setMinHouseholdSize(int minHouseholdSize) {
		this.minHouseholdSize = minHouseholdSize;
	}

	public ArrayList<Agent> getAgentList() {
		return agentList;
	}

	public void setAgentList(ArrayList<Agent> agentList) {
		this.agentList = agentList;
	}

	public ArrayList<HouseholdCluster> getClusterList() {
		return clusterList;
	}

	public void setClusterList(ArrayList<HouseholdCluster> clusterList) {
		this.clusterList = clusterList;
	}

	public int getClusterProportion() {
		return clusterProportion;
	}

	public void setClusterProportion(int clusterProportion) {
		this.clusterProportion = clusterProportion;
	}

	public ArrayList<Household> getHouseholdList() {
		return householdList;
	}

	public void setHouseholdList(ArrayList<Household> householdList) {
		this.householdList = householdList;
	}

	public HashMap getClusterMap() {
		return clusterMap;
	}

	public void setClusterMap(HashMap<Integer, HouseholdCluster> clusterMap) {
		this.clusterMap = clusterMap;
	}

	public HashMap getHouseMap() {
		return houseMap;
	}

	public void setHouseMap(HashMap<Integer, Household> houseMap) {
		this.houseMap = houseMap;
	}

	public ModelGUI getGui() {
		return gui;
	}

	public void setGui(ModelGUI gui) {
		this.gui = gui;
	}

	public ArrayList<Message> getInbox() {
		return inbox;
	}

	public void setInbox(ArrayList<Message> inbox) {
		this.inbox = inbox;
	}

	public ArrayList<Message> getOutbox() {
		return outbox;
	}

	public void setOutbox(ArrayList<Message> outbox) {
		this.outbox = outbox;
	}

	public HashMap<Integer, Agent> getAgentMap() {
		return agentMap;
	}

	public void setAgentMap(HashMap<Integer, Agent> agentMap) {
		this.agentMap = agentMap;
	}

	public Network2DDisplay getDisplayNetwork() {
		return displayNetwork;
	}

	public void setDisplayNetwork(Network2DDisplay displayNetwork) {
		this.displayNetwork = displayNetwork;
	}

	public AbstractGraphLayout getGraphLayout() {
		return graphLayout;
	}

	public void setGraphLayout(AbstractGraphLayout graphLayout) {
		this.graphLayout = graphLayout;
	}

	public DisplaySurface getSurface() {
		return surface;
	}

	public void setSurface(DisplaySurface surface) {
		this.surface = surface;
	}

	public Sigmoid getSig() {
		return sig;
	}

	public void setWorld(FarmingWorld world) { //###
		this.world = world;
	}

	public FarmingWorld getWorld() { //###
		return world;
	}

	public Market getMarket() {
		return market;
	}

	public void setMarket(Market market) {
		this.market = market;
	}

	public ArrayList<FuneralClub> getFuneralClubs() {
		return funeralClubs;
	}

	public void setFuneralClubs(ArrayList<FuneralClub> funeralClubs) {
		this.funeralClubs = funeralClubs;
	}

	public ArrayList<SavingsClub> getSavingsClubs() {
		return savingsClubs;
	}

	public void setSavingsClubs(ArrayList<SavingsClub> savingsClubs) {
		this.savingsClubs = savingsClubs;
	}

	public int getInitNumHouseholds() {
		return initNumHouseholds;
	}

	public void setInitNumHouseholds(int initNumHouseholds) {
		this.initNumHouseholds = initNumHouseholds;
	}

	public double getProportionFarm() {
		return proportionFarm;
	}  	
	
	/*
	 * this does the bookkeeping actually ... 
	 */
	public void createSavingsClub(SavingsClub club) {
		this.savingsClubs.add(club);
		//now those members who're not in the members's friends' list are added 
		//this would be O(n^2)
		ArrayList<Agent> temp = new ArrayList<Agent>(club.getMembers());
		/* * CAUTION: BUGGY */
		for (Agent agent1 : temp) {
			agent1.setMemberSavings(true);
//			agent1.setCommitted(false);  //### and reset committed flag; no, is done in setMemberSavings()
			for (Agent agent2 : temp) {
				if ((agent1.getId() != agent2.getId())
						&& (!agent1.getFriends().contains(agent2))){
					agent1.getFriends().add(agent2);
					agent2.getFriends().add(agent1);
				}
			}
			
		}		
		//currently we're relying on a random schedule ... but later on 			
	}

	public boolean isFuneralClub() {
		return isFuneralClub;
	}

	public void setFuneralClub(boolean isFuneralClub) {
		this.isFuneralClub = isFuneralClub;
	}
	
	
	  /*
     * from Friedrich's multilayer ...
     */
    private void makePlot(int i) {
    	switch (i) {
    	case 0: 
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time", "Death");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("from HIV/AIDS", new Sequence() {
                public double getSValue() {
                    return deathHIV;
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);
            
            plotGraph[i].addSequence("other modes", new Sequence() {
                public double getSValue() {
                    return deathWithoutHIV; 
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);
            break;
    	case 1:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time", "Number of Households");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
           
            plotGraph[i].addSequence("households not accommodated", new Sequence() {
                public double getSValue() {
                    return numHHaccom;
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);           

            plotGraph[i].addSequence("households dissolved", new Sequence() {
                public double getSValue() {
                    return numHHdissolved;
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);

    		
    		break;
    	case 2:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time", "Number of Savings Clubs");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("number sav", new Sequence() {
                public double getSValue() {
                    return numSavClub;
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);              		
    		break;
    	case 3:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "/resource_graph_data.txt", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time", "Wealth Per Agent");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("", new Sequence() {
                public double getSValue() {
//                    return hhWealth / numHH;
                	return hhWealth / agentList.size();
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);              		
    		break;
    	case 4:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "/resource_graph_data.txt", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time in months", "Number of Labour Available");
//            plotGraph[i].setYRange(0, 1);
//            plotGraph[i].setYIncrement(1);            
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);
            plotGraph[i].addSequence("", new Sequence() {
                public double getSValue() {
                    return eligibleLabFarm;            		
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);

            plotGraph[i].addSequence("Labour from hh without farms", new Sequence() {
                public double getSValue() {
                    return eligibleLabWoFarm;
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);            
    		break;    	                	
    	case 5:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "/resource_graph_data.txt", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time in months", "Number of Orphans");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("", new Sequence() {
                public double getSValue() {
                    return numOrphans;            		
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);
            break;                    	
    	case 6:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "/resource_graph_data.txt", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time in months", "Number of Adult Males");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("", new Sequence() {
                public double getSValue() {
                    return (double) numAdultMales;            		
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);              		
    		break;    		
    	case 7:
            plotGraph[i] = new OpenSequenceGraph(charts[i], this,
                    "/resource_graph_data.txt", PlotModel.CSV);
            plotGraph[i].setAxisTitles("Time in months", "Funeral Clubs total members");
            plotGraph[i].setYRange(0, 1);
            plotGraph[i].setYIncrement(1);
            plotGraph[i].setYViewPolicy(uchicago.src.sim.analysis.OpenSequenceGraph.SHOW_ALL);

            plotGraph[i].addSequence("", new Sequence() {
                public double getSValue() {
                    return (double) fcMembers;            		
                }
            }, OpenSequenceGraph.FILLED_CIRCLE);              		
    		break;      		    		    		
    		
    	default: break;   
    	}
    }	
    
    public void refreshVariables() {
    	deathHIV = 0; 
    	deathWithoutHIV = 0;
    	numHHdissolved = 0;
		numSavClub = 0;
		hhWealth = 0;
		numHHaccom = 0;
		eligibleLabFarm = 0;
		eligibleLabWoFarm = 0;
		numOrphans = 0;
		numAdultMales=0;
		numHH = 0;
		fcMembers = 0;
    }

	public Normal getMyNormal() {
		return myNormal;
	}
}