package gui;

import java.awt.Color;
import java.util.ArrayList;
import java.util.HashMap;

import clubs.FuneralClub;
import uchicago.src.sim.network.NetworkFactory;
import units.Agent;
import units.Household;
import units.HouseholdCluster;

public class ModelGUI {
	
	private Model model; 
	private ArrayList<AgentNode> agentNodeList;
	private ArrayList<HouseholdNode> houseNodeList;
	private ArrayList<ClusterNode> clusterNodeList;
	
	private HashMap <Integer, AgentNodeMap>agentNodeMap = new HashMap <Integer, AgentNodeMap>();
	private HashMap <Integer, HouseholdNodeMap> householdNodeMap = new HashMap<Integer, HouseholdNodeMap>();
	private HashMap <Integer, ClusterNodeMap> clusterNodeMap = new HashMap<Integer, ClusterNodeMap>();
	
	private double swProb = 0.2; 
	
	public ModelGUI(Model _model) {
		this.model = _model;
		initialize();
	}		
	
	public void initialize() {
		initializeAgentNodes();
		initializeHouseholdNodes();		
		//initializeClusterNodes();
		//initializeFuneralClubLinks();
	}
	
	//social nodes at the agent's level
	public void initializeAgentNodes() {
		agentNodeList = (ArrayList<AgentNode>) NetworkFactory.createUnlinkedNetwork(model.getAgentList().size(), AgentNode.class);
		int i=0;
		for (AgentNode node : agentNodeList) {
			node.setAgent(model.getAgentList().get(i));
			node.setColor(Color.GREEN);
			node.setNodeLabel(" " + node.getAgent().getId());
			agentNodeMap.put(new Integer(node.getAgent().getId()), new AgentNodeMap(node.getAgent(), node));
			i++;
		}		
	}
	
	public void initializeHouseholdNodes() {		
		//houseNodeList = (ArrayList<HouseholdNode>) NetworkFactory.createUnlinkedNetwork(model.getHouseholdList().size(), HouseholdNode.class);
		houseNodeList = (ArrayList<HouseholdNode>) NetworkFactory.createWattsStrogatzNetwork(model.getHouseholdList().size(), 2, swProb, HouseholdNode.class, SAEdge.class);
		//houseNodeList = (ArrayList<HouseholdNode>) NetworkFactory.createRandomDensityNetwork(model.getHouseholdList().size(), )
		int i=0;
		for (HouseholdNode node : houseNodeList) {
			node.setHouse(model.getHouseholdList().get(i));
			node.setColor(Color.GREEN);
			node.setNodeLabel(" " + node.getHousehold().getId());
			householdNodeMap.put(new Integer(node.getHousehold().getId()), new HouseholdNodeMap(node.getHousehold(), node));
			i++;
		}
	}
	
	public void initializeFuneralClubLinks() {
		for (FuneralClub funeralClub : model.getFuneralClubs()) {
			for (Household member1 : (ArrayList<Household>) funeralClub.getMembers()) {
				HouseholdNode node1 =  householdNodeMap.get(new Integer(member1.getId())).node;
				ArrayList nodeList = new ArrayList();				
				for (Household member2 : (ArrayList<Household>) funeralClub.getMembers()) {
					if (member1 != member2) {						
						HouseholdNode node2 =  householdNodeMap.get(new Integer(member2.getId())).node;
						nodeList.add(node2);
					}
				}
				node1.constructEdges(nodeList, Color.blue);				
			}			
		}
	}
	
	public void initializeClusterNodes() {
		//clusterNodeList = (ArrayList<ClusterNode>)NetworkFactory.createUnlinkedNetwork(model.getClusterList().size(), ClusterNode.class);
		clusterNodeList = (ArrayList<ClusterNode>)NetworkFactory.createWattsStrogatzNetwork(model.getClusterList().size(), 1, 0.1, ClusterNode.class, SAEdge.class); 		
		int i=0;
		for (ClusterNode node : clusterNodeList) {
			node.setCluster(model.getClusterList().get(i));
			clusterNodeMap.put(new Integer (node.getCluster().getId()), new ClusterNodeMap(node.getCluster(), node));
			i++;
		}		
	}
	
	public void removeHoushold (Household household) {				
		HouseholdNode node = (HouseholdNode) this.householdNodeMap.get(new Integer(household.getId())).node;					
		node.clearEdges();					
		model.getGraphLayout().getNodeList().remove(node);
		householdNodeMap.remove(new Integer (household.getId()));
	}
	
	public void addHousehold(Household household) {
		HouseholdNode node = new HouseholdNode(household);
		householdNodeMap.put(new Integer(household.getId()), new HouseholdNodeMap(household, node));
		model.getGraphLayout().getNodeList().add(node);
		node.setColor(Color.ORANGE);
		node.setNodeLabel(" " + node.getHousehold().getId());
				
//		ArrayList<HouseholdNode> temp = (ArrayList<HouseholdNode>) model.getGraphLayout().getNodeList();
//		for (HouseholdNode other : temp) {
//			if ((node != other) && (Math.random() <= swProb)) {
//				SAEdge fromEdge = new SAEdge(node, other);
//				node.addInEdge(fromEdge); node.addOutEdge(fromEdge);
//				SAEdge toEdge = new SAEdge(other, node);
//				other.addInEdge(toEdge); other.addOutEdge(toEdge);
//			}
//		}
	}
	
	/*
	 * inherit parents' links in this new household ...
	 */
	public void inheritLinks(Household household, Household p1, Household p2) {
		addHousehold(household);
		HouseholdNode householdNode = (HouseholdNode) this.getHouseholdNodeMap().get(new Integer(household.getId())).node;
		//this is an undirected relation ... so any would do ...
		HouseholdNode p1Node = (HouseholdNode) this.getHouseholdNodeMap().get(new Integer(p1.getId())).node;
		HouseholdNode p2Node = (HouseholdNode) this.getHouseholdNodeMap().get(new Integer(p2.getId())).node;
		householdNode.constructEdges(p1Node.getToNodes());				
		householdNode.constructEdges(p2Node.getToNodes());			
		normalizeHouseholdNodes(householdNode);
	}
	
	public void decayEdges(Household household) {
		HouseholdNode householdNode = (HouseholdNode) this.getHouseholdNodeMap().get(new Integer(household.getId())).node;		
		householdNode.removeRandomLink();
	}
	
	public void normalizeHouseholdNodes (HouseholdNode householdNode) {
		int size = householdNode.getToNodes().size();
		for (int i=0; i<size/2; i++) {
			householdNode.removeRandomLink();
		}
	}
	
	public void decayEdges(HouseholdCluster cluster) {
	}
	
	public ArrayList<AgentNode> getAgentNodeList() {
		return agentNodeList;
	}

	public void setAgentNodeList(ArrayList<AgentNode> agentNodeList) {
		this.agentNodeList = agentNodeList;
	}

	public ArrayList<ClusterNode> getClusterNodeList() {
		return clusterNodeList;
	}

	public void setClusterNodeList(ArrayList<ClusterNode> clusterNodeList) {
		this.clusterNodeList = clusterNodeList;
	}

	public ArrayList<HouseholdNode> getHouseNodeList() {
		return houseNodeList;
	}

	public void setHouseNodeList(ArrayList<HouseholdNode> houseNodeList) {
		this.houseNodeList = houseNodeList;
	}
	
	public class AgentNodeMap {
		public Agent agent; 
		public AgentNode node;
		
		public AgentNodeMap(Agent _agent, AgentNode _node) {
			this.agent = _agent;
			this.node = _node;
		}		
	}
	
	public class HouseholdNodeMap {
		public Household household; 
		public HouseholdNode node;
		
		public HouseholdNodeMap(Household _household, HouseholdNode _node) {
			this.household = _household;
			this.node = _node;
		}		
	}

	public class ClusterNodeMap {
		public HouseholdCluster cluster; 
		public ClusterNode node;
		
		public ClusterNodeMap(HouseholdCluster _cluster, ClusterNode _node) {
			this.cluster = _cluster;
			this.node = _node;
		}		
	}

	public HashMap<Integer, AgentNodeMap> getAgentNodeMap() {
		return agentNodeMap;
	}

	public void setAgentNodeMap(HashMap<Integer, AgentNodeMap> agentNodeMap) {
		this.agentNodeMap = agentNodeMap;
	}

	public HashMap<Integer, ClusterNodeMap> getClusterNodeMap() {
		return clusterNodeMap;
	}

	public void setClusterNodeMap(HashMap<Integer, ClusterNodeMap> clusterNodeMap) {
		this.clusterNodeMap = clusterNodeMap;
	}

	public HashMap<Integer, HouseholdNodeMap> getHouseholdNodeMap() {
		return householdNodeMap;
	}
	
	public HouseholdNode getHouseholdNode(Household household) {
		HouseholdNode node = (HouseholdNode) (householdNodeMap.get(new Integer (household.getId()))).node;
		return node;
	}

	public void setHouseholdNodeMap(HashMap<Integer, HouseholdNodeMap>  householdNodeMap) {
		this.householdNodeMap = householdNodeMap;
	}
}