package gui;

import java.awt.Color;
import java.util.ArrayList;
import uchicago.src.sim.gui.RectNetworkItem;
import uchicago.src.sim.network.DefaultDrawableNode;
import uchicago.src.sim.network.DefaultNode;

/**
 * Node class. SandboxNode extends
 * DefaultDrawableNode and so it is both a DefaultNode and a
 * DrawableNonGridNode. All of the important behavoir here as to do with
 * creating new links.
 * @version
 */

/*
 * shah: this is a tweaking of the code by Friedrich for multilayer network model ... 
 */

public class Node extends DefaultDrawableNode {
    int maxDegree;    
    double initEdgeStrength;  

    public Node() {   
    }

    /**
     * Initialize this saNode, this takes the place of the constructor when
     * a SandboxNode is created from a file using its no-arg constructor.
     */
    public void init(int x, int y) {
        RectNetworkItem rect = new RectNetworkItem(x, y);
        setDrawable(rect);
    }
    
    /*
     * my own ... pretty inefficient
     */
    public void clearEdges() {    	
    	ArrayList <Node> fromNodes = (ArrayList<Node>)this.getFromNodes(); 
    	ArrayList <Node> toNodes = (ArrayList<Node>)this.getToNodes();    	
    	for (Node node : fromNodes) {
    		node.removeEdgesFrom(this);
    	}    	
    	for (Node node : toNodes) {
    		node.removeEdgesTo(this);
    	}    	    	
    	clearInEdges(); clearOutEdges();
    }
    
    //undirected edges
    public void constructEdges(ArrayList<Node> nodes) {
    	for (Node node : nodes) {    		
    		makeEdgeToFrom(node);
    	}    	    	
    }
    
    //undirected edges
    public void constructEdges(ArrayList<Node> nodes, Color color) {
    	for (Node node : nodes) {
    		makeEdgeToFrom(node, color);
    	}    	    	
    }

    /**
     * 
     * WARNING: BUGGY
     */
	public void removeRandomLink() {
		Node randomNode = (Node) this.getRandomNodeOut();
		if (randomNode != null) {
			this.removeEdgesTo(randomNode);
			randomNode.removeEdgesFrom(this);
			this.removeEdgesFrom(randomNode);
			randomNode.removeEdgesTo(this);			
		}
	}


    /**
     * Makes an edge to the specified node and from the specifed node to
     * this saNode if both nodes do not already have edges to each other and
     * if adding the edge keeps their degrees less than maxDegree. If an edge allready
     * exitsts, its strength is refreshed.
     * 
     * @param node
     * @param maxDegree
     */
    public void makeEdgeToFrom(DefaultNode node) {
        if (hasEdgeTo(node)){
            ArrayList<SAEdge> outE = new ArrayList<SAEdge>(getOutEdges());
            for (SAEdge e : outE) {
                if (node.equals(e.getTo())) {
                    e.resetStrength();
                }
            }
            outE = new ArrayList<SAEdge>(node.getOutEdges());
            for (SAEdge e : outE) {
                if (this.equals(e.getTo())) {
                    e.resetStrength();
                }
            }
        }
        else {
//        if (getOutDegree() < maxDegree && node.getOutDegree() < maxDegree) {
            SAEdge edge = new SAEdge(this, node, Color.RED);
            addOutEdge(edge);
            node.addInEdge(edge);            
            SAEdge otherEdge = new SAEdge(node, this, Color.RED);
            node.addOutEdge(otherEdge);
            addInEdge(otherEdge);
        }
    }
    
    /**
     * Makes an edge to the specified node and from the specifed node to
     * this saNode if both nodes do not already have edges to each other and
     * if adding the edge keeps their degrees less than maxDegree. If an edge allready
     * exitsts, its strength is refreshed.
     * 
     * @param node
     * @param maxDegree
     */
    public void makeEdgeToFrom(DefaultNode node, Color color) {
        if (hasEdgeTo(node)){
            ArrayList<SAEdge> outE = new ArrayList<SAEdge>(getOutEdges());
            for (SAEdge e : outE) {
                if (node.equals(e.getTo())) {
                    e.resetStrength();
                }
            }
            outE = new ArrayList<SAEdge>(node.getOutEdges());
            for (SAEdge e : outE) {
                if (this.equals(e.getTo())) {
                    e.resetStrength();
                }
            }
        }
        else {
//        if (getOutDegree() < maxDegree && node.getOutDegree() < maxDegree) {
            SAEdge edge = new SAEdge(this, node, color);
            addOutEdge(edge);
            node.addInEdge(edge);            
            SAEdge otherEdge = new SAEdge(node, this, color);
            node.addOutEdge(otherEdge);
            addInEdge(otherEdge);
        }
    }
    

    /**
     * Decays the strengh of all edges out of this node by subtracting 1 from the strength of the edges.
     * Edges with strength of 0 are removed.
     * 
     */
    public void decayEdges() {
        ArrayList<SAEdge> outE = new ArrayList<SAEdge>(this.getOutEdges());
        for (SAEdge e : outE) {
            e.decay();
            if (e.getStrength() <= 0) {
                this.removeOutEdge(e);
            }
        }
    }

    /**
     * Resets the strengh of all edges out of this node.
     * 
     */
    public void resetEdgeStrenghts() {
        ArrayList<SAEdge> outE = new ArrayList<SAEdge>(this.getOutEdges());
        outE.addAll(this.getInEdges());
        for (SAEdge e : outE) {
            e.resetStrength();
        }
    }

    public void resetToEdgeStrenght(Node n) {
        ArrayList<SAEdge> outE = new ArrayList<SAEdge>(this.getOutEdges());
        for (SAEdge e : outE) {
            if (n.equals(e.getTo())){
                e.resetStrength();
            }
        }
    }

    /**
     * @return Returns the maxDegree.
     */
    public int getMaxDegree() {
        return maxDegree;
    }

    /**
     * @param maxDegree The maxDegree to set.
     */
    public void setMaxDegree(int maxDegree) {
        this.maxDegree = maxDegree;
    }
}