/*$$
 * Copyright (c) 1999, Trustees of the University of Chicago
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with 
 * or without modification, are permitted provided that the following 
 * conditions are met:
 *
 *	 Redistributions of source code must retain the above copyright notice,
 *	 this list of conditions and the following disclaimer.
 *
 *	 Redistributions in binary form must reproduce the above copyright notice,
 *	 this list of conditions and the following disclaimer in the documentation
 *	 and/or other materials provided with the distribution.
 *
 * Neither the name of the University of Chicago nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE TRUSTEES OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *$$*/
package gui;

import java.awt.Color;

import uchicago.src.sim.gui.DrawableEdge;
import uchicago.src.sim.gui.SimGraphics;
import uchicago.src.sim.network.DefaultEdge;
import uchicago.src.sim.network.Node;

/**
 * The edge between JinGirNewNodes.
 *
 * @version $Revision: 1.1 $ $Date: 2006/03/23 16:43:50 $
 */
public class SAEdge extends DefaultEdge implements DrawableEdge {

  private double initStrength;
  private Color color = Color.BLACK;
    
  public SAEdge() {}

  public void resetStrength()
  {
      setStrength(initStrength);
  }
  
  public SAEdge(Node from, Node to, double strength) {
    super(from, to, "");
    initStrength = strength;
    resetStrength();
  }
  
  public SAEdge(Node from, Node to) {
	    super(from, to);	    
  }
  
  public SAEdge(Node from, Node to, Color _color) {
	    super(from, to);	    
	    this.color = _color;
  }

  public void decay() {
    if (strength>=1) strength -=1;
  }
  
  
  public void draw(SimGraphics g, int fromX, int toX, int fromY, int toY) {
        if (strength >= initStrength / 3 * 2)
            g.drawDirectedLink(color, fromX, toX, fromY, toY);
        else if (strength >= initStrength / 3)
            g.drawDirectedLink(color, fromX, toX, fromY, toY);
        else
            g.drawDirectedLink(color, fromX, toX, fromY, toY);
    }

/**
 * @param initStrength The initStrength to set.
 */
public void setInitStrength(double initStrength) {
    this.initStrength = initStrength;
}
}
