package org.cfpm.ruth.modifiedCloud;

/**
 * Title: butternutCrop Implemented as part of a strategy for dealing
 * with crop variation
 * Description: Subclass of the generic crop class that sets
 * values appropriate to butternut in Mangondi village, Limpopo provice
 * South Africa. Butternut is traditionally grown as a Christmas crop
 * for sale purposes
 * Copyright: Copyright (c) 2003
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */


public class ButternutCrop extends Crop{
  public ButternutCrop() {
    //first set the default generic crop properties
    super();
    //now the specifics
    name=Crop.BUTTERNUT;
    //butternut (default-September)
    //apply the butternut crop coeeficients to cropWat.
    cropModel.setCropCoefficients(0.55f,0.75f,0.9f); //Sept-Dec
    cropMonth=FarmingWorld.SEPTEMBER;
    cost = 10;
  }
  
  //#### add convenience constructor
  public ButternutCrop(float area, int cropMonth, boolean plantThisYear){
	  this();
	  this.area = area;
	  this.cropMonth = cropMonth;
	  this.plantThisYear = plantThisYear;
  }  
  
  
  /**
 * This function calculates the yield given the Wrsi that comes
 * from cropWat - cropWat itself doesn't calculate it -
 * the numbers below are specfically for typical yields in
 * Mangondi for butternut. Yields are in kilograms per line
 * planted, and must be scaled for the crop area (this done in
 * the harvest method)
 * Data shows 18.8 tonnes/ha from FAOSTAT Database Results for 2000-2002
 * thus 4.7 t for 0.25 ha   
 * @param Wrsi wrsi value
 * @return yield in kg for area planted
 */
  protected float getYield(int Wrsi) {
     float normalizedYield=0;
     switch (Wrsi) {
       case 1:
         normalizedYield = 35*lines;//30 lines 1050kg/plot
         break;
       case 2:
         normalizedYield = 52*lines;//1560
         break;
       case 3:
         normalizedYield = 69*lines;//2070
         break;
       case 4:
         normalizedYield = 85*lines;//2550
         break;
       case 5:
         normalizedYield = 104*lines;//3120
         break;
       case 6:
         normalizedYield = 120*lines;//3600
         break;
    /*   case 6: normalizedYield = 139*lines;//4170  break;*/
       default:
        normalizedYield = 1;
       break;
     }
     return normalizedYield;
   }

}
