package org.cfpm.ruth.modifiedCloud;

/**
 * Title: cabbageCrop Implemented as part of a strategy for dealing
 * with crop variation
 * Description: Subclass of the generic crop class that sets
 * values appropriate to cabbage in mangondi village, limpopo provice
 * South Africa. Cabbage is grown as crop for sale at market or
 * at the field
 * Copyright: Copyright (c) 2003
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */

public class CabbageCrop extends Crop{
  public CabbageCrop() {
    //first set the default generic crop properties
    super();
    //now specifics
    name=Crop.CABBAGE;
    //cabbage (default-January)
    //apply the cabbage crop coeeficients to cropWat.
    cropModel.setCropCoefficients(0.712f,1.003f,1.035f); //Jan-March
    cropMonth=FarmingWorld.JANUARY;
    cost = 6;
  }
  
  //#### add convenience constructor
  public CabbageCrop(float area, int cropMonth, boolean plantThisYear){
	  this();
	  this.area = area;
	  this.cropMonth = cropMonth;
	  this.plantThisYear = plantThisYear;
  }
  
  
  /**
   * This function calculates the yield given the Wrsi that comes
   * from cropWat - cropWat itself doesn't calculate it
   * the numbers below are specfically for typical yields in
   * Mangondi for cabbage. Yields are in kilograms per line
   * planted, and must be scaled for the crop area (this done in
   * the harvest method).
   * Data shows  xx tonnes/ha from FAOSTAT Database Results for 2000-2002
   * thus xx t/ha for 0.25 ha
   * @param Wrsi wrsi value
   * @return yield in kg for area planted
   */

  protected float getYield(int Wrsi) {
      float normalizedYield=0;
      switch (Wrsi) {
        case 1:
         normalizedYield = 11*lines;//330
         break;
       case 2:
         normalizedYield = 45*lines;//1350
         break;
       case 3:
         normalizedYield = 75*lines;//2250
         break;
       case 4:
         normalizedYield = 110*lines;//3300
         break;
       case 5:
         normalizedYield = 140*lines;//4200
         break;
       case 6:
         normalizedYield = 170*lines;//5100
         break;
       /*case 6: normalizedYield = 201*lines;//6030   break;*/
       default:
         normalizedYield = 1;
         break;
      }
      return normalizedYield;
  }
}
