package org.cfpm.ruth.modifiedCloud;

import java.io.*;

/**
 * Title: Crop class
 * Description:
 * Creates a crop with planting month, which can be varied, yield which starts at 0,
 * area and density, where the default is 1, boolean values for planted,
 * cropredy and ruined,
 * all set to false, seasonSincePlanted set to -1 an ripenTime set to 3.
 * The crop is ready to harvest after ripenTime months, when cropredy will
 * be set to true. planted is true if the crop is in the field, plantThis Year
 * is true if the farmer is planning to plant the crop
 * cropWat is used as a separate crop model. The is class is abstract and
 * cannot be instantiated - subclasses set the correct set of coefficients
 * for each particular crop in cropWat.
 *
 * Copyright: Copyright (c) 2004
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */
public abstract class Crop {
  protected cropWat cropModel;
  public boolean high=false,plantThisYear=false;
  public float price, cost;
  public int seasonLength = 3;
  public int cropMonth, name;
  public float yield, area, density;
  public boolean planted, cropredy, ruined;
  public int monthsSincePlanted;
  public int ripenTime;
  /** number of lines in a plot **/
  public int lines=30;
  public static final int MAIZE = 0;
  public static final int BUTTERNUT = 1;
  public static final int CABBAGE = 2;
  
  //#### added by Ruth
  public static final String[] cropNames = new String[] {"maize", "butternut", "cabbage"};
  
  public static String getCropName(int crop){
	  if (crop >= MAIZE || crop <= CABBAGE) return cropNames[crop];
	  return "null";
  }


  public Crop(int planting_month, int crop_name) {
    name = crop_name;
    cropMonth = planting_month;
    init();
  }

  private void init() {
    cost=-1;
    price=-1;
    yield = 0;
    area = 0/*1*/;    // should be 0 not 1 at the beginning (nothing planted yet) ########
    density = 1;
    planted = false;
    cropredy = false;
    ruined = false;
    monthsSincePlanted = -1;
    plantThisYear=false;
    ripenTime = 3; //  originally 2 * seasonLength. Does may cuase problems for the second crop of
    //the year (if overlapping with the first).
    cropModel=new cropWat();
  }

  public Crop() {
    init();
  }

  /**
   * Calculates crop yield and resets the crop so that it is no longer
   * planted
   * @return yield
   */
  public float harvest() {
    if (yield < 0) {
      yield = -2;
    }
    if (!ripe()) {
      yield = -1;
    }
    int Wrsi = (int) cropModel.getWrsi();

    if ( (Wrsi >= 0) && (Wrsi <= 1)) {
      Wrsi = 1;
    }
	 //####
	 System.out.println("WRSI = " + Wrsi);
	 
    yield = getYield(Wrsi)* getArea();
    setArea(0);
    planted=false;
    cropredy=false;
    monthsSincePlanted=-1;
    ruined=false;
    plantThisYear=false;
	 	 
    return yield;
  }
  /**
   * This function has to be implemented by specific
   * subclasses to get the appropriate yield of each crop.
   * @param Wrsi int
   * @return float
   */
  protected abstract float getYield(int Wrsi);

  /** Called from field class by an agent to plant either maize, butternut or cabbage.
  * globalTime is needed to get to the right point in the precipitation time series
  * @param  globalTime the number of months since the start of the model run*/
  public void plant(int globalTime) {
    monthsSincePlanted = 0;
    planted = true;
    cropredy = false;
    ruined = false;
    cropModel.initialize(globalTime);
 }

  /** Called from world class to update monthsSincePlanted,
   *  cropredy and ruined variables of
   *  each agent in the agent list for each seasonal timestep.
   */
  public void update() {
    if (planted){
      monthsSincePlanted++;
    }
    if ( (monthsSincePlanted != 0) && (monthsSincePlanted < 3)) { //added(SB)
      float stressIndex = cropModel.update(monthsSincePlanted);
    }
    // At some point we should add a strategy to change crops if the
    // water stress seems too bad
    // if  (stressIndex < field.checkCrop())
    // changeCrop()// add strategy change

    if (monthsSincePlanted == ripenTime) {
      cropredy = true; // indicates whether crop is ripe and ready for harvest.
    }
    if (monthsSincePlanted > ripenTime + ripenTime / 10) {
      ruined = true;
    }
  }



  /**
   * Sets the price of the crop depending on where it is sold (market or garden which depends on the type of agent)
   * and climate conditions.
   * @param p sets the price for the crop.
   */
  public void setPrice(float p){
    price = p;
  }

  /**
   * Returns the price of the crop depending on where it is sold
   * (market or garden which depends on the type of agent) and climate conditions.
   * @return price of crop
   */
  public float getPrice(){
    return price;
  }

  public void setCost(float c){
    cost = c;
  }


  public float getCost(){
    return cost;
  }
  public float getWrsi(){
    return cropModel.getWrsi();
  }
  /**Returns boolean indicating whether the crop is ready for harvesting.
   * @return boolean indicating whether the crop is ready for harvesting.*/
  public boolean ripe() {
    return cropredy;
  }

  /** Set area of the crop to plant depending on decision-making of agents and
   *  risk-reducing behaviour.
   * @param a area of the crop to plant depending on decision-making of agents and
   *  risk-reducing behaviour.
   */
  public void setArea(float a) {
    area = a;
  }

  /** Returns area to be planted of the crop.
   * @return area to be planted of the crop.
   */
  public float getArea() {
    return area;
  }

  /**Set density of seeds to plant depending on decision-making of agents and
   * risk-reducing behaviour.
   * @param d  density of seeds to plant depending on decision-making of agents and
   * risk-reducing behaviour.
   */
  public void setDensity(float d) {
    density = d;
  }

  /** Returns density of the seeds to be planted.
   * @return density of the seeds to be planted.
   */
  public float getDensity() {
    return density;
  }
}

/**
*  If WRSI during the season is less that what is required as strategy choice is required...
*  either changeCrop, change timing (delay) - this should come in depending on the forecast
*  and not here really - ANTICIPATORY
*  OR change area (if both crops being grown decrease area) - this would also occur with respect
*  to the forecast - ANTICIPATORY
*  OR change crop (if growing one market crop, choose other) - this would happen when conditions
*  are unexpected and favour the alternative option - REACTIONARY
*/
/**
 * Adaptation options to be included are:
 * 1.Fertilizer x
 * 2.Density (thinning) x
 * 3.Irrigation x
 * 4.Drought resistant varieties (Changing crops)
 * 5.Changing planting dates (Changing crops)
 * 6.Increasing area under cultivation
 */

