package org.cfpm.ruth.modifiedCloud;

import java.util.*;
import java.lang.Math;
import java.awt.*;

import uchicago.src.sim.gui.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * Title: The Household class
 * Description: This does all the main work of the
 * current model. Here it is decided what crops to plant and how much,
 * what to sell, what to buy and how much of each
 * Copyright:Copyright (c) 2004
 * Company: University of Cambridge/SEI
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */
public class FarmingHousehold extends Agent implements Drawable  {
  private boolean negative_perception = false;
  private boolean positive_perception = false;
  private boolean perfect_perception = false; //for Experiment A - what will be the response
  private Field fields;
  private int forecast, weather;
  private boolean poor;
  public boolean checkLastYear;
  private ArrayList yield=new ArrayList();
  private ArrayList goodsList=new ArrayList();
  private ArrayList cropList=new ArrayList();
   
  private cropRule chooseCrop;
  private communicateRules communicator;
  private Crop cabbage,maize,butternut;
    FarmingWorld world;
  private boolean canMarket = true;
  public foodStore grainStore;
  private int id, x, y, /*members,*/ numberOfFields, availableEffort, spent, labour,
      maxEffort, perceived_forecast;
  private int interactionMonth=FarmingWorld.FEBRUARY;
  private boolean plantingTime, usingForecast, forecastSwitch,
      harvestTime, seasonEnd, midSeason;
  private boolean buying, selling, marketing, marketInitial;
  private float  trust, inc, dinc, tempinc, wealth;
  private float forecastYield, cost;
  private float deficit = 0, foodNeeded, req, density, pricePerTon;
  public int spring = 0;
  public int seasonLength = 3;
  public int numSeasons = 4;
  private Goods goods;
  private float amount, reserve, /*cashInHand,*/ excess,amount_needed;
  private Market marketPlace;
  public int climate_years;
  public float to_sell;
  
  //####
  private double cash;  //replaces cashInHand
  private ArrayList plannedToPlant = new ArrayList();
  private int numMembers;
  private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
  private boolean plantingPlanned = false;  // flag needed to stop Jess from firing rules more than once per step


  															 //### 
  public FarmingHousehold(FarmingWorld world, Market market, int numMembers) {
    this.world = world;
    this.marketPlace = market;
	 this.numMembers = numMembers;  //#### new: variable size of household
    inc=1.f/3;//default three years to trust forecast
    dinc=1.f/3;//but equally likely to lower trust in bad cases
    fields = new Field(this);
    amount = 0;
    foodNeeded=10;//default 10Kg per person per month?
//    reserve = foodNeeded*world.numPeople*3;//3 months reserve to cover harvesting of maize crop = 180kg
	 //###
    reserve = foodNeeded*numMembers*3;//3 months reserve to cover harvesting of maize crop = 180kg
	 											// ###better to have 6 months?	
    goods = new Goods(this, 0, 0, 0, 0);
    grainStore=new foodStore(this);
    //we currently only know about three crops
    maize=new MaizeCrop();
    butternut=new ButternutCrop();
    cabbage=new CabbageCrop();
    cropList.add(maize);
    cropList.add(butternut);
    cropList.add(cabbage);
    //by default assume we are average - set to a poor household
    //using setPoor below - no longer doing this - using percentages in World class
    //chooseCrop=new averageCropRule(maize,butternut,cabbage);
    //poor=false;
    communicator=new communicateRules(this);
  }


  /** The method called on each household in householdList by world class,
    * each timestep to metabolize, perceive, think, act and if there is interaction between the
    * households, to communicate.
    */

  public void step() {
    // output();
    //go through order - is there enough food in the first year to eat or money
    // to buy? moved setupmarket()
    updateFields();
	 //#### changed order: we need to harvest/update market first before eating
	 //#### --> incorporated metabolize() into act()
    //metabolize();
//	 System.out.println("-------------- HH" + getID());
    perceive();
    think();
    act();
    if (world.interaction) {
      //takes place in the months when nothing much else is happening!
      communicate();
    }
//   output();
	 
  }
  public void updateFields() {
     this.fields.update();
   }

  /** Information output regarding individual agents.
   * largely for debugging purposes
  */
  public void output() {
//    if ( (world.globalTime + 1) % 12 == 0) //end of the year output
//      System.out.println("Id: " + id + " Members: " + world.numPeople + " Year: " +
//                         world.globalTime / numSeasons / seasonLength +
//                         " GlobalTime: " + world.globalTime + " Inc: " +
//                         inc + "\n" + " DInc: " + dinc + " Trust:  " + trust +
//                         " Total: " + this.getWealth() + "\n" + " Spent: " +
//                         spent + " UsingForecast: " +
//                         usingForecast + " canMarket: " + canMarket);
	  //#########
//	  System.out.println("Household " + id + ": deficit = " + this.deficit);
//	  System.out.println("-- " + this.grainStore.toString());
	  
  }

  /** Update the household's wealth and resources.
   */
  public void metabolize() {
    //first check if anything sent to market has been
    //bought or sold. We buy maize to eat, and sell other
    //crops at present
    float inputFood=0;
    ArrayList doneDeals=new ArrayList();//completed market transactions
    for (Iterator iter = goodsList.iterator(); iter.hasNext(); ) {
      Goods item = (Goods)iter.next();
      if (item.name==Crop.MAIZE){
        if (item.bought!=0){
	      removeCash(item.bought * item.price);  //###
          inputFood += item.bought;
          doneDeals.add(item);
        }
      }
        if (item.sold != 0) {
          addCash(item.cash); //add the new cash to our holdings  //####
          //did it all get sold? if so get rid of the list item
          if (item.sold == item.quantity) {
            doneDeals.add(item);
          }
          //otherwise try to sell it next time - set the
          //amount sold and cash gained to zero
          else {
            item.cash = 0;
            item.sold = 0;
          }
        }
      }

    //clean out transactions that are finished
    goodsList.removeAll(doneDeals);
    doneDeals.clear();
    //now we know what we've bought, we can eat!
	 
	 //####
//	 System.out.println("Household " + id + " bought maize: " + inputFood);
	 
    //inputfood gets added to storage, as do crops harvested each month
    float consumption = (float) /*world.numPeople*/ numMembers * foodNeeded;
    grainStore.eat(inputFood, consumption);

  }

  /** Interact with the environment: possibly query other agents. Check whether it is harvestTime.
   * Check whether it is spring (plantingTime), and if so get the forecast and the weather.
   *  need to include here what agents percieve to be expcected rainfall given the forecast.
   *  Rainfall they expect may be different to that which is actually forecasted. Furthermore,
   *  the forecast may be wrong. How much their perceptions of the expected climate vary with
   *  the forecast will be a function of their memory of past climate and past climate events
   *  within this memory. i.e. if past rainfall in last three years has been wet, and forecast
   *  was normal, normal in fourth year will be considered wet and AN will be flood.
   *  if past rainfall in last three years has been wet, and forecast was dry, forecast
   *   might not be trusted.
   *  i.e. WILL SOME FORECASTS BE VIEWED AS WRONG, OR JUST MISINTERPRETED BY THE FARMERS????
   */
  public void perceive() {
    // adding new crop if old crop has been removed?? (CHANGING CROP) - REACTIONARY

//    if ( (world.month >= World.SEPTEMBER) || (world.month == World.JANUARY)) {	// ### it says February everywhere else!
	 if ( (world.month >= FarmingWorld.SEPTEMBER) || (world.month <= FarmingWorld.FEBRUARY)) {	
     plantingTime = true;
    }
    else{								// set planting time false again ###
		 plantingTime = false;
    }
    if (world.month == FarmingWorld.forecastIssueMonth) {
      //get the forecast information
      setForecast(world.forecast.getRaintype());
      setWeather(world.worldClimate.currentWeather.getRaintype());
    }
    //change the perception of the climate and/or the forecast
    //depending on the climate history - the climate
    //history should really be personal to the agent
    if (climate_years != 0) { //Experiments B and D (possibility of longer-term memory)
      if (world.year>climate_years){
          if (world.worldClimate.calcClimateMemory(world.year, climate_years)) { //if expecting dry...
        setForecast(getForecast() + Weather.DRY); //Farmer thinks it will be drier than the forecast predicts
      }
      else {
        setForecast(getForecast() + Weather.WET); //Farmer thinks it will be wetter than the forecast predicts
      }
      }

    }
  }

  /** Perform assessment on internal and environmental data possibly query other agents.
   *  Work out trust levels, decide on crops given trust and forecast. More likely to use
   *  good forecasts rather than bad....
   *  0.8-HIGHEST PROBABILITY OF USING FORECAST IN DRY CONDITIONS.
   *  1.0-HIGHEST PROBABILITY OF USING FORECAST IN WET CONDITIONS
   *  Forecast is only used when trust level increases beyond
   *  3 units and after 3 years.
   *  Current extension to the model also means that trust is now dependent on memory of past
   *  climate and is anchored by past climate events.
   *
   * Anticipatory action here to change timing of planting or to change area planted.
   * Changing planting date only happens from september since this is when the forecast is given.
   * If there is positive or negative perception, there is a lack of correct alignment
   * with the forecast given.
   * Dependent on experiences of floods or droughts?
   * Introduces lack of alignment with the forecast.
   * Strategy to change planting date chosen here depending on how forecast information is perceived.
   */

  public void think() {
//    to_sell=0;
//    if (plantingTime) {
//		 //###
//		 System.out.println("Household " + getID() + ": it's planting time --> calculate planting");
//		 //### adapted to changed interface of calculatePlanting()
//      this.plannedToPlant=chooseCrop.calculatePlanting(world.month,usingForecast,forecast,
//                                           cashInHand,checkLastYear,world.worldClimate,world.year);
//      setCost(chooseCrop.getCost());
//    }
//#### moved to act()	 
//    to_sell = setUpMarket(); // so that excess to sell is set up for farmers,
	 
    //work out trust levels,decide on crops given trust and forecast
    //do this just once a year after we get the new forecast
    if (world.month==FarmingWorld.forecastIssueMonth) {
      setForecastUse();
      //completely switch off use of the forecast
      if (!forecastSwitch) {
        usingForecast = false;
      }
      setTrust();
    }
  }


  /** Perform actions dependent on current state - plantingTime or harvestTime. If planting,
   * plant and deduct cost. If harvesting, get yield and put into storage.
   */
  public void act() {
	  //### changed order of actions:
	  // 1. harvest (if possible) and set up market transactions
	  // 2. eat
	  // 3. plant (if planned to do so)
	  

	//### harvest extracted into method harvest()
	  harvest();

	  //### update market transactions and eat
	  metabolize();
	  
	  
	//#### moved call to setUpMarket() from think() to here
   to_sell = setUpMarket(); // so that excess to sell is set up for farmers,
   //if poor put excess maize in market to sell
   if ((to_sell > 0)&&(this.poor)) {
          Goods item = new Goods(this,to_sell,0,0,Crop.MAIZE);
          //new item for excess need to be created, but variables should be set as in field.harvest().
          marketPlace.sell(item);
          selling = true;
          //add the items on sale to the list of things at the market
          goodsList.add(item);
   }
   //Set up buying: only buy maize to be put into store for later
   //(in priciple speculators could buy other crops and
   //store those for later sale - here we could only speculate
   //in the maize market). Buy as much of amount as cashInHand
   //will allow
   if (buying) {
     goods = new Goods(this, amount_needed, 0, this.cash, Crop.MAIZE);
     marketPlace.buy(goods);
     goodsList.add(goods);
   }
//     System.out.println("cashInHand: "+this.cash);	 


	  //### now plant anything
   if (plantingTime && plannedToPlant.size() > 0) // changed condition
	{//currently two planting seasons per year, between
	     //September and February
	     //run through the list of crops and plant those which
	     //are required this month
		  // ### can change now: only those which are required this month are in plannedToPlant!
	     fields.plant(/*cropList*/plannedToPlant,world.month,world.globalTime);
	     // money remaining after cost of crops planted has been accounted for.
	     //cost is set earlier by the call to the planting calculation
	     removeCash(getCost());  //###
		 plannedToPlant.clear(); //###
	   }
	

 }
  
  //#####
  // extracted from act()
  void harvest(){
	  //Harvest returns a list of goods that can be sold at Market,
	   //or else put into store. Calling harvest ensures that any ripe
	   //crops are removed tidily from the field
		 //###
//		 System.out.println("Household " + getID() + ": harvest");
		 //###
	   yield=fields.harvest();
	     //now sell anything that is marketable. At the moment farmers don't
	     //sell maize even to each other, although they do buy it for food!
	     //Currently its assumed that anything that the farmers want to sell can be sold,
	     //anything they want to buy can be bought.
	     for (Iterator iter = yield.iterator(); iter.hasNext(); ) {
	       Goods item = (Goods) iter.next();
	       //add things to the market to await selling
	       //Get the maize harvest (if any) and put it into store
	       if (item.name == Crop.MAIZE) {
	         grainStore.add(item.quantity);
				//####
//				System.out.println("-- got maize: " + item.quantity);
	       }
	       else {
	         marketPlace.sell(item);
	         selling = true;
	         //add the items on sale to the list of things at the market
	         goodsList.add(item);
				//####
//				System.out.println("-- got to sell: " + Crop.getCropName(item.name) + " " + item.quantity);

	       }
	     }
		 
		 if (!yield.isEmpty()) setPlantingPlanned(false); //#### reset Jess flag

		 //tidy up the yield list for next month
	     yield.clear();
		  
  }
  
  

  /**addField is not currently correctly set up for multiple fields!!
   * @param f number of fields added depending on social class of the agent. */
  public void addField(Field f) {
    Field fields = new Field(this);
    fields = f;
  }

  /** Adjust level of trust by looking at that of ones neighbours
   * Calls pollContacts() in communicator, but it could do other stuff too... */
  public void communicate() {
    communicator.pollContacts(world.linear);
  }

  private float setUpMarket() {
      //try to anticipate food needs - we have to buy food if we can when
      //we have less food than needed for next month in store
      //and we have spare cash over that we last used to plant
      //this strategy may suffer from timing errors.  //##### it does indeed! No kidding. <_< 
      excess = 0;
      amount = grainStore.getTotal() - (float) /*getNumPeople()*/ numMembers * foodNeeded;
      //just one months food bought at a time.

//here buying does not equal true for av agents therefvore maize does not become an eligable good to buy
      if (amount < 0){// && cashInHand-cost > 0){//change to +...amount becomes -
        amount_needed = amount * -1;
        buying = true;
      }

  //remove cashinhand cond so buying is true but agent fails in market class when cashinhand=0

  else{
        buying = false;
      }
      //we can sell off excess maize over some reserve amount
        if (amount - reserve > 0) { //if there is at least one seasons maize in storage - set to 3 months
          excess = amount - reserve;
        //  System.out.println("Excess is "+excess);
          grainStore.setStore(reserve); //min amount required to survive for next 3 months
          selling = true;
        }
        else {
          selling = false;
        }
      return excess;
  }

  private void setForecastUse() { //REVERSED VALUES from pilot model
    double forecastUsageLikelihood = 0;
    //More likely to use good forecasts rather than bad - so forecast usage
    //likelihood is higher in wet weather - does this need changing
    //with dry conditions likely to produce higher priced market crops?
    if (this.getForecast() == Weather.DRY) { //DRY
      forecastUsageLikelihood = 1.00;//0.8
      //HIGHEST PROBABILITY OF USING FORECAST IN DRY CONDITIONS i.e.less likely to use a bad forecast.
    }
    else
    if (this.getForecast() == Weather.WET) { //WET
      forecastUsageLikelihood = 0.8;//1.00
      //HIGHEST PROBABILITY OF USING FORECAST IN WET CONDITIONS
    }
    if (this.getTrust() < 6) {
      if (this.getForecast() == Weather.DRY) { //DRY
        forecastUsageLikelihood = 0.95;//0.7
      }
      else
      if (this.getForecast() == Weather.WET) { //WET
        forecastUsageLikelihood = 0.7;//0.95
      }
    }
    if (this.getTrust() < 5) {
      if (this.getForecast() == Weather.DRY) { //DRY
        forecastUsageLikelihood = 0.85;//0.6
      }
      else
      if (this.getForecast() == Weather.WET) { //WET
        forecastUsageLikelihood = 0.6;//0.85
      }
    }
    if (this.getTrust() < 4) {
      if (this.getForecast() == Weather.DRY) { //DRY
        forecastUsageLikelihood = 0.75;//0.5
      }
      else
      if (this.getForecast() == Weather.WET) { //WET
        forecastUsageLikelihood = 0.5;//0.75
      }
    }
    //Forecast is only used when trust level increases beyond 3 units and after 3 years.
    if (this.getTrust() < 3) {
      forecastUsageLikelihood = 0;
    }
    usingForecast = false;

    if ( (float) Math.random() < forecastUsageLikelihood) {
      usingForecast = true;
    }
    if (this.getForecast() == Weather.WET && grainStore.getTotal()< 0.12) {
      usingForecast = false;
    }
  }

  private void setTrust() {
    if (usingForecast) {
      //increase trust if weather is as forecasted.
      //forecast use will continue if trust >3, so a value of inc
      //of 1./3 will give a useable trust from zero after 3 years
      //if the forecast is correct. (3 years seems about right from
      //Lesotho fieldwork)
      if (this.getForecast() == this.getWeather()) {
        trust += inc;
      }
      //decrease trust if forecast is wrong
      if (Math.abs(this.getForecast() - this.getWeather()) == 1) {
        trust -= dinc;
      }
      //drop back to a trust value just below three if the forecast is very bad
      //the following is a kind of "once bitten twice shy rule"
      if (Math.abs(this.getForecast() - this.getWeather()) == 2 &&
          usingForecast) {
        trust = 2 + dinc;
        //if using the forecast and we have become very hungry
        //decrease the subsequent rate of trust increase
        if (grainStore.getTotal() < -0.6 && this.getForecast() == 1) {
          inc = inc / 2;
          dinc = dinc; //#### ???? dinc = inc ? 
        }
        //never trust the forecast again!!
        if (inc <= 0.05) {
          inc = 0;
        }
      }
    }
    else {
      //increase trust if not using the forecast, but its correct
      //decrease otherwise
      if (this.getForecast() == this.getWeather()) {
        trust += inc;
      }
      if (Math.abs(this.getForecast() - this.getWeather()) >= 1) {
        trust -= dinc;
      }
    }
    //add trust change that arises from interaction with other households
    trust += getTempinc();
  }

 // Just as the model needs get and set accessor methods in order for
 // its initial parameters to be displayed and thus subject to modification,
 // an agent (or any other object) can be probed through similar get and set
 // methods.
 //
 // Probing consists of clicking on an object in the display, causing that
 // object's current state to be displayed. What is displayed depends on
 // the various get and set methods implemented by the object. For example,
 // if an object has a setX and a getX method, a X field will be displayed
 // providing the current value of the X variable and allowing the user to
 // change the value by entering a new value and pressing enter. As of this
 // release only number, String, and boolean fields can be displayed. Of course
 // a user can use the get and set methods to turn a Vector, for example, into
 // a String or whatever is appropriate.

 //accesoor methods for the households wealth class
 //note that changing the poor value alters the rules used
 //to decide on crop planting
 public boolean getPoor() {
    return poor;
  }

  public void setPoor(boolean isPoor) {
	  this.poor = isPoor;  //#####
//    if (isPoor){
//      poor = true;
//      chooseCrop = new poorCropRule(/*maize, butternut, cabbage*/);
//    }else{
//      poor = false;
//      chooseCrop = new averageCropRule(/*maize, butternut, cabbage*/);
//    }
  }

  /**
 * @return number of years of climate memory
 */
public int getClimateMemory() {
  return climate_years;
}

public void setClimateMemory(int y) {
  climate_years = y;
}

  /**
   * set the initial store of food			// ### nie benutzt
   * @param s float
   */
  public void setStore(float s) {
    grainStore.setStore(s);
  }

  /** @return total amount of grain in storage
   * which is regarded as part of the 'wealth' of the household.*/
  public float getStore() {
    return grainStore.getTotal();
  }

   /** @return the cost incurred by this agent, which is a value that is offset against the yield
    * relating to the cost of getting some of the seed from the field for the next years planting.
    */
   public float getCost() {
     return cost;
   }

   /** @return wealth of this agent which is
    * cash in hand plus the total store of food converted to
    * currency (at a nominal rate of 2rand/Kg)
    */
   public double getWealth() {
//     return cashInHand;
//
     return grainStore.getTotal()*2.f + this.cash;
   }
	
	
   public double getCash() {  //### changed to Java conventions for Jess
     return this.cash;
   }

   //sets of parameters related to the forecast and the weather
   public void setTrust(float t) {
      trust = t;
    }
    /** @return current trust level of this household in the forecast.*/
    public float getTrust() {
      return trust;
    }
    public boolean getUsingForecast() {
      return usingForecast;
    }
    //these parameters set how rapidly forecast trust increases or decreases
    public void setInc(float i) {
      inc = i;
    }
    public float getInc() {
      return inc;
    }
    public void setDinc(float t) {
      dinc = t;
    }

    public float getDinc() {
      return dinc;
    }

    public void setForecast(int f) {
      forecast = f;
      setPerceivedForecast(forecast);
      //limit range of forecast to available values
      forecast=Weather.checkValues(forecast);
    }

    /** @return forecasted raintype. */
    public int getForecast() {
      return forecast;
    }
    public void setPerceivedForecast(int f) {
      perceived_forecast=f;
    }
    public int getPerceivedForecast() {
      return perceived_forecast;
    }

    public void setWeather(int s) {
      weather = s;
    }

    /** @return actual weather raintype. */
    public int getWeather() {
      return weather;
    }


    public void setTempinc(float t) {
      tempinc = t;
    }

    /** @return actual weather raintype. */
    public boolean getCheckLastYear() {
      return checkLastYear;
    }


    public void setCheckLastYear(boolean c) {
      checkLastYear = c;
    }

    /** @return the temporary increase in trust of this agent,
     * if there is interaction between households. */
    public float getTempinc() {
      return tempinc;
    }
    /** @return cumulative yield deficit.*/
    public float getDeficit() {
      return deficit;
    }

  public void setXY(int x, int y) {
    this.x = x;
    this.y = y;
  }

  public int getX() {
    return x;
  }

  public int getY() {
    return y;
  }

  public int getID() {
    return id;
  }

  public void setID(int i) {
    id = i;
  }

//  /** @return number of people.*/
//  public int getNumPeople() {
//    return members;
//  }
//
//  public void setNumPeople(int p) {
//    members = p;
//  }

    // define color constants
 final Color COLOR[] = {Color.gray, Color.pink, Color.green, Color.blue};

   public void draw(SimGraphics g) {
     // This method is required to plot the actor on the display.
     // Here the size, shape, and color are specified.
     int type = 0;// gray poor and buying
     if (this.getPoor()&&selling){ //if poor farmer and selling
       type=1; //pink
     }
     if (!this.getPoor()){ //if average farmer and buying
       type=2; //green
     }
     if (!this.getPoor()&&selling){ //if average farmer and selling
       type=3; //blue
     }
     g.setDrawingParameters(28, 28, 28);
     g.drawFastRoundRect(COLOR[type]);
   }

  /**
   * addToDeficit
   * called from the foodStore if we run out of food
   * @param d double
   */
  public void addToDeficit(double d) {
//    deficit+=d;
	  setDeficit(deficit + (float)d);  //###
  }

  /**
   * setforecastSwitch
   * @param s boolean
   */
  public void setforecastSwitch(boolean s) {
    forecastSwitch=s;
  }

  /**
   * setfoodNeeded
   * @param f float
   */
  public void setfoodNeeded(float f) {
    foodNeeded=f;
  }

  public ArrayList getCrops() {
    return fields.cropsPlanted;
  }
  
  //#####
  public int getNumMembers(){
	  return this.numMembers;
  }
  
  public void addMember(){
	  //this.numMembers++;
	 addMembers(1);
  }
  
  public void addMembers(int n){
//	  this.numMembers += n;
	  setNumMembers(numMembers + n);
  }
  
  public void removeMember(){
//	  this.numMembers--;
	  removeMembers(1);
  }
  
  public void removeMembers(int n){
//	  this.numMembers -= n;
	  setNumMembers(numMembers - n);
  }
  
  public void dissolve() {
	  this.world.householdDeath(this);
  }
  
//  public void addCashInHand(double cash) {
//	  System.out.println("%%%%%%%%%%%%%% added cash: " + cash + " %%%%%%%%%%%%%%%%%%%");
//	   this.cashInHand += cash;
// }
// 
// public void deductCashInHand(double cash) {
//	  System.out.println("%%%%%%%%%%%%%% removed cash: " + cash + " %%%%%%%%%%%%%%%%%%%");
//	  this.cashInHand -= cash;
// }

 //### changes for Jess
 public void planToPlant(Crop[] crops){
	  this.plannedToPlant = new ArrayList();
	  System.out.print("Household " + this.id + " decided to plant ");
	  for (int i=0; i<crops.length; i++){
		 this.plannedToPlant.add(crops[i]); 
		 System.out.print(crops[i].area + " " + Crop.getCropName(crops[i].name) + " ");
	  }
	  System.out.println("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
 }
 
 public void setUsingForecast(boolean u) {
	  boolean old = this.usingForecast;
	  this.usingForecast = u;
	  pcs.firePropertyChange("usingForecast", old, this.usingForecast);
 }
 
 public void setCash(double w) { //### changed to Java conventions for Jess
	  double old = this.cash;
	  this.cash = w;
	  pcs.firePropertyChange("cash", old, this.cash);
  } 
 
 public void removeCash(double c){
	  System.out.println("%%%%%%%%%%%%%% removed cash: " + c + " %%%%%%%%%%%%%%%%%%%");
	  setCash(this.cash - c);
 }
 
 public void addCash(double c){
	  System.out.println("%%%%%%%%%%%%%% added cash: " + c + " %%%%%%%%%%%%%%%%%%%");
	  setCash(this.cash + c);
 }
 
 public void setCost(float c) {
	  float old = this.cost;
	  this.cost = c;
	  pcs.firePropertyChange("cost", old, this.cost);
 }

 public void setCost(int c) {
	  float old = this.cost;
	  this.cost = c;
	  pcs.firePropertyChange("cost", old, this.cost);
 }
 
 public void setNumMembers(int numMembers){
	 int old = this.numMembers;
	 this.numMembers = numMembers;
	 pcs.firePropertyChange("numMembers", old, this.numMembers); 
  }

 public void setDeficit(float d) {
	  float old = this.deficit;
	  this.deficit = d;
	  pcs.firePropertyChange("deficit", old, this.deficit);
 }
 
 
 public boolean getPlantingPlanned(){
	  return this.plantingPlanned;
 }
 
 public void setPlantingPlanned(boolean plantingPlanned){
	  boolean old = this.plantingPlanned;
	  this.plantingPlanned = plantingPlanned;
	  pcs.firePropertyChange("plantingPlanned", old, this.plantingPlanned);
	  
 }
 

	public void addPropertyChangeListener(PropertyChangeListener p){
		pcs.addPropertyChangeListener(p);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener p){
		pcs.removePropertyChangeListener(p);
	}
 

}
