package org.cfpm.ruth.modifiedCloud;

import java.util.Vector;
import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Collections;
import java.lang.Math;
import uchicago.src.sim.engine.*;
import uchicago.src.sim.util.SimUtilities;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.analysis.*;
import uchicago.src.reflector.BooleanPropertyDescriptor;
import uchicago.src.sim.space.Object2DGrid;
import uchicago.src.sim.gui.*;

import jess.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/** World generates household agents.*/

/** Simulation models should extend SimModelImpl which does some basic setup
 * and allows a controller easy access to a simulations initial parameters.
 */
public class FarmingWorld /*extends SimpleModel*/ {

	//####
	private Rete engine;
	private PropertyChangeSupport pcs = new PropertyChangeSupport(this);

	public Weather forecast = new Weather();
  public Climate worldClimate = new Climate();
  public GCM forecastModel = new GCM();
  public Market marketPlace= new Market();
  public cropWat cropModel=new cropWat();
  private Representation r;
  private ArrayList contacts=new ArrayList();

  /** A list of all the households. Convenient for any method that
   *  iterates through a list of households. A least one list like this is common
   *  to most simulations.
   */
  public ArrayList householdList = new ArrayList();

  /** A list of households who have been "birthed" and should be introduced to the simulation
   *  in the next turn if replace is checked.*/
  private Vector birthList = new Vector();

  /** A queue that tracks dead households.*/
  private Vector reaperQueue = new Vector();

  // initial starting parameters of the model
  public int i=1;
  private int numhouseholds = 2;
  public int agentCount = 0;
  public boolean perception=false;
  public boolean usingForecast = true;  //### changed to true
  public boolean checkLastYear=false;	 
  public int perceived_normal;
  public int met_normal=30;
  /** Social interaction can be linear or non-linear. */
  public boolean linear = false;
  /** Switch to turn on interaction between households. */
  public boolean interaction;
  /** timimg information and month names
   * The model at present has a monthly timestep
   */
  public int globalTime;
  public int month=1;
  public int year=1;
  public int time;
  public static final int SEPTEMBER=9;
  public static final int OCTOBER=10;
  public static final int NOVEMBER=11;
  public static final int DECEMBER=12;
  public static final int JANUARY=1;
  public static final int FEBRUARY=2;
  public static final int forecastIssueMonth=SEPTEMBER;
  public int spring = 0;
  public int seasonLength = 3;
  public int numSeasons = 4;
  public float foodNeeded = 10;
  public int climate_years = 0;
  /** Household size based on survey data*/
  //public int numPeople = 6; //
  //Can experiment with poor=8, average=4 (from data)
  public float startingWealth = 5000; //
  // starting wealth should be different for poor and average agents?
  //720*R2=1440  (6people*10kg*12months=720kg/year)
  // default - one years grain storage, but can be varied in control panel.
  public boolean variable_skill=false;
  /**Repast parameters*/

  public DisplaySurface dsurf;     // A RePast display surface
  Object2DGrid  world;  // RePast object defining the 2D world
  /** Every model must have a schedule. */
  private Schedule schedule;

  /** A DataRecorder allows data generated by a simulation to be written to a file.*/
  private DataRecorder recorder;

  /** A graph that plots sequence.*/
  private OpenSequenceGraph wealthGraph;
  private OpenSequenceGraph forecastGraph;
  private OpenSequenceGraph trustGraph;
  private OpenSequenceGraph normalGraph;
  private OpenSequenceGraph wrsiGraph;
  /** A histogram.*/
  private OpenHistogram wealthBar;

  // These inner classes are used by all the data collection methods - i.e.
  // DataRecorder, SequenceGraph and the Histogram.

  // Creates a DataSource and Sequence object that returns the average value for
  // all the households in the household list. It gets the individual households value
  // by calling the method get...() on the household.
    /** Creates a DataSource and Sequence object that returns the forecasted skill.*/
 /*   public class Year
        implements DataSource, Sequence {
      public Object execute() {
        return new Double(getSValue());
      }

      public double getSValue() {
        return year;
      }
    };
*/
  /** Creates a DataSource and Sequence object that returns the forecasted skill.*/
  /*public class ForecastSkill
      implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int forecast_skill=forecastModel.getError();
      return forecast_skill;
    }
  };*/

  /** Creates a DataSource and Sequence object that returns the forecasted raintype.*/
  public class ActualForecast
      implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int i=0;  //### 1;  // a non-empty list doesn't mean it contains more than 1 element!
      int f=0;
      if(!householdList.isEmpty()){
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        f = household.getForecast();
      }
      return f;
    }
  };

  /** Creates a DataSource and Sequence object that returns the actual raintype.*/
public class PerceivedForecast
    implements DataSource, Sequence {
  public Object execute() {
    return new Double(getSValue());
  }

  public double getSValue() {
    int i=0;  //### 1;  // a non-empty list doesn't mean it contains more than 1 element!
    int pf=0;
    if(!householdList.isEmpty()){
      FarmingHousehold household = (FarmingHousehold) householdList.get(i);
      pf = household.getPerceivedForecast();
    }
    return pf;
  }
};

/** Creates a DataSource and Sequence object that returns the forecasted raintype.*/
 public class ActualWeather
     implements DataSource, Sequence {
   public Object execute() {
     return new Double(getSValue());
   }

   public double getSValue() {
       return worldClimate.currentWeather.getRaintype();
   }
 };


   /** Creates a DataSource and Sequnce object that returns the average value for
    *  all the trust of all the households in the household list. It gets the individual
    *  households value
    *  by calling the method getTrust() on each household.
    */
  public class AvgPoorHouseholdTrust implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int size = 0; //size should be no of POOR households not total no of households
      double val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        if (household.getPoor()){
          val += household.getTrust();
          size++;
        }
      }
      if (size>0) return val / size; else return -1;
    }
  };

  public class AvgAvgHouseholdTrust implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
     int size = 0; //size should be no of AVERAGE households not total no of households
      double val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        if (!household.getPoor()){
          val += household.getTrust();
          size++;
        }
      }
      if (size>0)return val / size; else return -1;
    }
  };

  public class AvgPoorHouseholdWealth  implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int size = 0; //size should be no of POOR households not total no of households
      double val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        if (household.getPoor()){
          val += household.getCash();
          size++;
        }
      }
      return val / size;
    }
  };

  public class ButternutWrsi  implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int size = 0;
      float val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        ArrayList crops = household.getCrops();
        for (int j = 0; j < crops.size(); j++) {
          Crop c = (Crop) crops.get(j);
          if (c.name==c.BUTTERNUT) {
            val += c.getWrsi();
            size++;
          }
        }
      }
      return val / size;
    }
  };

  public class CabbageWrsi  implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int size = 0;
      float val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        ArrayList crops = household.getCrops();
        for (int j = 0; j < crops.size(); j++) {
          Crop c = (Crop) crops.get(j);
          if (c.name==c.CABBAGE) {
            val += c.getWrsi();
            size++;
          }
        }
      }
      return val / size;
    }
  };

  public class MaizeWrsi  implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      int size = 0; //size should be no of POOR households not total no of households
      float val = 0;
      for (int i = 0; i < householdList.size(); i++) {
        FarmingHousehold household = (FarmingHousehold) householdList.get(i);
        ArrayList crops = household.getCrops();
        for (int j = 0; j < crops.size(); j++) {
          Crop c = (Crop) crops.get(j);
          if (c.name==c.MAIZE) {
            val += c.getWrsi();
            size++;
          }
        }
      }
      return val / size;
    }
  };

 public class AvgAvHouseholdWealth  implements DataSource, Sequence {
   public Object execute() {
     return new Double(getSValue());
   }

   public double getSValue() {
     int size = 0; //size should be no of AVERAGE households not total no of households
     double val = 0;
     for (int i = 0; i < householdList.size(); i++) {
       FarmingHousehold household = (FarmingHousehold) householdList.get(i);
       if (!household.getPoor()){
         val += household.getCash();
         size++;
       }
     }
     return val / size;
   }
 };

  public class MetNormal  implements DataSource, Sequence {
   public Object execute() {
     return new Double(getSValue());
   }

   public double getSValue() {
     float val = 0;
       val=worldClimate.N[year];
     return val;
   }
  };

  public class VariableSkill  implements DataSource, Sequence {
    public Object execute() {
      return new Double(getSValue());
    }

    public double getSValue() {
      float val = 0;
        val=forecastModel.skill[year];
      return val;
    }
   };

 public class OND  implements DataSource, Sequence {
   public Object execute() {
     return new Double(getSValue());
   }

   public double getSValue() {
     float val = 0;
       val=worldClimate.OND[year];
     return val;
   }
  };

  public class Pet  implements DataSource, Sequence {
   public Object execute() {
     return new Double(getSValue());
   }

   public double getSValue() {
     float val = 0;
       val=cropModel.pet[year];
     return val;
   }
  };

// All simulations should have a no argument constructor - this is empty and
  // thus not strictly necessary
  public FarmingWorld() {
//    BooleanPropertyDescriptor bd = new BooleanPropertyDescriptor("Replacement",
//        false);
//    descriptors.put("Replacement", bd);
    // read precipitation and PET data only once
    cropWat.getPrec(); // store monthly rainfall for 1960-2000 in an array at the beginning
    cropWat.getPet();
	 //###
  householdList = new ArrayList();
  birthList = new Vector();
  reaperQueue = new Vector();

	 //#### changes for JESS
	 this.engine = new Rete();
	 
	 // helper functions
	 String weatherFunction ="(deffunction convert-raintype (?x)" +
	 		"(if (< ?x 0) then (return dry))" +
	 		"(if (> ?x 0) then (return wet))" +
	 		"(return normal)" +
	 		")";
	 
	  // define the rules
	 String weatherForecast = "(defrule weather-forecast " +
	 							"(world (month 9) (year ?y))" +
	 							"?f <- (forecast (year =(- ?y 1)))" +
	 							"=>" +
	 							"(modify ?f (year ?y) (type (convert-raintype (call (fetch world) getForecast))))" +
	 							")";
	 
	  String chooseCropSeason1Average = "(defrule choose-crop-season-1-average " +
	  										"(world (month 9))" +
	  										"?agent <- (household (plantingPlanned FALSE) (poor FALSE) (cash ?c&:(>= ?c 750)))" +
	  										"=>" +
	  										"(bind ?agent (fact-slot-value ?agent OBJECT))" +
	  										"(bind ?crop (new ButternutCrop 1.0 9 TRUE))" +
	  										"(call ?agent planToPlant (create$ ?crop))" +
	  										"(call ?agent setCost 300)" +
	  										"(call ?agent setPlantingPlanned TRUE)" + 
	  										")";
	  
	  String chooseCropSeason2Average = "(defrule choose-crop-season-2-average " +
		  								   "(world (month 1))" +
		  								   "(forecast (type dry))" +
	  										"?agent <- (household (plantingPlanned FALSE) (poor FALSE) (usingForecast TRUE) (cash ?c&:(>= ?c 630)))" +
	  										"=>" +
	  										"(bind ?agent (fact-slot-value ?agent OBJECT))" +
	  										"(bind ?crop (new CabbageCrop 1.0 1 TRUE))" +
	  										"(call ?agent planToPlant (create$ ?crop))" +
	  										"(call ?agent setCost 180)" +
	  										"(call ?agent setPlantingPlanned TRUE)" + 
	  										")";
	  
	  String[] chooseCropSeason1Poor = new String[6];
	  chooseCropSeason1Poor[0] = "(defrule choose-crop-season-1-poor-0 " +
	  									"(world (month 9))" +
	  									"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (cash ?c&:(< ?c 435)))" +
	  									"=>" +
	  									"(bind ?agent (fact-slot-value ?agent OBJECT))" +
	  									"(bind ?crop (new MaizeCrop 0.25 9 TRUE))" +
	  									"(call ?agent planToPlant (create$ ?crop))" +
	  									"(call ?agent setCost 0)" +
										"(call ?agent setPlantingPlanned TRUE)" + 
	  									")";
	  
	  chooseCropSeason1Poor[1] = "(defrule choose-crop-season-1-poor-1 " +
	  	"(world (month 9))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (usingForecast FALSE) (cash ?c&:(and (>= ?c 435) (< ?c 525))))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop (new MaizeCrop 1.0 9 TRUE))" +
		"(call ?agent planToPlant (create$ ?crop))" +
		"(call ?agent setCost 120)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";

	  chooseCropSeason1Poor[2] = "(defrule choose-crop-season-1-poor-2 " +
	  	"(world (month 9))" +
	  	"(not (forecast  (type dry)))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (usingForecast TRUE) (cash ?c&:(and (>= ?c 435) (< ?c 525))))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?a OBJECT))" +
		"(bind ?crop (new MaizeCrop 1.0 9 TRUE))" +
		"(call ?agent planToPlant (create$ ?crop))" +
		"(call ?agent setCost 120)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";	//### should be possible to combine 1 and 2; question: HOW?????
	  
	  chooseCropSeason1Poor[3] = "(defrule choose-crop-season-1-poor-3 " +
	  	"(world (month 9))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE)(cash ?c&:(>= ?c 525)))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop1 (new MaizeCrop 0.5 9 TRUE))" +
		"(bind ?crop2 (new ButternutCrop 0.5 9 TRUE))" + 
		"(call ?agent planToPlant (create$ ?crop1 ?crop2))" +
		"(call ?agent setCost 210)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
	  chooseCropSeason1Poor[4] = "(defrule choose-crop-season-1-poor-4 " +
	  	"(world (month 9))" +
	  	"(forecast (type dry))" + 
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE)(cash ?c&:(>= ?c 570)))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop1 (new MaizeCrop 0.25 9 TRUE))" +
		"(bind ?crop2 (new ButternutCrop 0.75 9 TRUE))" + 
		"(call ?agent planToPlant (create$ ?crop1 ?crop2))" +
		"(call ?agent setCost 255)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
	  chooseCropSeason1Poor[5] = "(defrule choose-crop-season-1-poor-5 " +
	  	"(world (month 10))" +
	  	"(forecast (type dry))" + 
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE)(usingForecast TRUE)(cash ?c&:(>= ?c 435)))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop (new MaizeCrop 1.0 10 TRUE))" +
		"(call ?agent planToPlant (create$ ?crop))" +
		"(call ?agent setCost 120)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
	  String[] chooseCropSeason2Poor = new String[4];
	  chooseCropSeason2Poor[0] = "(defrule choose-crop-season-2-poor-0 " +
	  									"(world (month 1))" +
	  									"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (cash ?c&:(< ?c 435)))" +
	  									"=>" +
	  									"(bind ?agent (fact-slot-value ?agent OBJECT))" +
	  									"(bind ?crop (new MaizeCrop 0.25 1 TRUE))" +
	  									"(call ?agent planToPlant (create$ ?crop))" +
	  									"(call ?agent setCost 0)" +
										"(call ?agent setPlantingPlanned TRUE)" + 
	  									")";
	  
	  chooseCropSeason2Poor[1] = "(defrule choose-crop-season-2-poor-1 " +
	  	"(world (month 1))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (cash ?c&:(>= ?c 435)))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop (new MaizeCrop 1.0 1 TRUE))" +
		"(call ?agent planToPlant (create$ ?crop))" +
		"(call ?agent setCost 120)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
	  chooseCropSeason2Poor[2] = "(defrule choose-crop-season-2-poor-2 " +
	  	"(world (month 1))" +
	  	"(forecast (type dry))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (usingForecast TRUE) (cash ?c&:(and (>= ?c 450) (< ?c 495))))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop1 (new MaizeCrop 0.75 1 TRUE))" +
		"(bind ?crop2 (new CabbageCrop 0.25 1 TRUE))" + 
		"(call ?agent planToPlant (create$ ?crop1 ?crop2))" +
		"(call ?agent setCost 130)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
	  chooseCropSeason2Poor[3] = "(defrule choose-crop-season-2-poor-3 " +
	  	"(world (month 2))" +
	  	"?agent <- (household (plantingPlanned FALSE) (poor TRUE) (cash ?c&:(>= ?c 495)))" +
		"=>" +
		"(bind ?agent (fact-slot-value ?agent OBJECT))" +
		"(bind ?crop (new CabbageCrop 1.0 1 TRUE))" +
		"(call ?agent planToPlant (create$ ?crop))" +
		"(call ?agent setCost 180)" +
			"(call ?agent setPlantingPlanned TRUE)" + 
		")";
	  
		try {
			// define templates
			engine.executeCommand("(deftemplate forecast (slot type) (slot year))");
			// define classes used in facts and rules
			engine.executeCommand("(import org.cfpm.ruth.modifiedCloud.*)");
			engine.defclass("maize", "MaizeCrop", null);
			engine.defclass("butternut", "ButternutCrop", null);
			engine.defclass("cabbage", "CabbageCrop", null);
			engine.defclass("world", "FarmingWorld", null);
			engine.defclass("household", "FarmingHousehold", null);
			// define rules
			engine.executeCommand(weatherFunction);
			engine.executeCommand(weatherForecast);
			engine.executeCommand(chooseCropSeason1Average);
			engine.executeCommand(chooseCropSeason2Average);
			for (int i = 0; i < chooseCropSeason1Poor.length; i++){
				engine.executeCommand(chooseCropSeason1Poor[i]);
			}
			for (int i = 0; i < chooseCropSeason2Poor.length; i++){
				engine.executeCommand(chooseCropSeason2Poor[i]);
			}
			// put initial facts in Jess
			engine.assertString("(forecast (type normal) (year -1))");
			engine.definstance("world", this, true);			
			// store a reference to the model in Jess
			engine.store("world", this);
			engine.watchAll();
			engine.executeCommand("(facts)");
			engine.executeCommand("(rules)");
//			engine.watch(WatchConstants.ACTIVATIONS);
		}
		catch(JessException e){
			System.out.println(e.getMessage());
			e.printStackTrace();			
		}	
 
  }

//  /** The typical way to code a simulation is to divide up the creation of the
//   *  simulation into three methods - buildModel(), buildDisplay(),
//   *  buildSchedule(). This division is not strictly necessary, but it does
//   *  make the creation conceptually clearer.
//   *  The buildModel() method is responsible for creating those parts of the
//   *  simulation that represent what is being modeled. Consequently, the households
//   *  and their environment are typically created here together with any
//   *  optional data collection objects. Of course, this method may call other
//   *  methods to help build the model. Calls addNewHousehold for number of households
//   *  specified in numhousholds, which can be varied by the user.
//   */
//  public void buildModel() {
//
//    world = new Object2DGrid(numhouseholds,numhouseholds); // First create the 2D world
//
//    for (int agentCount = 0; agentCount < numhouseholds*numhouseholds; agentCount++) {
//      addNewHousehold();
//    }
//
//    DisplayConstants.CELL_WIDTH = 35;  // These are the dimensions of each cell
//    DisplayConstants.CELL_HEIGHT = 35;
//
//    for (int x = 0;  x < numhouseholds; x++)
//       for (int y = 0; y < numhouseholds; y++) {
//         FarmingHousehold h = (FarmingHousehold) householdList.get(x*numhouseholds+y);
//         world.putObjectAt(x, y, h);  // This call puts the object where it belongs
//         h.setXY(x, y);  // The player will have to record its own position
//         h.setLocation(x, y);
//       }
//
//     // Now create the 2D-display itself
//          Object2DDisplay display = new Object2DDisplay(world);
//          display.setObjectList(householdList);  // The display will show players from this list
//          dsurf.addDisplayableProbeable(display, "Display");
//          addSimEventListener(dsurf);  // Make sure the display will be updated
//          dsurf.display();  // Get it to show right away
//
// // create a new DataRecorder and write the data to ./data.txt
//   /* recorder = new DataRecorder("./output/data.txt", this);
//
//    // adds an ObjectDataSource to the recorder as a source of data.
//    recorder.addObjectDataSource("Avg. Poor Trust", new AvgPoorHouseholdTrust());
//    recorder.addObjectDataSource("Avg. Average Trust", new AvgAvgHouseholdTrust());
//    recorder.addObjectDataSource("Avg. Poor Wealth", new AvgPoorHouseholdWealth());
//    recorder.addObjectDataSource("Avg. Beffer-Off Wealth", new AvgAvHouseholdWealth());
//    recorder.addObjectDataSource("Met Normal", new MetNormal());
//    recorder.addObjectDataSource("OND", new OND());
//    recorder.addObjectDataSource("Pet", new Pet());
//    if (variable_skill){
//      recorder.addObjectDataSource("Skill", new VariableSkill());
//    }
//    recorder.addObjectDataSource("ButternutWrsi", new ButternutWrsi());
//    recorder.addObjectDataSource("CabbageWrsi", new CabbageWrsi());
//    recorder.addObjectDataSource("MaizeWrsi", new MaizeWrsi());
//*/
//    /** Adds a data source that counts the households alive
//     * in the simulation. It does this by getting the size of the householdList.
//     * first we make an inner class that implements the DataSource interface
//     */
//    class HouseholdCountClass implements DataSource {
//      public Object execute() {
//        int i = householdList.size();
//        return new Integer(i);
//      }
//    };
//    //  recorder.addObjectDataSource("householdCount", new HouseholdCountClass());
//  }

  /** buildDisplay() builds those parts of the simulation that have to do with
   *  displaying the simulation to a user.
   */
//  private void buildDisplay() {
//    // Any display code....
//    // Adds two Sequence to be displayed by the graph, these Squences will
//    // return the households' average trust, and average wealth. For how the
//    // averages are computed see the two inner classes, AvghouseholdTrust,
//    // and AvghouseholdWealth above.
//
//   wealthGraph.addSequence("Avg. Poor Wealth", new AvgPoorHouseholdWealth());
//    wealthGraph.addSequence("Avg. Average Wealth", new AvgAvHouseholdWealth());
//
//    forecastGraph.addSequence("Actual Forecast", new ActualForecast());
//    forecastGraph.addSequence("Perceived Forecast", new PerceivedForecast());
//    forecastGraph.addSequence("Actual Weather", new ActualWeather());
//
//    trustGraph.addSequence("Avg. Poor Trust", new AvgPoorHouseholdTrust());
//    trustGraph.addSequence("Avg. Avg Trust", new AvgAvgHouseholdTrust());
//
//    normalGraph.addSequence("Met Normal", new MetNormal());
//    normalGraph.addSequence("OND", new OND());
//    normalGraph.addSequence("Pet", new Pet());
//    if (variable_skill){
//      normalGraph.addSequence("Skill", new VariableSkill());
//    }
//
//    wrsiGraph.addSequence("ButternutWrsi", new ButternutWrsi());
//    wrsiGraph.addSequence("CabbageWrsi", new CabbageWrsi());
//    wrsiGraph.addSequence("MaizeWrsi", new MaizeWrsi());
//	 
//  //  normalGraph.addSequence("Perceived Normal", new PNormal());
//
//
//    wealthGraph.setAxisTitles("Time", "Avg");
//    wealthGraph.setXRange(0, 100);
//    wealthGraph.setYRange(0, 10);
//    wealthGraph.setSize(400, 250);
//
//    forecastGraph.setAxisTitles("Time", "Forecast/Perceived");
//    forecastGraph.setXRange(0, 100);
//    forecastGraph.setYRange(0, 1);
//    forecastGraph.setSize(400, 250);
//
//    trustGraph.setAxisTitles("Time", "Trust");
//    trustGraph.setXRange(0, 100);
//    trustGraph.setYRange(0, 1);
//    trustGraph.setSize(400, 250);
//
//   normalGraph.setAxisTitles("Time", "OND, Met Normal & Rainfall");
//   normalGraph.setXRange(0, 50);
//   normalGraph.setYRange(0, 1);
//   normalGraph.setSize(400, 250);
//
//   wrsiGraph.setAxisTitles("Time", "Wrsi");
//   wrsiGraph.setXRange(0, 50);
//   wrsiGraph.setYRange(0, 1);
//   wrsiGraph.setSize(400, 250);
//
//   // Histograms get their data from histogramItems. This creates an item
//    // called wealth that gets its data by calling getWealth on each household
//    // in the household list.
//    wealthBar.createHistogramItem("Wealth", householdList, new BinDataSource() {
//      public double getBinValue(Object o) {
//        FarmingHousehold household = (FarmingHousehold) o;
//        return household.getCash();
//      }
//    });/*
//    bar2.createHistogramItem("Deficit", householdList, new BinDataSource() {
//      public double getBinValue(Object o) {
//        Household household = (Household) o;
//        return household.getDeficit();
//      }
//    });*/
//
//    wealthBar.setYRange(0, 100.0);
//    //bar2.setYRange(0, 100.0);
//    //bar.createHistogramItem("Wealth", householdList, "getWealth");
//  }

  /**
   * Contains HouseholdRunner which calls the birthhHouseholds, shuffleHouseholds
   * and calls step on each household in the householdList. BuildSchedule builds the
   * schedule that changes the simulations state.
   * Under this scheme, a simulation is a state machine where all transitions
   * between states are the result of actions initiated by a schedule.
   */
//  public void buildSchedule() {
//    // this is a static schedule (no need to add or replace actions) so
//    // we can create an inner class that extends from basic action. The
//    // execute method of the inner class will execute all the methods that
//    // we wish to schedule on the households and the environment.
//
//    /**
//     * Calls the birthhHouseholds, shuffleHouseholds method and
//     * calls the step method on each household
//     * in the householdList.*/
//    class HouseholdRunner
//        extends BasicAction {
//      public void execute() {
//        // call the birthhHouseholds methods on this model
//        //  birthHouseholds();
//        // call the shuffleHouseholds method on this model
//        shuffleHouseholds();
//        updateWorld();
//       // should call update display after the things that it displays
//       // have changed their state. Otherwise, displays won't be in
//       // synch with what it displays at end or at pause.
//       wealthGraph.step();
//       forecastGraph.step();
//       trustGraph.step();
//       normalGraph.step();
//       wrsiGraph.step();
//       i++; //for graphs
//       wealthBar.step();
//       dsurf.updateDisplay();
//   //    recorder.record();
//       reapHouseholds();
//      }
//    };
//    // the schedule has been created in setup()
//    schedule.scheduleActionBeginning(0, new HouseholdRunner());
//
//    // On a pause in the simulation run, call the writeToFile method
//    // on the recorder object. (Writes the data collected by the recorder
//    // to a file.
////    schedule.scheduleActionAtPause(recorder, "writeToFile");
//	 
//    schedule.scheduleActionAtPause(wealthGraph, "writeToFile");
//    schedule.scheduleActionAtPause(forecastGraph, "writeToFile");
//    schedule.scheduleActionAtPause(trustGraph, "writeToFile");
//    schedule.scheduleActionAtPause(normalGraph, "writeToFile");
//    schedule.scheduleActionAtPause(wrsiGraph, "writeToFile");
//    
//    // When the simulation run ends, call the writeToFile method
//    // on the recorder object. (Writes the data collected by the recorder
//    // to a file.
////    schedule.scheduleActionAtEnd(recorder, "writeToFile");
//	 
//    schedule.scheduleActionAtEnd(wealthGraph, "writeToFile");
//    schedule.scheduleActionAtEnd(forecastGraph, "writeToFile");
//    schedule.scheduleActionAtEnd(trustGraph, "writeToFile");
//    schedule.scheduleActionAtEnd(normalGraph, "writeToFile");
//    schedule.scheduleActionAtEnd(wrsiGraph, "writeToFile");
//   
//    schedule.scheduleActionAt(1679,this,"stop");
//  }

  //#### to replace the schedule (see above)
  public void step(){
     birthHouseholds();
     shuffleHouseholds();
     updateWorld();
     reapHouseholds();	  
  }
  
  public void updateWorld() {
  //RAINFALL IN CLIMATE HAS BEEN CONSOLIDATED TO ALLOW RAINTYPE
  //TO REPRESENT THE WHOLE SEASONS RAINFALL
	  
	  
	  //### allow for more than 1679 steps
	  if (globalTime == 1680) {
		  globalTime = 1;
	  }

  setMonth(globalTime%12+1);  //###
  if (month==1){
	setYear(globalTime/12);   //###
    System.out.println("Year is " + year);
  }
  //###
//  System.out.println("\n*************************** month " + month);
  //###

  if (globalTime==0){
    worldClimate.setNormalYears(this.getMetNormal());
    worldClimate.generateNThreshold();
  }
    // start of the season - need to calculate for 3 months rainfall
    if (month==forecastIssueMonth) {
      worldClimate.update(year);
      forecastModel.update(worldClimate,forecast,year,variable_skill);
    }
	
	//#### Jess
	try {
		engine.executeCommand("(facts)");
		engine.run();
	}
	catch (JessException e) {
		System.out.println(e.getMessage());
		e.printStackTrace();			
	}

	// call the step method on each household in the householdList
    for (int i = 0; i < householdList.size(); i++) {
      FarmingHousehold household = (FarmingHousehold) householdList.get(i);
      household.step();
    }
    marketPlace.update();
    globalTime++;
  }


  /** @return weather-i.e.raintype.  */
  public int getWeather() {
    return worldClimate.currentWeather.getRaintype();
  }

  /** @return forecast-i.e.raintype.   */
  public int getForecast() {
    return forecast.getRaintype();
  }

  /** Randomize the order of the objects (the households) in the householdList
   */
  public void shuffleHouseholds() {
    SimUtilities.shuffle(householdList);
  }

  /** Adds a new household using initial simulation parameters.
   *  Creates average and poor housholds and sets main household attributes.
   */
  public FarmingHousehold addNewHousehold(int size, double startingWealth, int id) {  //#### new: variable size of household; created household is returned
    int numFields = 1;
    FarmingHousehold household = new FarmingHousehold(this, marketPlace, size);
    // Use the initial simulation parameters to construct the households
//    household.setID(agentCount);
    household.setID(id); //### have same id as corresponding household in our model
    household.setPoor(true); //55% poor households. Average households set below.
    household.grainStore.setStore(3*foodNeeded*/*numPeople*/size); //####### 3 months of food
//    household.grainStore.setStore(1680);//foodNeeded * numPeople);
    //x months of food for household  //######## where does x come from??????
    float r = (float) Math.random();
    float mean = 0f, var = 0.2f, xmin = -0.9f, xmax = 0.9f, xdel = 0.1f;
    float trust = psel(mean, var, xmin, xmax, xdel);
    if (r < 0.4) { //40% average HOUSEHOLDS, rather than 45%
      household.setPoor(false);
      household.grainStore.setStore((foodNeeded * /*numPeople*/ size));//0; //one month of food  //#### back to old version: one month of food instead 0
    }    //household properties
    household.setfoodNeeded(foodNeeded);
 //   household.setcashInHand(startingWealth);
	household.setCash(startingWealth);
 //   household.setNumPeople(numPeople);
    household.setforecastSwitch(getUsingForecast());
    household.setTrust(trust);
    household.setCheckLastYear(getCheckLastYear());
    household.setClimateMemory(climate_years);
    addContact(household, 1);
    agentCount++;
    householdBirth(household);
	//### Jess
	try {
		engine.definstance("household", household, true);
	}
	catch (JessException e){
		System.out.println(e.getMessage());
		e.printStackTrace();			
	}
	
//	 System.out.println("%%%%%%% created farming household with size = " + size + " and id = " + id + " %%%%%%%%%%%%");
	 return household;
  }
  /** @param a household.
   *  @param tr trust value.
   */
  public void addContact(FarmingHousehold a, float tr) {
    r = new Representation(a, tr);
    contacts.add(r);
  }

  //the list of households known to this household
  public ArrayList getContacts(){
   return contacts;
  }

  /**
   * @param mean mean
   * @param var variance
   * @param xmin min value in range
   * @param xmax max value in range
   * @param xdel xdel
   * @return a gaussian using the cumulative distribution
   *  to ensure that uniform random numbers map to values
   *  for x with gaussian frequency.
   */
  public float psel(float mean,float var,float xmin,float xmax,float xdel) {
    float a = 0, x = xmin - xdel;
    float r = (float)Math.random();
    while (a < r && x < xmax + xdel) {
      a = a + (float)Math.exp( - (x - mean + xdel) * (x - mean + xdel) / 2 / var / var) /
      (float)Math.sqrt(2 * 3.1415827) / var * xdel;
      x += xdel;
    }
    return x;

  }

  /** Adds households to be created to a birth list.
   * @param household household object.
   */
  public void householdBirth(FarmingHousehold household) {
//    if (this.getTickCount() == 0) {
//      householdList.add(household);
//    }
//    else {
      birthList.add(household);
//    }
  }

  /**Creates the households.*/
  public void birthHouseholds() {
    householdList.addAll(birthList);
    birthList.clear();
  }

  /** When an household "dies" it is added to the reaperQueue if boolean 'replace' is true.
   * @param household household object.
   */
   public void householdDeath(FarmingHousehold household) {
    reaperQueue.add(household);
    // if (replace) {
    //     addNewHousehold();
      //}
	//### Jess: remove shadow fact
	try {
		engine.undefinstance(household);
	}
	catch (JessException e){
		System.out.println(e.getMessage());
		e.printStackTrace();			
	}
     }

    /** Household object that 'died' is removed from the list of agents.*/
   public void reapHouseholds() {
     ListIterator li = reaperQueue.listIterator();
     while (li.hasNext()) {
       FarmingHousehold household = (FarmingHousehold) li.next();
       householdList.remove(household);
     }
     reaperQueue.clear();
   }

  // When a simulation is started through SimInit, some BaseController is
  // created to control the running of that model. This BaseController calls
  // getInitParam() on the model and receives a list of initial parameters
  // that can be displayed for modification to the user. In order to display
  // the value of these parameters the controller determines if the model
  // has implemented get and set methods for that parameter. If so, the
  // controller calls the get method and displays the result to the user. A
  // similar process occurs when a model's initial starting parameters
  // are written to a data file.
  //
  // What this means is that if a user wants to display some initial starting
  // parameter and have this parameter be modifiable, the name of the parameter
  // must be returned by the getInitParam() method and the model must contain
  // the appropriate get and set methods. For example, to do the above for
  // a parameter called Numhouseholds, "Numhouseholds" must be present in the array
  // return by getInitParam and the model must have a getNumhouseholds method and a
  // setNumhouseholds method. The parameter name must match the method names minus
  // the "get" and "set". So "numhouseholds" and getNumhouseholds won't work, but
  // "Numhouseholds" and getNumhouseholds will.
  //
  // The following methods are an example of this pattern.
  /** @return current number of households in the simulation. */
  public int getNumHouseholds() {
    return numhouseholds;
  }

  /** Allows number of households to be varied by the user through the control panel or a parameter file.
   * @param num number of households.
   */
  public void setNumHouseholds(int num) {
    numhouseholds = num;
  }

  /** @return starting wealth.*/
  public float getStartingWealth() {
    return startingWealth;
  }

  /** @param w starting wealth can be varied by the user through the control panel or a parameter file.*/
  public void setStartingWealth(float w) {
    startingWealth = w;
  }

  /** @return starting wealth.*/
  public double getForecastSkillError() {
    return forecastModel.getTwoTercileError();
  }

  /** @param e allows starting skill error to be varied by the user through the control panel or a parameter file.*/
 public void setForecastSkillError(double e) {
    forecastModel.setTwoTercileError(e);
  }

  public void setCorrectForecast(double p){
    forecastModel.setCorrectForecast(p);
  }

  public double getCorrectForecast(){
    return forecastModel.getCorrectForecast();
  }

  /**
   * @return number of years of climate memory
   */
  public int getClimateMemory() {
    return climate_years;
  }

  public void setClimateMemory(int y) {
    climate_years = y;
  }

  /**
 * @return perception of the climate
 */
public boolean getCheckLastYear() {
  return checkLastYear;
}

public void setCheckLastYear(boolean c) {
  checkLastYear = c;
}


  public boolean getLinear() {
    return linear;
  }

  public void setLinear(boolean interaction_type) {
    linear = interaction_type;
  }

  public boolean getInteraction() {
   return interaction;
 }

 public void setInteraction(boolean a) {
   interaction = a;
 }
 /**
  * @param n number of normal years - a paramter that can be set by the user. Default = 30.
  */
 public void setMetNormal(int n){
   met_normal=n;
 }

 /**
  * @return number of normal years - can be a user-defined parameter. Default = 30.
  */
 public int getMetNormal(){
   return met_normal;
 }

 /**
  * @param p household perceived normal
  */
 public void setPerceivedNormal(int p){
 }

 public boolean getUsingForecast() {
  return usingForecast;
}

 public boolean getVariableSkill() {
   return variable_skill;
}
public void setVariableSkill(boolean v) {
  variable_skill=v;
}


 /**
  * @return household perceived normal
  */
 public int getPerceivedNormal(){
   return perceived_normal;
 }


  /** @return paramerters which can be varied by the user via the RePast control panel. */
  public String[] getInitParam() {
    String[] params = {
        "NumHouseholds", "StartingWealth", "CheckLastYear", "MetNormal",
        "Linear","Interaction","ClimateMemory","UsingForecast", "ForecastSkillError","CorrectForecast","VariableSkill"};
    return params;
  }

  /** Every model must have begin() and setup() methods (required for implementing the SimModel interface).
   *  Intializes the model for the start of a run. Consequently, the build* methods are called here,
   *  and any displays are displayed. build()is called whenever the start button (or the step button if
   *  the run has not yet started) is clicked.
   */
  public void begin() {
    globalTime=0;
//    buildModel();
//    buildDisplay();
//    buildSchedule();
//    wealthGraph.display();
//    forecastGraph.display();
//    trustGraph.display();
//    normalGraph.display();
//    wrsiGraph.display();
//    wealthBar.display();
//    dsurf.display();
  }

  /** Prepares the model for another run. Called whenver the setup button
   *  is clicked. Setup should set any objects that are created over the course
   *  of the run to null, and dispose of any DisplaySurfaces or graphs. While
   *  not strictly necessary this should some prevent memory leaks and calling
   *  System.gc() helps too. The initial parameters should be set to whatever
   *  defaults the user wants to see initially.
   */
//  public void setup() {
//    super.setup();
//    schedule = null;
//    householdList = new ArrayList();
//    birthList = new Vector();
//    reaperQueue = new Vector();
//
//    if (wealthGraph != null)
//      wealthGraph.dispose();
//    wealthGraph = null;
//
//    if (forecastGraph != null)
//      forecastGraph.dispose();
//    forecastGraph = null;
//
//    if (trustGraph != null)
//      trustGraph.dispose();
//    trustGraph = null;
//
//    if (normalGraph != null)
//      normalGraph.dispose();
//    normalGraph = null;
//
//
//    if (wrsiGraph != null)
//      wrsiGraph.dispose();
//    wrsiGraph = null;
//
//    if (wealthBar != null)
//      wealthBar.dispose();
//    wealthBar = null;
//
//    if (dsurf != null)
//      dsurf.dispose();
//
//    System.gc();
//
//    // create a schedule with an interval of one.
//    schedule = new Schedule(1);
//
//    // By registering a DisplaySurface, you are allowing Repast to automate
//    // some tasks on the displays surface, such as making movies or taking
//    // snapshots.
//    dsurf = new DisplaySurface(this, "Garden plots");
//    registerDisplaySurface("Main", dsurf);
//
//    // creates a dynamic histogram of the distribution of wealth
//    wealthBar = new OpenHistogram("Wealth (R)", 10, 0, this);
//    //bar2 = new OpenHistogram("Cumulative Grain Deficit (tons)", 10, 0, this);
//    // Create a sequence graph called household Attributes and allow the data
//    // plotted by this graph to be written to the file ./graph_data.txt
//    // in comma delimited format.
//    wealthGraph = new OpenSequenceGraph("Household Wealth", this,"./output/wealth_data.txt",
//                                  OpenSeqStatistic.CSV);
//    forecastGraph = new OpenSequenceGraph("Climate history", this, "./output/skill_data.txt",
//                                   OpenSeqStatistic.CSV);
//    trustGraph = new OpenSequenceGraph("Trust", this, "./output/trust_data.txt",
//                                   OpenSeqStatistic.CSV);
//    normalGraph = new OpenSequenceGraph("OND Met Normal & Rainfall", this, "./output/met_normal_rainfall.txt",
//                                   OpenSeqStatistic.CSV);
//
//    wrsiGraph = new OpenSequenceGraph("Wrsi", this, "./output/wrsi.txt",
//                                   OpenSeqStatistic.CSV);
//    this.registerMediaProducer("Hist", wealthBar);
//    this.registerMediaProducer("Plot", wealthGraph);
//    this.registerMediaProducer("Plot", forecastGraph);
//    this.registerMediaProducer("Plot", trustGraph);
//    this.registerMediaProducer("Plot", normalGraph);
//    this.registerMediaProducer("Plot", wrsiGraph);
//  }

  // a required method
//  public Schedule getSchedule() {
//    return schedule;
//  }

  /** @return name of model which is displayed on the Controller toolbar.*/
  public String getName() {
    return "Modified CLOUD Model";
  }

//  public static void main(String[] args) {
//     SimInit init = new SimInit();
//	 Random.setSeed(1L);
//	 Random.createUniform();
//	 FarmingWorld model = new FarmingWorld();
//	 model.setRngSeed(1L);
////         init.loadModel(model, "params.txt", false);
//         init.loadModel(model, "", false);
//  }
  
  //#### all for Jess
  public int getMonth(){
	  return month;
  }
  
  public void setMonth(int month){
	  int old = this.month;
	  this.month = month;
	  pcs.firePropertyChange("month", old, this.month);
  }
  public int getYear(){
	  return year;
  }
  
  public void setYear(int year){
	  int old = this.year;
	  this.year = year;
	  pcs.firePropertyChange("year", old, this.year);
  }
  
  public void setUsingForecast(boolean u) {
	  boolean old = this.usingForecast;
	  this.usingForecast = u;
	  pcs.firePropertyChange("usingForecast", old, this.usingForecast);

}
 
	public void addPropertyChangeListener(PropertyChangeListener p){
		pcs.addPropertyChangeListener(p);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener p){
		pcs.removePropertyChangeListener(p);
	}
   
}
