package org.cfpm.ruth.modifiedCloud;

import java.util.ArrayList;
import java.util.*;
/**
 * Title: Field
 * Description: A container for the crops that have been planted to date
 * currently intended to have up to three active crops selected from
 * maize butternut and cabbage
 * Copyright: Copyright (c) 2004
 * Company:
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */
public class Field {

  public boolean maize_planted = false, butternut_planted = false,
      cabbage_planted = false;
  public ArrayList cropsPlanted = new ArrayList();
  private FarmingHousehold owner;

  public Field(FarmingHousehold h) {
    owner=h;
  }

  public Field(int x, int y) {
    //  setLocation(x,y);
  }

  /** Iterates through each crop in the list of this agent and harvests crops
   * that are ripe - returns theses as an ArrayList of goods
   * @return ArrayList with yield of goods from crops harvested.
   */
  public ArrayList harvest() {
    ArrayList yield = new ArrayList();
	 //######### FINALLY found the bug: cropsPlanted is modified inside the for loop if crop is ripe --> can't use for loop!!!!
//    for (int i = 0; i < cropsPlanted.size(); i++) {
	 int i = 0;
	 while (i < cropsPlanted.size()) {
      Crop c = (Crop) cropsPlanted.get(i);
        if (c.ripe()){
//			  System.out.println("crop is ripe");
          cropsPlanted.remove(c); //remove from list when harvested.
          float quantity=c.harvest();
          //only add goods if there's anything to add!
          //prices are set to zero (these are obtained at the market later)
          if (quantity>0)yield.add(new Goods(owner,quantity,0,0,c.name));
//         System.out.println("Yield of " + owner.getID() + " "+ owner.getPoor() + " " + c.name + " is " + quantity);
        }
        else {
			  i++;
//			  System.out.println(Crop.getCropName(c.name) + " is not ripe yet");  //####
        }
	  }
   return yield;
  }

  /**
   * Remove crop due to poor crop development.
   * (or poor mid-season forecast in the future)
   * @param c Crop object.
   */
  public void removeCrop(Crop c) {
    for (int i = 0; i < cropsPlanted.size(); i++) {
      if (c.name != Crop.MAIZE) { // remove any crop except maize.
        if (cropsPlanted.get(i) == c) {
          cropsPlanted.remove(c);
          System.out.println("Removing crop " + c.name);
        }
      }
    }
  }

  /** CropWat is not sensitive enough to be able to do this.
   * Crop failure - below subsistence requirements. WRSI can be a user-defined parameter.
   */
  public void checkCrops() {
    int wrsiThreshold = 0; // make this a threshold that each crop has to reach at each growing stage
    for (int i = 0; i < cropsPlanted.size(); i++) {
      Crop c = (Crop) cropsPlanted.get(i);
      if (c.getWrsi() < wrsiThreshold) { //threshold below which crop cannot successfully be grown.
        removeCrop(c); // need option to plant something else
      }
    }
  }

  public void update() {
    for (int i = 0; i < cropsPlanted.size(); i++) {
      Crop c = (Crop) cropsPlanted.get(i);
      c.update();
    }
  }

  /** Iterates through each crop in the crop list of this agent to check if any is ripe.
   * @return whether any of the planted crops is ripe or not.
   */
  public boolean ripe() {
    boolean r = false;
    for (int i = 0; i < cropsPlanted.size(); i++) {
      Crop c = (Crop) cropsPlanted.get(i);
      r = r || c.ripe();
      if (r == true)break;
    }
    return r;
  }

  /**
   * Adds the crops to be planted from an input list
   * The crop will only actually be planted if the
   * crop's planting month matches the current month,
   * it hasn't already been planted (this is rest on harvest)
   * and it is flagged as needed for this year. The field
   * keeps track of the currently planted crops in
   * and arraylist (also updated on harvest)
   * @param crops ArrayList
   * @param month int
   * @param globalTime current time
   */
  public void plant(ArrayList crops,int month,int globalTime){
    for (Iterator iter = crops.iterator(); iter.hasNext(); ) {
      Crop c = (Crop) iter.next();
      if (c.cropMonth == month && !c.planted && c.plantThisYear) {
        c.plant(globalTime);
        cropsPlanted.add(c);
			//####
			System.out.println("-- planting " + Crop.getCropName(c.name));
     }
    }
    whichCrops();
	}

  public void whichCrops() {
    for (int i = 0; i < cropsPlanted.size(); i++) {
      Crop c = (Crop) cropsPlanted.get(i);
      if (c.name == Crop.MAIZE) {
        maize_planted = true;
      }
      if (c.name == Crop.BUTTERNUT) {
        butternut_planted = true;
      }
      if (c.name == Crop.CABBAGE) {
        cabbage_planted = true;
      }
    }
  }

  /**
   * @return true if maize is planted.
   */
  public boolean getMaize() {
    return maize_planted;
  }

  public void setMaize(boolean m) {
    maize_planted = m;
  }

  /**
   * @return true if butternut is planted.
   */
  public boolean getButternut() {
    return butternut_planted;
  }

  public void setButternut(boolean b) {
    butternut_planted = b;
  }

  /**
   * @return true if cabbage is planted.
   */
  public boolean getCabbage() {
    return cabbage_planted;
  }

  public void setCabbage(boolean c) {
    cabbage_planted = c;
  }
}
