package org.cfpm.ruth.modifiedCloud;

import java.io.BufferedReader;
import java.io.FileReader;
/**
 * Title: General Circulation Model
 * Description:A simple model of a forecast model
 * Copyright: Copyright (c) 2004
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */
public class GCM {

  /**Number of terciles out.*/
  public int error = 0;

  /**Percentage that forecast is not two terciles out.
   * Forecast correct 65% of the time, way out 10% of the time.
   * 65% error=0, 10% error=2, 25% error=1. Now compared with real timeseries data.
   */
  public double percentage = 0.9;
  public double correct_forecast = 0.65;
  public double midseason_forecast;
  public static float skill[] = new float[140];
  public boolean variable_skill = false;

  public GCM() {
    try {
      String line;
      float number;
      int count=0;

      BufferedReader input = new BufferedReader(new FileReader("skill.txt"));
      line=input.readLine();
      while (line!=null){
        number=Float.parseFloat(line);
        skill[count] = number;
        count++;
        line=input.readLine();
      }
    }
    catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

  /** Generates the forecast error.
   * @param worldClimate climate object
   * @param forecast weather object
   * @param year skill for year
   */
  public void update(Climate worldClimate, Weather forecast, int year, boolean s) {
      variable_skill=s;
      float r = (float) Math.random();
//r can still be random but correct forecast will now reflect real rainfall data - see below
      if (variable_skill){
        if(skill[year]<0.3){
          setError(2);
        }
        if(skill[year]>= getCorrectForecast())
          setError(0);
        if((skill[year]>0.3)&&(skill[year]< getCorrectForecast())){
          setError(1);
        }
      }
  if (!variable_skill){
    if (r <= getCorrectForecast())
        setError(0); else {
      if (r > this.getTwoTercileError())
        setError(2);
      else
        setError(1);
    }
  }
    generateForecast(worldClimate, forecast);
  }

  public void setSkill(int year, float s) {
    skill[year] = s;
  }

  public float getSkill(int year) {
    return skill[year];
  }

  public void setError(int e) {
    error = e;
  }

  public int getError() {
    return error;
  }

  public void setTwoTercileError(double p) {
    percentage = p;
  }

  public double getTwoTercileError() {
    return percentage;
  }

  public void setCorrectForecast(double p) {
    correct_forecast = p;
  }

  public double getCorrectForecast() {
    return correct_forecast;
  }

  /** If error=0, forecast = weather.
   *  If wet weather(1) and error=1, forecast=0 (normal). If error=2 forecast=-1(dry).
   *  If normal weather(0) and error=1, forecast=1(wet). If error=2, forecast=2,
   *  which equates to 1(wet, see line 443) but see line 435 which  equates to dry 50%
   *  of the time if weather is normal and error is type 2.
   *  If dry(-1) and error=1, forecast=0(normal). If error=2, forecast=1(wet).
   *  If current weather is normal, forecast error still included 50% of time,
   *  but error could be 0 which it is 65% of the time.
   *  50% of the time error should be included if weather is normal
   *  but error could be 0 which it is 65% of the time.
   *  to allow error=2(-2) to generate dry forecast when weather is normal (line 438).
   *
   * @param worldClimate Climate object
   * @param forecast Weather object
   */
  public void generateForecast(Climate worldClimate, Weather forecast) {
    if (worldClimate.currentWeather.getRaintype() > Weather.NORMAL) { // wet=1
      forecast.setRaintype(worldClimate.currentWeather.getRaintype() - getError());
      // If error=0, forecast = weather.
      // If wet weather(1) and error=1, forecast=0 (normal). If error=2 forecast=-1(dry).
    }
    else { //normal=0,dry=-1
      forecast.setRaintype(worldClimate.currentWeather.getRaintype() + getError());
      // If normal weather(0) and error=1, forecast=1(wet). If error=2, forecast=2,
      // which equates to 1(wet, see line 443) but see line 435 which  equates to dry 50%
      // of the time if weather is normal and error is type 2.
      // If dry(-1) and error=1, forecast=0(normal). If error=2, forecast=1(wet).
      if (worldClimate.currentWeather.getRaintype() == Weather.NORMAL) {
        // If current weather is normal, forecast error still included 50% of time,
        // but error could be 0 which it is 65% of the time.
        if ( (float) Math.random() > 0.5) {
          forecast.setRaintype( -getError());
          // 50% of the time error should be included if weather is normal
          // but error could be 0 which it is 65% of the time.
        }
      }
    }
    forecast.setRaintype(Math.max(Math.min(forecast.getRaintype(), 1), -1));
    //to allow error=2(-2) to generate dry forecast when weather is normal.
      System.out.println("forecast: "+forecast.getRaintype());
  }

}

