package org.cfpm.ruth.modifiedCloud;

/**
 * Title: maizeCrop Implemented as part of a strategy for dealing
 * with crop variation
 *
 * Description: Subclass of the generic crop class that sets
 * values appropriate to cabbage in mangondi village, limpopo provice
 * South Africa. Maize is grown as a subsistence crop by poorer farmers
 * Copyright: Copyright (c) 2003
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */


public class MaizeCrop extends Crop{
  public MaizeCrop() {
    //first set the default generic crop properties
    super();
    //now the specifics
    name=Crop.MAIZE;
    //maize (default-September)
    //apply the maize crop coefficients to cropWat.
    cropModel.setCropCoefficients(0.5328f,1.027f,0.9f);
    cropMonth=FarmingWorld.SEPTEMBER;
    cost = 4;
  }
  
  
  //#### add convenience constructor
  public MaizeCrop(float area, int cropMonth, boolean plantThisYear){
	  this();
	  this.area = area;
	  this.cropMonth = cropMonth;
	  this.plantThisYear = plantThisYear;
  }
  
  /**
 * This function calculates the yield given the Wrsi that comes
 * from cropWat - cropWat itself doesn't calculate it
 * the numbers below are specfically for typical yields in
 * Mangondi for maize. Yields are in kilograms per plot
 * planted, and must be scaled for the crop area (this done in
 * the harvest method)
 * Data shows xx tonnes/ha from FAOSTAT Database Results for 2000-2002
 * thus xx t/ha for 0.25 ha
 * @param Wrsi wrsi value
 * @return yield in kg for area planted
 */

  protected float getYield(int Wrsi) {
     float normalizedYield=0;
       switch (Wrsi) {
        case 1:                                                  //
          normalizedYield = 2.8f*lines;//84kg                   //11*lines;//330
           break;
         case 2:
           normalizedYield =  5*lines;//150                   18*lines;//540        //
           break;
         case 3:
           normalizedYield = 7.2f*lines;//216            25*lines;//750                           //
           break;
         case 4:
           normalizedYield = 9.5f*lines;//285  32*lines;//960        //                 //
           break;
         case 5:
           normalizedYield = 11.9f*lines;//357   39*lines;//1170       //               //
           break;
         case 6:
           normalizedYield = 14.1f*lines;//423  47*lines;//1410      //               //53*lines;//1590
           break;
         default:
           normalizedYield = 1;
           break;

//### try and return to original(?) values since maize crop seems to be much too low compared to butternut and cabbage
//		 case 1:                                                  //
//		          normalizedYield = 11*lines;//330
//		           break;
//		         case 2:
//		           normalizedYield =  18*lines;//540        //
//		           break;
//		         case 3:
//		           normalizedYield = 25*lines;//750                           //
//		           break;
//		         case 4:
//		           normalizedYield = 32*lines;//960        //                 //
//		           break;
//		         case 5:
//		           normalizedYield = 39*lines;//1170       //               //
//		           break;
//		         case 6:
//		           normalizedYield = 47*lines;//1410      //               //53*lines;//1590
//		           break;
//		         default:
//		           normalizedYield = 1;
//		           break;
//		 
		 
		 }
      return normalizedYield;
  }
}
