package org.cfpm.ruth.modifiedCloud;

import java.util.*;
/**
 * Title: container for goods that are to be bought or sold
 * Description: Holds a lists of GOODS, buyers and sellers,
 * set the prices (currently for urban or rural market) and allocates
 * quantities according to cash that is included in each bundle of goods
 * Copyright: Copyright (c) 2004
 * Company: University of Cambridge/SEI Oxford
 * @author Mike Bithell/Sukaina Bharwani
 * @version 1.0
 */
public class Market {

  public Climate rainfall = new Climate();
  public Weather forecast = new Weather();
  public boolean high_market_demand;
  public boolean is_commodity;
  public boolean high_prices;
  public Vector buyers = new Vector();
  public Vector sellers = new Vector();
  public Market() {}


   /**
    *
    * @param f goods to buy
    */
   public void buy(Goods f) {
     buyers.add(f);
   }

  /**
   *
   * @param f goods to sell
   */
  public void sell(Goods f) {
    sellers.add(f);
  }

/**
 *prices for urban and rural market in Mangondi
 ******************************************************************************
 *                       1       2      3        4        5      6            *
 * ****************************************************************************
 * Yield(kg)		2.8	5	7.2	9.5	11.9	14.1	0     *
 * (30lines)	 	84	150	216	285	357	423	0     *
 * maize R1.75          147  	31.5	43.75	56	68.25	82.25	0     *
 * maize R2.50	 	27.5	45	62.5	80	97.5	117.5	0     *
 ******************************************************************************
 * Yield(kg)	 	35	52	69	85	104	120	139   *
 * (30lines)	 	1050	1560	2070	2550	3120	3600	4170  *
 * b/nut R0.87	 	913.5	45.24	60.03	73.95	90.48	104.4	120.93*
 *       (50%)         (456.75)
 * b/nut R1.74	 	1827	90.48	3601.8	147.9	180.96	208.8	241.86*
 ******************************************************************************
 * Yield(kg)	 	11	45	75	110	140	170	201   *
 * (30lines)	 	330	1350	2250	3300	4200	5100	6030  *
 * cabbage R2.0	 	22	90	150	220	280	340	402   *
 * cabbage R2.33 	768.9	3145.5	5242.5	256.3	326.2	396.1	468.33*
 ******************************************************************************
 */
public void update() {
 // System.out.println("market updated: Buyers: " + buyers.size() +
  //                    " Sellers: " + sellers.size());
    if (buyers.size() != 0) {
      for (int i = 0; i < buyers.size(); i++) {
        Goods buyer = (Goods) buyers.get(i);
        //buyers and sellers get different prices depending on
        //where they can sell (poor sell/buy at the field, others at market)
        buyer.price=getPurchasePrice(buyer.owner.getPoor(),buyer.name);
        //cash returned to the nearest "cent"
        double cash = (double) ( (int) (buyer.quantity * buyer.price * 100)) / 100;
        //buyers try to buy the amount they want - if not they buy as much as
        //the cash allocated to this good will allow
        if (cash <= buyer.cash) {
          buyer.cash -= cash;
          buyer.bought = buyer.quantity;
        }
        else {
   //       if they do not have enough cash here agent should be removed as this is req for sub
//          buyer.owner.world.householdDeath(buyer.owner);
			  //### no, don't let them die here!
          buyer.bought = (float)buyer.cash / buyer.price;
          buyer.cash = 0;
        }
     //   sometimes av agents are not buying maize for subs as they do not have enough cash,
     //         but there must be a penalty/effect of this
      }
    }
    if (sellers.size() != 0) {
      for (int i = 0; i < sellers.size(); i++) {
        Goods seller = (Goods) sellers.get(i);
        seller.price=getSellingPrice(seller.owner.getPoor(),seller.name);
        //cash returned to the nearest "cent"
        //sellers can sell as much as they like
        seller.cash += (double) ( (int) (seller.quantity * seller.price * 100)) /
            100.;
        seller.sold = seller.quantity;
      }
    }
    empty();
  }

  /**
   * getPurchasePrice
   * @param isPoor if the household is poor
   * @param cropName name of the crop
   * @return price of the crop dependent on where it is sold (at the garden(rural) or at market(urban))
   */
  private float getPurchasePrice(boolean isPoor, int cropName) {
    float price;
    if (isPoor) price=rural.getPurchasePrice(cropName);
    else        price=   urban.getPurchasePrice(cropName);
    return price;
  }

  /**
   * getSellingPrice
   *
   * @param isPoor if the household is poor
   * @param cropName name of the crop
   * @return float price of the crop dependent on where it is sold (at the garden(rural) or at market(urban))
   */
  private float getSellingPrice(boolean isPoor, int cropName) {
    float price;
    if (isPoor) price = rural.getSellingPrice(cropName);
    else        price = urban.getSellingPrice(cropName);
    //elevate the selling price dependent on the climate
    price=price*getPriceBoost();
    return price;
  }

  public void empty() {
    buyers.clear();
    sellers.clear();
  }

  /**
   * Coupling market to climate.
   * Household gets forecast for the season to get 'expected' market demand - affects crop choices since prices are expected
   * to change. However, prices will only be affected if the weather is in fact dry as predicted.
   * @return expected price boost if forecast is dry
   */
 private float getPriceBoost() {
   if (rainfall.currentWeather.getRaintype() == Weather.DRY) {
     return 1.2f;
   }
   return 1f;
 }
}
