package org.cfpm.ruth.modifiedCloud;

import java.io.*;
/**
 * COMMENTS BY TED FEB 2003. Based on programs by G.J.Kenny and P.A.Harrison.
 * Recoded to VB: TE Downing 18 September 1995
 * Recoded to Java: S Bharwani 10 March 2002
 * Program to calculate FAO water balance
 * for maize, butternut and cabbage, version 3.
 * The main inputs to this program are precipitation, PET,
 * available_water_holding_capacity
 * and crop coefficients for different stages of crop development.
 * FAO Plant Production and
 * Protection Paper No.73 is the source of the water balance used here.
 */

public class cropWat {
  public cropWat() {
  }

  public int current_stage;
  /** Crop coefficient for initiation (first)stage.*/
  public int initiation = 0;
  /** Crop coefficient for growing (second)stage.*/
  public int growing = 1;
  /** Crop coefficient for flowering (third)stage.*/
  public int flowering = 2;
  /** Crop coefficient for maturation/harvesting (fourth)stage.*/
  // public int maturation = 3;
  /**Precipitation (monthly).*/
  public static float monthly_precipitation[] = new float[1680];
  /**Potential evapotranspiration (monthly).*/
  public static float pet[] = new float[1680];
  /**Crop co-efficient values for each stage of crop development.*/
  public float crcoeff[] = new float[3];
  /**For water requirement calculation.*/
  public float water_req[] = new float[3];
  /**For water requirement calculation.*/
  public float available_water[] = new float[3];
  /** Amount of water remaining in the soil.*/
  public float reserve_soil_water[] = new float[3];
  /**Runoff*/
  public float surplus[] = new float[3];
  /**Water deficit which is cumulative.**/
  public float deficit[] = new float[3];
  /**Accumulated water deficit.**/
  public float accum_deficit[] = new float[3];
  /**Water requirements satisfaction index (wrsi)--final output of relative water stress.**/
  public float water_req_sat_index[] = new float[3];
  public float current_wrsi_value=0;
  /**Accumulated over season.**/
  public float tot_water_req=0;

  /**
   * THE FAO DATA BASE RECORD LENGTH OF GROWING SEASON AND DEVELOPMENT STAGES IN DAYS--
   * THESE CONVERT INTO MONTHS:
   * Calculate the length of the growing season
   * LDays=InitStageDays+DevtStageDays+MidStageDays+LateStageDays
   * l=LDays/30
   * For testing, maize stages are 25, 35, 40, 30; LDays is 130:
   * Represents each stage of the growing season.
   */
  public int LMonths = 3;
  /** Available water holding capacity of soil, in mm. Use 25, 50 and 100 as representative
   * of poorer, sandy soils or 50, 100 and 200 as wider range pm. For testing use loam soil
   * and corresponding available_water_holding_capacity.
   */
  public int available_water_holding_capacity = 50;

  /**
   * Get Kc coefficients from dBase, interpolate DevtKc from InitKc and MidKc
   * Check on method for doing this from CropWat, below is a rough approximation
   * NB : need to wrap months around end of year
   * For testing, maize Kc by stage are 0.45, 1.1, 0.55
   * For i = 1 to LMonths
   * Next i
   * Open input and output files, and read in data
   * monthly_precipitation = precip;
   * don't need lat and lon. Only need to read files for months j to k
   * read(1, * , end = 200) lat, lon, (monthly_precipitation(i), i = 3, 12),
   * (monthly_precipitation(i), i = 1, 2)
   * read(2, * , end = 200) lat, lon, (pet(i), i = 3, 12), (pet(i), i = 1, 2)
   * read(4, * , end = 200) lat, lon, available_water_holding_capacity
   */

  /**
   * @param globalTime current month
   */
  public void initialize(int globalTime) {
    init();
    //I think this is right - start cropwat in the month planting occurs
    calcCropWatReq(globalTime);
    //is 0 correct for initialization?
    calcSoilWat1(0);
    calcWrsi1(0);
  }

  /**
   * Called from crop to update the current state
   * @param current_stage current stage
   * @return wrsi
   */
  public float update(int current_stage) {
    calcSoilWatN(current_stage);
    calcWrsiN(current_stage);
    calcWrsiIndex(current_stage);
    return getWrsi();
  }

  /**
   * Retrieve precipitation data.
   */
  public static void getPrec() {
    try {
      String line;
      float number;
      int count = 0;
      BufferedReader input = new BufferedReader(new FileReader("prec.txt"));
      //only need values for the growing season? - need all months as planting date may change.
      line = input.readLine();
      while (line != null) {
        number = Float.parseFloat(line);
        monthly_precipitation[count] = number;
        count++;
        line = input.readLine();
      }
    }
    catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

  /**
   * Retrieve PET data.
   */
  public static void getPet() {
    try {
      String line;
      float number;
      int count = 0;
      BufferedReader input = new BufferedReader(new FileReader("pet.txt"));
//need all months not just growing season as planting date may change.
      line = input.readLine();
      while (line != null) {
        number = Float.parseFloat(line);
        pet[count] = number;
        count++;
        line = input.readLine();
      }
    }
    catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

  /**
   * Calculate the water requirement of the crop for month(water_req) and
   * season(tot_water_req) and available water(available_water).
   * pet[j] to allow any month to be the starting month of the growing season
   * total water requirements is the value that will change, when a different crop is chosen
   * affecting available_water[i]. available_water[i] is used in further calculations.
   * All three month crops.
   * @param zeroth first stage
   * @param first second stage
   * @param second third stage
   */

  public void setCropCoefficients(float zeroth, float first, float second) {
    crcoeff[0] = zeroth;
    crcoeff[1] = first;
    crcoeff[2] = second;
  }

  public void calcCropWatReq(int j) {
    tot_water_req=0;
    for (int i = 0; i < LMonths; i++) {
      water_req[i] = pet[j] * crcoeff[i];
      // pet[j] to allow any month to be the starting month of the growing season
      tot_water_req = tot_water_req + water_req[i];
      //total water requirements is the value that will change, when a different crop is chosen.
      available_water[i] = monthly_precipitation[j] - water_req[i];
      j++; // to get pet[] and prec[] for subsequent months
    }
  }

  /**
   * Calculate the soil water reserve (reserve_soil_water)for each month, and whether
   * there is a surplus or deficit. In the first month if available_water is positive
   * then this becomes the reserve_soil_water, up to a maximum of the
   * available_water_holding_capacity. The excess above available_water_holding_capacity
   * is assigned as the surplus, s. If available_water is negative then reserve_soil_water
   * is reduced by that amount to a minimum of 0, and available_water is equal to a deficit(D).
   * The same rules are applied in successive months. Note that reserve_soil_water is cumulative
   * up to a maximum of available_water_holding_capacity, and that it is never less than 0.
   * A monthly surplus is not cumulative, but the deficit is, for the calculation of the water
   * requirement satisfaction index(WRSI). This is described later.
   * Calculate reserve_soil_water, deficit and surplus for month j,
   * the first month in the growing season for maize.
   * @param j represents growth development stage.
   */
  public void calcSoilWat1(int j) { //
    if ( (available_water[j] > 0) &&
        (available_water[j] < available_water_holding_capacity)) {
      reserve_soil_water[j] = available_water[j];
      surplus[j] = 0;
      deficit[j] = 0;
    }
    else
    if (available_water[j] >= available_water_holding_capacity) {
      reserve_soil_water[j] = available_water_holding_capacity;
      surplus[j] = available_water[j] - available_water_holding_capacity;
      deficit[j] = 0;
    }
    else {
      reserve_soil_water[j] = 0;
      surplus[j] = 0;
      deficit[j] = available_water[j];
    }
//Set the monthly deficit d equal to accum_deficit, which is to be an accumulated deficit
    accum_deficit[j] = deficit[j];
  }

  /** Calculate reserve_soil_water, deficit, surplus and accum_deficit for all remaining months.
   * @param i represents growth development stage.
   */
  public void calcSoilWatN(int i) { //
    // i=2 for second month
    // for (int i = 2; i < LMonths; i++) {
    if ( (available_water[i] + reserve_soil_water[i - 1] > 0) &&
        (available_water[i] + reserve_soil_water[i - 1] <
         available_water_holding_capacity)) {
      reserve_soil_water[i] = available_water[i] + reserve_soil_water[i - 1];
      surplus[i] = 0;
      deficit[i] = 0;
    }
    else
    if (available_water[i] + reserve_soil_water[i - 1] >=
        available_water_holding_capacity) {
      reserve_soil_water[i] = available_water_holding_capacity;
      surplus[i] = (available_water[i] + reserve_soil_water[i - 1]) -
          available_water_holding_capacity;
      deficit[i] = 0;
    }
    else {
      reserve_soil_water[i] = 0;
      surplus[i] = 0;
      deficit[i] = available_water[i] + reserve_soil_water[i - 1];
    }
    accum_deficit[i] = accum_deficit[i - 1] + deficit[i];
    // }
  }

  /**
   * Calculate the water requirement satisfaction index (WRSI). If soil
   * moisture is greater than or equal to 0 then WRSI is equal to 100.
   * Otherwise WRSI is reduced by the fraction of the deficit (d) over the
   * total water requirement (tot_water_req).
   * Calculate WRSI for the first month.
   * @param j first month.
   */
  public void calcWrsi1(int j) {
    if (deficit[j] == 0) {
      water_req_sat_index[j] = 100;
    }
    else {
      water_req_sat_index[j] = 100 + ( (deficit[j] / tot_water_req) * 100);
    }
  }

  /**
   * Calculate water_req_sat_index for all remaining months.
   * @param i any month after first month
   * @return wrsi for month i
   */
  public float calcWrsiN(int i) {
    // for (int i = 2; i < LMonths; i++) {
    if ( (deficit[i] == 0) && (water_req_sat_index[i - 1] == 100)) { //.eq.100))
      water_req_sat_index[i] = 100;
    }
    else
    if ( (deficit[i] == 0) && (water_req_sat_index[i - 1] < 100)) {
      water_req_sat_index[i] = water_req_sat_index[i - 1];
    }
    else {
      water_req_sat_index[i] = 100 +
          ( (accum_deficit[i] / tot_water_req) * 100);
    }
    //  }
    return water_req_sat_index[i];
  }

  /**
   * Any missing values, set to -1; all corresponding, derived variables  are set to - 1. //### nope, set to 0
   */
  public void init() {
    for (int i = 0; i < LMonths; i++) {
        water_req[i]=0;
        available_water[i]=0;
        reserve_soil_water[i]=0;
        surplus[i]=0;
        deficit[i]=0;
        accum_deficit[i]=0;
        water_req_sat_index[i]=0;
    }
  }

  /**
   * The final output assigns yield classes(% reductions) to ranges of wrsi. These could be
   * user-controlled parameters--drought resistant crops would have different
   * yield sensitivity. It could also be handled in a (nonlinear?) formula rather than classes?
   * Code index values, to assign an integer value to each class (refer toFAO, 1986) for the
   * classification. Output the coded WRSI for the last month of the growing season. This final
   * value reflects the effects of the accumulated water budget through
   * the growing season, and is taken as an index of yield, under rainfed
   * conditions.  Complete crop failure occurs when the index is less than
   * 50.  Originally unclear what k should be - LMonths; should be the last month
   * of the calculated wrsi since it is cumulative.
   * Calculated at every stage and checked so that new strategy can be implemented if water req
   * cannot be met.
   * @param current_month current month
   */
  public void calcWrsiIndex(int current_month) {
    // to get at the final month of wrsi calc so far for midseason checks.
    // relate these indices to yields for each crop.
    int k = current_month;
    for (int i = 0; i < water_req_sat_index.length; i++) {
      k = i;
    }
    if (water_req_sat_index[k] == -1) {
      setWrsi( -1);
    }
    else
    if ( (water_req_sat_index[k] < 50) && (water_req_sat_index[k] >= 0)) {
      setWrsi(1);
    }
    else
    if ( (water_req_sat_index[k] >= 50) && (water_req_sat_index[k] <= 59)) {
      setWrsi(2);
    }
    else
    if ( (water_req_sat_index[k] >= 60) && (water_req_sat_index[k] <= 79)) {
      setWrsi(3);
    }
    else
    if ( (water_req_sat_index[k] >= 80) && (water_req_sat_index[k] <= 94)) {
      setWrsi(4);
    }
    else
    if ( (water_req_sat_index[k] >= 95) && (water_req_sat_index[k] <= 99)) {
      setWrsi(5);
    }
    else
    if (water_req_sat_index[k] >= 100) {
      setWrsi(6);
    }
   // System.out.println("WRSI is " + getWrsi());
  }

  public void setWrsi(float a) {
    current_wrsi_value = a;
  }

  /**Returns wrsi value
   * @return wrsi value.*/
  public float getWrsi() {
    return current_wrsi_value;
  }
}
 /**
  *  If WRSI during the season is less that what is required as strategy choice is required...
  *  either changeCrop, change timing (delay) - this should come in depending on the forecast
  *  - ANTICIPATORY
  *  OR change area (if both crops being grown decrease area) - this would also occur with respect
  *  to the forecast - ANTICIPATORY
  *  OR change crop (if growing one market crop, choose other) - this would happen when conditions
  *  are unexpected and favour the alternative option - REACTIONARY
  */
  /**
   * Adaptation options to be included are:
   * 1.Fertilizer x
   * 2.Density (thinning) x
   * 3.Irrigation x
   * 4.Drought resistant varieties (Changing crops)
   * 5.Changing planting dates (Changing crops)
   * 6.Increasing area under cultivation
   */

