package units;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;

import jess.JessException;
import jess.QueryResult;

import clubs.Club;
import clubs.SavingsClub;

import hivaids.Sigmoid;
import infastructure.Message;
import infastructure.Settings;
import cern.jet.random.Normal;
import uchicago.src.sim.util.Random;
import uchicago.src.sim.util.SimUtilities;
import economics.Grant;
import gui.HouseholdNode;
import gui.Model;

public class Agent {
	/* agent's id ... */ 
	private int id = -1;
	/* assign 0 -> 'Female'; '1' -> 'Male' ; -1 -> 'unassigned' */ 
	private int gender = -1;
	/*ID of this household to this individual belongs */
	private int houseHoldID = -1;	
	/*health status of the agent is important ... very important*/
	private int healthStatus = Settings.HEALTH_STATUS.WELL;
	/* age of the agent ... */ 
	private int age = 0;
	//i.e. child, adult, senior
	private int ageGroup = -1;
	/* a grant is an agent's property, though it's shared by the 
	 * entire household ... it's also assumed that an individual 
	 * is getting exactly one grant.. e.g. if there is a child suffering from HIV 
	 * we allocate the grant which is more ... i.e., the child_grant and when he/she gets old,
	 * it start receiving the hiv grant because the child grant stops
	 */
	private Grant grant;	
	private Model model;	
	private boolean isAlive = true;	
	private int maritalStatus = -1;	
	private int motherID = -1;
	private int fatherID = -1;
	private int spouseID = -1;	
	//this is inspired from emma and bogdan's sahel 
	private int healthCount = 100;
	private boolean isOrphan = false;	
	private int feedStatus = Settings.FEED_STATUS.FULL_FED;	
	private boolean remittance = false;
	private int remittance_Amount = 0;	
	//is head of the household
	private boolean isHead = false;	
	
	/*
	 * Last Age: is drawn from a normal distribution with a mean life expectancy and
	 * standard deviation ... both PARAMETERS in the Settings ...
	 * SEE constructor ... needs separate parameter values for males and females 
	 */
	private int expectedLife = -1;	
	private int infectedTick = -1;
	private int infectedAge = -1;	
	private int birthTick = -1;	
	
	//temporary
	Sigmoid sigmoid = new Sigmoid();
	
	private boolean isInnovator = false;
	private boolean isMemberSavings = false;
	private boolean pullOutMembership = false;
	private boolean committed = false;
	
	private ArrayList<Agent> friends = new ArrayList<Agent>();	
//	private ArrayList<Message> inbox = new ArrayList<Message>();
//	private ArrayList<Message> outbox = new ArrayList<Message>();	
	private ArrayList<Agent> invited = new  ArrayList<Agent>();
	private ArrayList<Agent> acceptances = new ArrayList<Agent>();

	//### for Jess: changes to attributes isMemberSavings and committed need to be visible instantly in Jess
	private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
	private double lastTryStokvel = 0;
	private int numStokvelTries = 0;
	
	public static int INVITE = 0;
	public static int INVITE2 = 1;
	public static int ACCEPT = 2;
	public static int REJECT = 3;
	public static int STOKVEL = 4;
	
	public Agent(){		
	}
	
	public Agent(int _id){
		this.id = _id;
	}
	
	public Agent(int _id, int _gender, int _ageGroup, int _healthStatus, int _houseHoldID, Model _model) {
		this.id = _id;
		this.ageGroup = _ageGroup;
		this.healthStatus = _healthStatus;
		this.houseHoldID = _houseHoldID;
		this.gender = _gender;
		
		this.model = _model;		

		switch (this.ageGroup) {
		case Settings.AGE.CHILD:
			age = Random.uniform.nextIntFromTo(1, Settings.CHILD_AGE_LIMIT);					
			break;
		case Settings.AGE.ADULT:
			age = Random.uniform.nextIntFromTo(Settings.CHILD_AGE_LIMIT+1, Settings.ADULT_AGE_LIMIT);
			break;
		case Settings.AGE.SENIOR:			
			age = Random.uniform.nextIntFromTo(Settings.ADULT_AGE_LIMIT+1, (int) Settings.MEAN_LIFE_EXPECTANCY);
			break; 
		default: break;
		}		
		initialize();
	}	
	
	//this is for child who's born during simulation
	public Agent(int _id, int _gender, int _healthStatus, int _houseHoldID, Model _model) {
		this.id = _id;
		this.ageGroup = Settings.AGE.CHILD;
		this.age = 1;		
		this.healthStatus = _healthStatus;
		this.houseHoldID = _houseHoldID;		
		this.model = _model;			
		this.gender = _gender;		
		initialize();
		initializeFriends();
	}	
	
	private void initialize() {
		//CAUTION: this is same for male/females ... need different 
		this.expectedLife = (int) model.getMyNormal().nextDouble();
		while(this.expectedLife <= this.age) {
			this.expectedLife = (int) model.getMyNormal().nextDouble();
		}
		this.maritalStatus = Settings.MARITAL_STATUS.SINGLE;
//		System.out.println("**************** agent: " + this.id + "has age: " + this.age + "********* and life: " + this.expectedLife);
		this.isInnovator = Math.random() < Settings.INNOVATOR_PROBABILITY ? true : false;		
		model.getAgentList().add(this);
		model.getAgentMap().put(new Integer(this.id), this);
		this.birthTick = (int)model.getTickCount();
		grant = new Grant(Settings.NO_GRANT, this);		
	}
	
	/*
	 * friendship is dyadic (symmetric)  
	 */	
	public void initializeFriends() {
		if (this.ageGroup == Settings.AGE.CHILD) {
			return;
		}		
		int friends = this.friends.size();
		SimUtilities.shuffle(model.getAgentList());
		for (Agent agent: model.getAgentList()) {
			if ((agent.getAgeGroup() != Settings.AGE.CHILD) 
					&& (this.houseHoldID != agent.getHouseHoldID())
					&& (this.id != agent.getId())) {
				if ((friends < Settings.FRIENDSHIP_UPPER_LIMIT) && (!this.friends.contains(agent))) {
					this.friends.add(agent);					
					agent.getFriends().add(this);
					friends++;
				}
			}
		}		
	}
	
	
	public void setAge(int _age) {
		age = _age;
	}
	
	public int getAge() {
		return this.age;
	}
	
	public int getGrantValue() {
		return this.grant.getType();
	}
	
	public Grant getGrant() {
		return this.grant;
	}
	
	public void setGrant(int grantType) {
		this.grant = new Grant(grantType, this);
	}
	
	public void removeGrant() {
		this.grant = new Grant(Settings.NO_GRANT, this);
	} 		

	public int getAgeGroup() {
		return ageGroup;
	}

	public void setAgeGroup(int _ageGroup) {
		this.ageGroup = _ageGroup;
	}

	public int getGender() {
		return gender;
	}

	public void setGender(int gender) {
		this.gender = gender;
	}

	public int getHealthCount() {
		return healthCount;
	}

	public void setHealthCount(int _healthCount) {
		this.healthCount = _healthCount;
	}
	
	public void setHealthStatus(int _healthStatus) {
		this.healthStatus = _healthStatus;
	}
	
	public int getHealthStatus() {
		return this.healthStatus;
	}

	public int getHouseHoldID() {
		return houseHoldID;
	}

	public void setHouseHoldID(int houseHoldID) {
		this.houseHoldID = houseHoldID;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}
	
	public void incrementAge(){
		this.age++;
	}

	public void setIsAlive(boolean flag) {
		this.isAlive = flag;
	}
	
	public boolean getIsAlive() {
		return this.isAlive;
	}
	
	/* this does a lot of stuff ...  to start with: killing agents */
	public void step(int currentTick) {				
		if ((currentTick - this.birthTick) % 12 == 0) { 
			updateAge();
		}		
		//health is updated every month
		updateHealthCount(currentTick);		
		if (dead()) {
			//System.out.println("This is dead.");
		}		
		else {			
			//check if eligible for external labor ... 			
			isLabourFit(currentTick);
//			processOutbox();
			if ((currentTick - this.lastTryStokvel) > 6) { // wait 6 months to try again
				this.setCommitted(false);
				this.setNumStokvelTries(0); //### will this work???
			   this.lastTryStokvel = currentTick;
			}
		}
	}
	
	public void updateHealthCount(int currentTick) {
		//this can be optimized ... it's not, right now		
//		System.out.println("Agent: " + this.id + " has current health count: " + this.healthCount);
//		if (this.healthStatus == Settings.HEALTH_STATUS.HIV) {
//			System.out.println("");
//			
//		}
		if ((this.healthStatus == Settings.HEALTH_STATUS.HIV) 
				&& ((currentTick - this.infectedTick)) >= Settings.INFECTED_LAG ) {
			fastDecayHealth(currentTick);
			return;
		}			
		switch (this.feedStatus) {
		case Settings.FEED_STATUS.FULL_FED:
			if (this.healthCount < 98) {
				this.healthCount += 2;
			}
			else {
				this.healthCount = 100;
			}									
			break;
		case Settings.FEED_STATUS.HALF_FED:		
			this.healthCount -= 2;
			break;
		case Settings.FEED_STATUS.NOT_FED:
			this.healthCount -= 5;
			//testing ...
			if (this.healthCount <= 50) {
				if (this.ageGroup == Settings.AGE.SENIOR) {
					this.healthStatus = Settings.HEALTH_STATUS.DISABLE;
				}
				else {
					this.healthStatus = Settings.HEALTH_STATUS.ILL;
				}	
			}
			
			break;
		default: break;
		}
//		System.out.println(" and now the updated is : " + this.healthCount);
	}
	
	public void fastDecayHealth(int currentTick) {	
		this.healthCount *= model.getSig().valueofFunction(this.age - this.infectedAge);											
	}	
	
	public void updateAge() {
		//System.out.println("agent: " + this.id + ", previous age: " + this.age);
		this.age++;
		//System.out.println("agent: " + this.id + ", new age: " + this.age);			
		boolean ageGroupChanged = false;		
		if ((this.ageGroup == Settings.AGE.CHILD) 
				&& (this.age > Settings.CHILD_AGE_LIMIT) 
				&& (this.age <= Settings.ADULT_AGE_LIMIT)) {
			setAgeGroup(Settings.AGE.ADULT);
			ageGroupChanged = true;
		}
		else if ((this.ageGroup == Settings.AGE.ADULT) 
				&&(this.age > Settings.ADULT_AGE_LIMIT)) {
			setAgeGroup(Settings.AGE.SENIOR);
			ageGroupChanged = true;			
		}	
		
		if (ageGroupChanged && (this.ageGroup == Settings.AGE.ADULT)){			
			Household household = (Household) model.getHouseMap().get(new Integer(this.houseHoldID));
			household.updateAgentAgeGroup(this, this.ageGroup);
			//SJA Mar 02:  the child who becomes adult? Does she gets any friends?
			// we were not doing that before  ... 
			initializeFriends(); 
		}
	}	
	
	/*
	 * there could be several reasons for a person dying ... so that's why we've got a separate
	 *  method - to kill ... 
	 * - old age ==  Need Normal Dist central at 50 ~ life expectancy rate: South Africa
	 * - hiv/aids 
	 * - lack of nutrition or illness
	 */
	public boolean dead(){
		if ((this.age >= this.expectedLife) || (this.healthCount <= 0)) {
			this.isAlive = false;			
			return true;
		}		
		else {
			return false;
		}
	}
	
	//### renamed so Jess doesn't think GPHousehold is a slot (attribute)
	public Household findGPHousehold() {
		Agent father = model.getAgentMap().get(new Integer(this.fatherID));
		Agent mother = model.getAgentMap().get(new Integer(this.motherID));	

		if ((father.getFatherID() == -1) || (mother.getFatherID() == -1)) {
			return null;
		}		
		Household gpHouseohold = retrieveGPHousehold(father); 
		if (gpHouseohold != null) {
			return gpHouseohold;
		}
		else {
			return retrieveGPHousehold(mother);
		}
	}
	
	public boolean fertility(int currentTick) {
		if (this.gender == Settings.GENDER.MALE) {
			return false;
		}		
		if ((this.healthStatus == Settings.HEALTH_STATUS.HIV) 
				&& ((currentTick - this.infectedTick) >= Settings.FERTILITY_DURATIION))				
		{
//			System.out.println("********* fertility declined ... .*****");			
			return false;
		}
		return true;
	}	

	public Household retrieveGPHousehold (Agent parent) {
		Agent grandParent = null;		
		Household household = null;
		grandParent = model.getAgentMap().get(new Integer(parent.getFatherID()));

		if (grandParent.getIsAlive()) {
			household = (Household) model.getHouseMap().get(new Integer(grandParent.getHouseHoldID()));
		}
		else {
			grandParent = model.getAgentMap().get(new Integer(parent.getMotherID()));
			if ((grandParent != null) && (grandParent.getIsAlive())) {
				household = (Household) model.getHouseMap().get(new Integer(grandParent.getHouseHoldID()));
			}			
		}		
		return household;		
	}
	
	/**
	 * following meeting with Gina at Aberdeen ... usually, it's 60% male and 20% female 
	 */
	public boolean isLabourFit(int currentTick) {
        this.remittance = false;        
        
         //for someone to be eligible for labor: they have to be (currently), adult and not getting a grant         
        if ((this.ageGroup != Settings.AGE.ADULT) || (this.grant.getType() != Settings.NO_GRANT)) {            
            return false;
        }
        
        //usually, it's 60% male and 20% female getting the labor job ... this is negation down here         
        if (((this.gender == Settings.GENDER.FEMALE) && (Math.random() > 0.25))
        		|| ((this.gender == Settings.GENDER.MALE) && Math.random() > 0.65)) {
        	return false; 
        }
                           
        if (((this.healthStatus != Settings.HEALTH_STATUS.HIV)
                ||  ((this.infectedTick + Settings.INFECTED_LAG) <= currentTick))
                && (Math.random() <= Settings.LABOUR_PROBABILITY)) {        	
                 this.remittance = true;
                this.remittance_Amount = (int) Normal.staticNextDouble(Settings.REMITTANCE.MEAN,
                        Settings.REMITTANCE.STD_DEV);
                return true;
            }        
        else {
            return false;
        }              
    }
	
	
//	public void sendInvitation() {
//		//just an upper bound ... so that not all social neighbours are 
//		//sent the invitation...
////		int numNeighbours = Settings.FRIENDSHIP_UPPER_LIMIT;
//		
//		HouseholdNode myNode = null;
//		
//		//feb16 shah
//		if (model.getGui().getHouseholdNodeMap().get(new Integer(this.houseHoldID)) == null) {
//			return;
//		}
//		
//		myNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.houseHoldID)).node;
//
//		ArrayList<HouseholdNode> neighbours = myNode.getToNodes();
//		SimUtilities.shuffle(neighbours);
//		for (HouseholdNode node : neighbours) {
////			if (this.invited.size() >= numNeighbours) {
////				break;
////			}
//			Agent neighbourHead = node.getHousehold().getHead();			
//			//this is an strict assumption ... the gender one ... 
//			if ((neighbourHead != null) && (neighbourHead.getGender() == Settings.GENDER.FEMALE)) {
////				Message message = new Message(this, neighbourHead, Settings.MESSAGE_TYPE.CLUB_INVITATION);
////				outbox.add(message);
//				//### assert message fact in Jess
//				try {
////					this.model.engine.executeCommand("(assert (message (sender " + this + ") (receiver " + neighbourHead + ") (type stokvel-invitation)))");
//					this.model.engine.executeCommand("(assert (message (sender " + this.id + ") (receiver " + neighbourHead.id + ") (type stokvel-invitation)))");
//				}
//				catch(JessException e){
//					System.out.println(e.getMessage());
//					e.printStackTrace();			
//				}	
//				this.invited.add(neighbourHead);									
//			}
//		}				
//		//this could be a bad idea ... 
//		ArrayList<Agent> temp = new ArrayList<Agent>(this.friends);
////		for (Agent friend : this.friends) {
//		for (Agent friend : temp) {
//			if (!model.getAgentList().contains(friend)) {
//				//if friend is dead ... no longer exists
//				//CAUTION: BUGGY
//				this.friends.remove(friend);								
//			}
//			else if ((friend.getGender() == Settings.GENDER.FEMALE)
//					&& (!this.invited.contains(friend))
//					&& (friend.isHead() == true)) {
////				Message message = new Message(this, friend, Settings.MESSAGE_TYPE.CLUB_INVITATION);
////				outbox.add(message);
//				//### assert message fact in Jess
//				try {
////					this.model.engine.executeCommand("(assert (message (sender " + this + ") (receiver " + friend + ") (type stokvel-invitation)))");
//					this.model.engine.executeCommand("(assert (message (sender " + this.id + ") (receiver " + friend.id + ") (type stokvel-invitation)))");
//				}
//				catch(JessException e){
//					System.out.println(e.getMessage());
//					e.printStackTrace();			
//				}	
//				this.invited.add(friend);
//			}
//		}
//		//### set committed flag to true so that the initiate-stokvel rule in Jess fires only once
//		setCommitted(true);
//		incNumStokvelTries(); // WHY does Jess need this instead of just the committed flag?????
//	}
	
	public void sendInvitation(){
		//### set committed flag to true so that the initiate-stokvel rule in Jess fires only once
		setCommitted(true);
		inviteStokvelCandidates(getStokvelCandidates());
		incNumStokvelTries(); // WHY does Jess need this instead of just the committed flag?????	
	}
	
	protected ArrayList<Agent> getStokvelCandidates(){
		HouseholdNode myNode = null;
		ArrayList<Agent> candidates = new ArrayList<Agent>();
		
		if (model.getGui().getHouseholdNodeMap().get(new Integer(this.houseHoldID)) == null) {
			// return empty list
			return candidates;
		}
		
		myNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.houseHoldID)).node;
		// fetch all social neighbours who are female and household heads
		ArrayList<HouseholdNode> neighbours = myNode.getToNodes();
//		SimUtilities.shuffle(neighbours);
		for (HouseholdNode node : neighbours) {
			Agent neighbourHead = node.getHousehold().getHead();			
			//this is an strict assumption ... the gender one ... 
			if ((neighbourHead != null) && (neighbourHead.getGender() == Settings.GENDER.FEMALE)) {
				candidates.add(neighbourHead);									
			}
		}				
		// fetch all friends who are female and household heads. 
		ArrayList<Agent> temp = new ArrayList<Agent>(this.friends);
		for (Agent friend : temp) {
			if (!model.getAgentList().contains(friend)) {
				//if friend is dead ... no longer exists
				//CAUTION: BUGGY
				this.friends.remove(friend);								
			}
			else if ((friend.getGender() == Settings.GENDER.FEMALE)
					&& (!candidates.contains(friend))
					&& friend.isHead()) {
				candidates.add(friend);
			}
		}
		return candidates;
	}
	
	protected void inviteStokvelCandidates(ArrayList<Agent> candidates){
		for (Agent agent : candidates){
			try {
				System.out.println("Agent \t" + this.id + "\t invites agent \t" + agent.id);
				this.model.engine.executeCommand("(assert (message (sender " + this.id + ") (receiver " + agent.id + ") (type stokvel-invitation)))");
				// record stuff in stokvel network
				this.model.updateStokvelNetwork(this.id, agent.id, (this.numStokvelTries==0? INVITE : INVITE2));
			}
			catch(JessException e){
				System.out.println(e.getMessage());
				e.printStackTrace();			
			}	
			this.invited.add(agent);
		}
	}
	
//	public void processInbox () {		
//		for (Message message : this.inbox) {
//			handleMessage(message);
//		}		
//		this.inbox.clear();
//		if (!this.outbox.isEmpty()) {
//			model.getOutbox().addAll(this.outbox);
//			this.outbox.clear();
//		}
//	}
//	
//	public void processOutbox() {		
//	if (sendStokvelInvitation()) {
//			sendInvitation();		
//		}		
//	}
	
//	public boolean sendStokvelInvitation() {
//		/*If I'm not an innovator, or already a member of a savings club (Stokvel) or if I'm not a head
//		 * or if i'm a male ... i wouldn't invite ...the underlying assumption is that, 
//		 * at the rural level, stokvels are run by women ... which maybe strict, and needs improvement ... 
//		 */		
//		if (!this.isInnovator || isMemberSavings || !isHead || (this.gender != Settings.GENDER.FEMALE)) {
//			return false;
//		}			
//		return isSavingsPossible();	
//	}
	
//	public boolean isSavingsPossible() {
//		//now check whether I have the money to start a stokvel myself
//		double wealth;
//		Household myHousehold = (Household) model.getHouseMap().get(new Integer(this.houseHoldID));
//		wealth = myHousehold.getWealth();		
//		if (wealth < Settings.STOKVEL_AMOUNT) {
//			return false;
//		}		
//		return true;
//	}
	
	public void handleMessage(Message message) {
		switch (message.getMessage()) {
		case Settings.MESSAGE_TYPE.NO_MESSAGE:
			break;
//		case Settings.MESSAGE_TYPE.CLUB_INVITATION:
//			handleIntivation(message);
//			break;		
//		case Settings.MESSAGE_TYPE.ACCEPTANCE:
//			handleAcceptance(message);
//			break;			
//		case Settings.MESSAGE_TYPE.REJECTION:
//			handleRejection(message);
//			break;
		default: break;
		}				
	}
	
//	public void handleIntivation(Message message) {
//		Agent sender = (Agent) message.getSender();		
//		if (!isSavingsPossible() || isMemberSavings || committed){
//			sendRejection(new Message(this, sender, Settings.MESSAGE_TYPE.REJECTION));
//		}
//		else {
//			//the sender of the original message is now the receiver
//			committed = true;
//			sendAcceptance(new Message(this, sender, Settings.MESSAGE_TYPE.ACCEPTANCE));
//		}		
//	}	
	
//	public void sendAcceptance(Message message) {		
//		outbox.add(message);
//	}
	
//	public void handleAcceptance(Message message) {
//		Agent sender = (Agent) message.getSender();
//		acceptances.add(sender);
//		invited.remove(sender);
//		
//		//-1 because one is us
//		if (invited.isEmpty() && (acceptances.size() >= Settings.MIN_SAVINGS_MEMBERS-1)) {
//			this.acceptances.add(this);
//		}				
//	}
	
	//### for calling from Jess
//	public void handleAcceptance(Agent sender) {
//		acceptances.add(sender);
//		invited.remove(sender);
//		
//		//-1 because one is us
//		if (invited.isEmpty() && (acceptances.size() >= Settings.MIN_SAVINGS_MEMBERS-1)) {
//			this.acceptances.add(this);
//			// moved here from postoffice
//			createStokvel();
//		}				
//	}

	public void handleAcceptance(int senderID) {
		Agent sender = model.getAgentMap().get(new Integer(senderID));
		acceptances.add(sender);
		invited.remove(sender);
		
		checkInvitationProgress();
		
	}
	
	private void checkInvitationProgress() {
		// everyone replied?
		if (invited.isEmpty()) {
			if ((acceptances.size() >= Settings.MIN_SAVINGS_MEMBERS-1)) {
				// enough people to form a stokvel 
				this.acceptances.add(this);
				createStokvel(); 	// moved here from postoffice
			}
			else {
				// lets ask all acceptances for their friends and relations and invite them too
				if (this.numStokvelTries <= 1) {
					System.out.println("*********** agent \t" + this.id + "\t invites the friends of friends to form a stokvel");
					ArrayList<Agent> candidates = new ArrayList<Agent>();
					for (Agent agent : acceptances){
						ArrayList<Agent> temp = agent.getStokvelCandidates();
						for (Agent candidate : temp){
							if (!candidates.contains(candidate) && !acceptances.contains(candidate) && candidate!=this) 
								candidates.add(candidate);
						}
					}
					inviteStokvelCandidates(candidates);
					incNumStokvelTries();
				}
				else {
					// not enough accepts --> disband
					for(Agent agent : acceptances){
						agent.setCommitted(false);
					}
					this.acceptances.clear();
//					this.setCommitted(false); // <--- have to wait for several ticks, cos this makes Jess fire the invitation rule immediately again!
					this.lastTryStokvel = model.getTickCount();
					System.out.println("*********** not enough acceptances to form a stokvel for agent \t" + this.id);
				}
			}
		}
	}
	
	// for calls from rules
	public void updateStokvelNetwork(int toAgent, int what){
		this.model.updateStokvelNetwork(this.id, toAgent, what);
	}
	
	public void createStokvel() {		
		SavingsClub club = new SavingsClub(Club.MEMBERSHIP_TYPE.BY_REFERENCE,this, this.acceptances, model);
		
		System.out.print("\nAgent \t" + this.id + "\t created new savings club with members ");
		for (Agent member : this.acceptances){
			System.out.print(member.id + " ");
			updateStokvelNetwork(member.id, STOKVEL);
		}
		System.out.println("\n");
		
		this.acceptances.clear();
		model.createSavingsClub(club);
	}
	
//	public void sendRejection(Message message) {
//		outbox.add(message);
//	}
//	
//	public void handleRejection(Message message) {	
//		Agent sender = (Agent) message.getSender();
//		invited.remove(sender);
//	}
	
	//### for calling from Jess
//	public void handleRejection(Agent sender) {	
//		invited.remove(sender);
//	}
	
	//### for calling from Jess
	public void handleRejection(int senderID) {	
		Agent sender = model.getAgentMap().get(new Integer(senderID));
		invited.remove(sender);
//		// reset their committed flag
//		sender.setCommitted(false);
		
		checkInvitationProgress();
	}
	
	public static void main(String[] args) {
	}

	public int getFatherID() {		
		return fatherID;		
	}

	public void setFatherID(int fatherID) {
		this.fatherID = fatherID;
	}

	public int getMotherID() {
		return motherID;
	}

	public void setMotherID(int motherID) {
		this.motherID = motherID;
	}

	public int getMaritalStatus() {
		return maritalStatus;
	}

	public void setMaritalStatus(int maritalStatus) {
		this.maritalStatus = maritalStatus;
	}

	public boolean isOrphan() {
		return isOrphan;
	}

	public void setOrphan(boolean isOrphan) {
		this.isOrphan = isOrphan;
		model.numOrphans++;
	}

	public int getSpouseID() {
		return spouseID;
	}

	public void setSpouseID(int spouseID) {
		this.spouseID = spouseID;
	}

	public int getExpectedLife() {
		return expectedLife;
	}

	public void setExpectedLife(int expectedLife) {
		this.expectedLife = expectedLife;
	}
	
	/*
	 * this method used for monthly 
	 */
	public void setInfectedTick(int currentMonth) {
		this.infectedTick = currentMonth;
		this.infectedAge = this.age;
	}
	
	public int getInfectedTick() {
		return this.infectedTick;
	}

	public int getFeedStatus() {
		return feedStatus;
	}

	public void setFeedStatus(int feedStatus) {
		this.feedStatus = feedStatus;
	}

	public boolean isRemittance() {
		return remittance;
	}

	public void setRemittance(boolean remittance) {
		this.remittance = remittance;
	}

	public int getRemittance_Amount() {
		return remittance_Amount;
	}

	public void setRemittance_Amount(int remittance_Amount) {
		this.remittance_Amount = remittance_Amount;
	}
	
	public boolean isInnovator() {
		return isInnovator;
	}

	public void setInnovator(boolean isInnovator) {
		this.isInnovator = isInnovator;
	}

	public boolean isMemberSavings() {
		return isMemberSavings;
	}

	public void setMemberSavings(boolean isMemberSavings) {
		//### changed for Jess, so that changes are instantly visible in shadow facts
		boolean old = this.isMemberSavings;
		this.isMemberSavings = isMemberSavings;
		pcs.firePropertyChange("memberSavings", old, this.isMemberSavings);
		setCommitted(false);
//		this.isMemberSavings = isMemberSavings;
//		this.committed = false;
	}

	public boolean isHead() {
		return isHead;
	}

	public void setHead(boolean isHead) {
		this.isHead = isHead;
	}
	
//	public ArrayList<Message> getInbox() {
//		return inbox;
//	}
//
//	public void setInbox(ArrayList<Message> inbox) {
//		this.inbox = inbox;
//	}
//
//	public ArrayList<Message> getOutbox() {
//		return outbox;
//	}
//
//	public void setOutbox(ArrayList<Message> outbox) {
//		this.outbox = outbox;
//	}

	public ArrayList<Agent> getFriends() {
		return friends;
	}

	public void setFriends(ArrayList<Agent> friends) {
		this.friends = friends;
	}

	public boolean isPullOutMembership() {
		return pullOutMembership;
	}

	public void setPullOutMembership(boolean pullOutMembership) {
		this.pullOutMembership = pullOutMembership;
	}
	
	public void setSavingsPool(int sum) {
		Household household = (Household) model.getHouseMap().get(new Integer(this.houseHoldID));
		household.addWealth((double) sum);
	}
	
	public void flushMembership() {
		this.isMemberSavings = false;
		if (!this.pullOutMembership) {
			//if we were to contribute, but the club ceased to be viable 
			//we deposit back the money (which was deducted from the household pool,
			//back to the household
			setSavingsPool(Settings.STOKVEL_AMOUNT);
		}
		this.pullOutMembership = false;
	}

	public ArrayList<Agent> getAcceptances() {
		return acceptances;
	}

	public void setAcceptances(ArrayList<Agent> acceptances) {
		this.acceptances = acceptances;				
	}
	
	/*
	 * currently, it's not the youngest ... but picking the last male in the family
	 * ... the last one to marry ...  
	 */
	public boolean isYoungestSon() {
		//### this method proved problematic for Jess, generated an error when shadow facts for agents were generated
		//### for the time being, just set to return false
		
//		Household household = (Household) model.getHouseMap().get(new Integer(this.houseHoldID));		
//		int numMales = 0;
//		for (Agent agent : household.getAdults()) {
//			if (agent.getGender() == Settings.GENDER.MALE) {
//				numMales++;
//			}			
//		}
//		return numMales == 1 ? true : false;	
		return false;
	}
	
	//### methods added for Jess
	
	public int decideWhichStokvel(QueryResult result){
		ArrayList<Agent> invitingFriends = new ArrayList<Agent>();
		ArrayList<Agent> invitingNeighbours = new ArrayList<Agent>();
		
		while (result.next()){
			try {
				int senderID = result.getInt("sender");
				Agent sender = model.getAgentMap().get(new Integer(senderID));
				// is it a friend?
				if (this.friends.contains(sender)) invitingFriends.add(sender);
				else invitingNeighbours.add(sender);
			}
			catch(JessException e){
				System.out.println(e.getMessage());
				e.printStackTrace();							
			}
		}
		// if there are any invites from friends, choose randomly amongst them
		// otherwise, choose randomly amongst the neighbours
		if (invitingFriends.isEmpty()){
			if (invitingNeighbours.isEmpty()){
				System.out.println("----------- neither friends nor neighbours for agent \t" + this.id + "\t. OOPS!");
				return -1;
			}
			else {
				System.out.println("------------- agent \t" + this.id + "\t choosing a neighbour from \t" + invitingNeighbours.size());
				return chooseRandomly(invitingNeighbours);
			}
		}
		else {
			System.out.println("------------- agent \t" + this.id + "\t choosing a friend from \t" + invitingFriends.size());
			return chooseRandomly(invitingFriends);
		}
	}
	
	private int chooseRandomly(ArrayList<Agent> list){
		// choose a random agent from the list
		// and return its id
		SimUtilities.shuffle(list);
		return list.get(0).getId();
	}
	
	public boolean isCommitted() {
		return this.committed;
	}
	
	public void setCommitted(boolean committed){
		boolean old = this.committed;
		this.committed = committed;
		pcs.firePropertyChange("committed", old, this.committed);
	}
	
	public int getNumStokvelTries() {
		return this.numStokvelTries;
	}
	
	public void setNumStokvelTries(int n){
		int old = this.numStokvelTries;
		this.numStokvelTries = n;
		pcs.firePropertyChange("numStokvelTries", old, this.numStokvelTries);
	}
	
	public void incNumStokvelTries() {
		setNumStokvelTries(this.numStokvelTries + 1);
	}
	
	public void decNumStokvelTries() {
		setNumStokvelTries(this.numStokvelTries - 1);
	}
	
	
	public void addPropertyChangeListener(PropertyChangeListener p){
		pcs.addPropertyChangeListener(p);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener p){
		pcs.removePropertyChangeListener(p);
	}
	
}