package units;

import infastructure.Settings;
import java.util.ArrayList;
import java.util.HashMap;

// for Jess
import jess.JessException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import clubs.FuneralClub;
//import cropmodel.Farm;
import org.cfpm.ruth.modifiedCloud.FarmingHousehold; //###

import uchicago.src.sim.util.SimUtilities;
import gui.HouseholdNode;
import gui.Model;

public class Household {
	private int id = -1;
	private Agent head = null;
	private ArrayList resources;		
	private ArrayList <Agent> children = new ArrayList<Agent>();
	private ArrayList <Agent> adults = new ArrayList<Agent>();
	private ArrayList <Agent> seniors = new ArrayList<Agent>();
	//this might be redundant
	private ArrayList <Agent> members = new ArrayList<Agent>();
	private double wealth = 0;
	/*id of the cluster to which this household belongs*/
	private int clusterID = -1;
	
	//reference to the repast model
	private Model model;			
	private boolean noGuardians = false;
	//this is just for temporary purposes ... am checking it 
	private int currentMembersOnLabor = 0;
	
	/*SEI CLOUD Cropping Model: the farm, if any, belongs to the household */
	private FarmingHousehold farm;  //###
	
	private boolean isFarm = false;
	
	private boolean isMemberFuneralClub = false;
	
	private FuneralClub funeralClub = null;
	private int defaultFuneralClubCount = 0;
	
	//### for Jess: changes to attribute wealth need to be visible instantly in Jess
	private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
	
	
	public Household(){		
	}
	
	private ArrayList <Couple> coupleList = new ArrayList<Couple>();
	
	public Household(int _id, Model _model){
		this.id = _id;
		this.model = _model;
//		CAUTION: Assuming all householdsa are poor here ... as 'used' by CLOUD
//		farm = new Farm(model.getWorld(), model.getMarket(), this);  //###
//		farm.initialize();		 									 //###					
	}
	
	public int getId() {
		return this.id;		
	}
	
	/* This an important method that assigns head, roles, wealth and other attributes ...
	 * depending upon the logic 
	 */ 
	public void initialize() {
    	/* although as reported in Gina's study, out of 20 houses, 12 had male heads
    	 * and the rest female ... we're having a random pick for the head here ... 
    	 * we would like to make it more parameterized ... later on */
		SimUtilities.shuffle(this.adults);
		//System.out.println("adults size: " + adults.size());
		setHead(this.adults.get(0));			
	}
	
	public void addAgent(Agent agent) {
		switch (agent.getAgeGroup()){
		case Settings.AGE.CHILD:
			children.add(agent);
			break;		
		case Settings.AGE.ADULT:
			adults.add(agent);
			break;
		case Settings.AGE.SENIOR:
			seniors.add(agent);
			break;
		default: break;
		}
		agent.setHouseHoldID(this.id);
		this.members.add(agent);
		
		//###
		if (this.isFarm) this.farm.addMember();

	}
	
	/*
	 * this gets more complex when an adult dies because we then might have to 
	 * move the household to another if there's no more adult left ... 
	 */
	public void removeAgent(Agent agent) {
		switch (agent.getAgeGroup()){
		case Settings.AGE.CHILD:
			if (children.contains(agent)) {
				children.remove(agent);	
			}			
			break;		
		case Settings.AGE.ADULT:
			//either married or widowed ...
			int maritalStatus = agent.getMaritalStatus();	
			if (maritalStatus == Settings.MARITAL_STATUS.MARRIED) {
				Agent spouse = model.getAgentMap().get(new Integer(agent.getSpouseID()));
				if (spouse != null) {
					spouse.setMaritalStatus(Settings.MARITAL_STATUS.WIDOWED);					
				}				
				makeChildrenOrphan(agent);
			}								
			adults.remove(agent);
			break;
		case Settings.AGE.SENIOR:
			seniors.remove(agent);
			break;
		default: break;		
		}		
		
		boolean isHead = false;
		if (this.getHead() != null) {
			if ((agent.getId() == this.getHead().getId())) {
				isHead = true;
				//this.setHead(null);
				this.head = null;
			}
		}
		
		boolean flag = 	joinFuneralClub(agent);		 
		if ((isHead) && (!this.adults.isEmpty())) {
	    	SimUtilities.shuffle(this.adults);
	    	setHead(adults.get(0));
		}
		
		if (!this.isMemberFuneralClub && flag && (this.head != null)) {
			FuneralClub club = findFuneralClub();
			if (club != null) {
				//we join the club
				this.isMemberFuneralClub = true;
				club.addMember(this);
				this.funeralClub = club;				
			}
		}
		else {
			if ((agent.getAgeGroup() == Settings.AGE.ADULT) 
					&& (this.isMemberFuneralClub)) {
				this.funeralClub.request(this);
			}
		}
		
		//now pay for the funeral cost ... 
		//there's a catch in this following line ... if the wealth goes zero and 
		//we're not farming ... this means 
//		this.wealth -= Settings.FUNERAL_COST;		
		deductWealth((double) Settings.FUNERAL_COST);
//		System.out.println("Funeral Cost deducted for household: " + this.id + " --- tick: " + model.getTickCount());
		this.members.remove(agent);
		
		if (this.isFarm) this.farm.removeMember(); //###
	}	
	
	public void makeChildrenOrphan(Agent agent) {		
		for (Agent member : this.children) {
			member.setOrphan(true);			
		}
	}
	
	public void updateAgentAgeGroup(Agent agent, int ageGroup) {
		if (ageGroup == Settings.AGE.ADULT) {
			this.children.remove(agent);
			this.adults.add(agent);
		}
		else if (ageGroup == Settings.AGE.SENIOR) {
			this.adults.remove(agent);
			this.seniors.add(agent);
		}
	}
		
	public ArrayList<Agent> getMembers() {
		return this.members;
	}
	
	public int getSize() {
		return this.members.size();
	}
	
	public int getClusterID(){
		return this.clusterID;
	}
	
	public void setClusterID(int _clusterID){
		this.clusterID = _clusterID;
	}
	
	public double getWealth() {
		if (this.isFarm) {
//			return farm.getCashInHand();
			return farm.getWealth();
		}
		else {
			return this.wealth;			
		}
	}
	
	public void setWealth(double _wealth) {
		//### changed for Jess
		double old = this.wealth;
		this.wealth = _wealth;
		pcs.firePropertyChange("wealth", old, this.wealth);
	} 	
		
	public ArrayList<Agent> getChildren(){
		return this.children;
	}
	
	public ArrayList<Agent> getAdults(){
		return this.adults;
	}

	public ArrayList<Agent> getSeniors(){
		return this.seniors;
	}	
	
	public Agent getHead() {
		return head;
	}
	
	public void setHead(Agent _head) {
		this.head = _head;
		_head.setHead(true);
		//### for Jess: put household head as shadow fact into memory
		//-- for the time being, only household heads are used as agents in Jess
		try {
			this.model.engine.definstance("agent", _head, true);
		}
		catch (JessException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();							
		}
		
	}

	public ArrayList getResources() {
		return resources;
	}
	
	public void preStep() {		
		if (!existGuardian()) {
			//System.out.println("**************no more guardians remaining in the household.");
			this.noGuardians = true;		
			return;
		}				
	}
	
	/*
	 * there needs to be two cases: 
	 * 1. When model.isFarm is TRUE ... that is there's farming going on.
	 * 2. When no farming, and the sole income is labor
	 */	
	public void step() {
		double income = accumulateIncome();		
		addWealth(income);
//		System.out.println("Tick: " + model.getTickCount() + "  ...household: " + this.id + " ... members: " + this.members.size() + " ... income: " + income + 
//				" ... wealth: " + this.wealth);	
		int feedStatus = Settings.FEED_STATUS.NOT_FED;				
		if (isFarm) {
			feedStatus = feedWithFarm(income);
		}
		else {
//			feedStatus = feedWithoutFarm(income);
			feedStatus = feedWithoutFarm();
		}			
		feedMembers(feedStatus);
		if(model.healthExpense) {
			deductHealthExpenditure();
		}
		deductClubsFee();
//		System.out.println("Tick: " + model.getTickCount() + "  ...household: " + this.id + " ... members: " + this.members.size() + " ... income: " + income + 
//				" ... wealth: " + this.wealth);		
	}
	
	public void deductHealthExpenditure() {
		int countIlls = 0;
		for (Agent agent : this.members) {
			if (agent.getHealthStatus() != Settings.HEALTH_STATUS.WELL) {
				countIlls++;
			}
		}
		int exp = countIlls * Settings.HEALTH_EXPENSE;
//		System.out.println("Household: " + this.id + " has health expenditure:" + exp + " --- tick: " + model.getTickCount());
		deductWealth((double) exp);		
	}
	
	public int feedWithFarm(double income) {
		int feedStatus = Settings.FEED_STATUS.NOT_FED;			
//###		farm.step();
		if (farm.getDeficit() == 0) {
			feedStatus = Settings.FEED_STATUS.FULL_FED;
		}		
//		else if ((farm.getDeficit() > 0) && (income > 0)) {
//			feedStatus = Settings.FEED_STATUS.HALF_FED;
//		}
		else if (income > 0) {
			feedStatus = Settings.FEED_STATUS.HALF_FED;
		}	
		return feedStatus;		
	}
	
	public int feedWithoutFarm() {				
		int feedStatus = Settings.FEED_STATUS.NOT_FED;		
		
//		int requirement = this.members.size() * Settings.MONEY_TO_FEED;
		int requirement = (this.members.size() - this.currentMembersOnLabor) * Settings.MONEY_TO_FEED;
		
		if (wealth >= requirement) {
			feedStatus = Settings.FEED_STATUS.FULL_FED;
		}
		else if ((wealth < requirement) && (wealth > requirement / 4)) {
			feedStatus = Settings.FEED_STATUS.HALF_FED;
		}
		return feedStatus;
	}	
	
	public void feedMembers(int fed_status) {
		for (Agent agent : this.members) {
			if (!agent.isRemittance()) {
				agent.setFeedStatus(fed_status);
			}
			else {
				agent.setFeedStatus(Settings.FEED_STATUS.FULL_FED);
			}
		}
 	}
	
	/*
	 * it deducts (or supposed to deduct)the fee for the funeral club or savings 
	 * club, the household is member of ... the membership is annual here ... 
	 * so every month, some money is accumulated in the fund for clubs ... 
	 * if there's no money at a moment, the household may get money out of it
	 * ... this may force a household to pull out of a savings club 
	 */
	public void deductClubsFee() {
		/*our first priority is to be able to pay for teh funeral club
		 * we're member of ... 
		 */		
		if (this.head == null) {
			return;
		}
		
		//if this hh is a memeber of a funeral club ...
		int diff = (int) this.wealth - Settings.FUNERALCLUB_FEE;
		if ((this.isMemberFuneralClub) && ((diff > 0))){
			defaultFuneralClubCount++;
			if (defaultFuneralClubCount > 0) defaultFuneralClubCount = 0;
			deductWealth((double) Settings.FUNERALCLUB_FEE);
			this.funeralClub.addContribution(Settings.FUNERALCLUB_FEE);
		}
		else {
			defaultFuneralClubCount--;
		}
		
		//now for the savings club (stokvel)
		if (this.head.isMemberSavings()) {
			if (this.wealth - Settings.STOKVEL_AMOUNT > 0) {
				//if the head is the member of a savings club
				//deduct the club fee			
				deductWealth(Settings.STOKVEL_AMOUNT);
			}
			else {
				this.head.setPullOutMembership(true);
			}
		} 
	}		
	
	public double accumulateIncome() {
		//this is the income this month 
		double income = 0;
		this.currentMembersOnLabor = 0;
		for (Agent agent : this.members) {			
			if (agent.isRemittance()) {
				income += agent.getRemittance_Amount();				
				currentMembersOnLabor++;
			}
			else {
				income += agent.getGrantValue();
			}
		}		
		return income;
	}
		
	/*
	 * this checks if any guardians remnains or not
	 * Question: what if there's not even a child as well? ?// no problem, this is a
	 * code optimization issue  
	 */
	public boolean existGuardian() {
		if (this.members.isEmpty()) {
			return false;
		}
		
 		if (!this.adults.isEmpty()) {
			return true; 			
		}
		for (Agent senior : this.seniors) {
			if (senior.getGrant().getType() == Settings.PENSION_GRANT) {
				if (this.head == null) {
					setHead(senior);
				}
				return true;
			}
		}			
		return false;	
	}
	
	public boolean getNoGuardians() {
		return this.noGuardians;
	}

	public FarmingHousehold getFarm() { //###
		return farm;
	}

//	public void setFarm(Farm farm) {
//		this.farm = farm;
//	}
	
	//this is a weird implementation ... sorry
	public void addWealth(double money) {		
		if (isFarm) {
			farm.addCash(money);  //#####
			wealth = farm.getWealth();
		}
		else {
			this.wealth += money;			
		}
	}
	
	public void deductWealth(double money) {
		if (isFarm) {
			farm.removeCash(money);
			wealth = farm.getWealth();
		}
		else {
			this.wealth -= money;
		}
		
		if (this.wealth < 0) {
			this.wealth = 0;
		}
	}

	public boolean isFarm() {
		return isFarm;
	}

	public void setFarm(boolean isFarm) {
		this.isFarm = isFarm;
		// #### create the farm
		if (isFarm)
		this.farm = model.getWorld().addNewHousehold(this.getSize(), this.wealth, this.id);
	}
	
	/*
	 * find a neighbour .. in future we would like to give preference to those to whom we have stronger/multiple ties
	 */
	public Household accommodateNeighbour() {
		HouseholdNode householdNode = (HouseholdNode) model.getGui().getHouseholdNodeMap().get(new Integer(this.id)).node;
		ArrayList neighbourNodes = householdNode.getToNodes();
		HouseholdNode neighbour = new HouseholdNode();
		if (!neighbourNodes.isEmpty()) {
			neighbour = (HouseholdNode) neighbourNodes.get(0);			
			return neighbour.getHousehold();
		}
		else {
			System.out.println("No Neighbour found ... Can't Purge Household");
			return null;
		}
	}

	public FuneralClub getFuneralClub() {
		return funeralClub;
	}

	public void setFuneralClub(FuneralClub funeralClub) {
		this.funeralClub = funeralClub;
	}

	public boolean isMemberFuneralClub() {
		return isMemberFuneralClub;
	}

	public void setMemberFuneralClub(boolean isMemberFuneralClub) {
		this.isMemberFuneralClub = isMemberFuneralClub;
	}	    
	
	public boolean joinFuneralClub(Agent deceased) {
		//only a death of an adult household member from HIV/AIDS is (or if ) viewed as a threat
		//unfortunately, for children and seniors it's not considered ... 
		if ((deceased.getHealthStatus() != Settings.HEALTH_STATUS.HIV) 
				|| (deceased.getAgeGroup() != Settings.AGE.ADULT)
				|| (this.wealth < Settings.FUNERALCLUB_FEE)) {
			return false;
		}			
		else {
			return true;
		}
	}
	
	/*now once decided to join a funeral club, we choose which one to join
	 */  
	public FuneralClub findFuneralClub() {
		FuneralClub club = null;
		HouseholdCluster cluster = (HouseholdCluster) model.getClusterMap().get(this.getClusterID());
		if (cluster == null) {
//			System.out.println(" ... cluster null ... we're in household.findFunderalClub() ... ");
		}
		/*first we look if some other household in our hhcluster is a member 
		 * of any funeral club ... right now, we breakaway and choose the club
		 * as soon as we find the first hh in our cluster and returns its funeral club
		 */
		else {
			for (Household household : cluster.getHouseholds()) {
				if ((household.isMemberFuneralClub()) && (household.getFuneralClub() != null)) {
					club = (FuneralClub) household.getFuneralClub();
				}				
			}
		}
		
		if (club != null) {
			return club;
		}
		/*
		 * if our earlier search didnt work .. we look into the household head's friend 
		 * ... no current justification, why we're doing this ...and nothing else
		 * and this is badly written ... can be optimized ... it will slow down an
		 * already very very very slow simulation  
		 */				
		for (Agent agent : this.head.getFriends()) {
//			Household household = (Household) model.getHouseholdList().get(new Integer(agent.getHouseHoldID()));
			Household household = (Household) model.getHouseMap().get(new Integer(agent.getHouseHoldID()));
			if ((household.isMemberFuneralClub()) && (household.getFuneralClub() != null)) {
				club = (FuneralClub) household.getFuneralClub();
			}
		}
		
		if (club != null) {
			//join a random club ...
			ArrayList<FuneralClub> temp = new ArrayList<FuneralClub>(model.getFuneralClubs());
			SimUtilities.shuffle(temp);
			if (temp.size() > 0) {
				club = (FuneralClub) temp.get(0);
			}			
		}		
		return null;
	}

	public int getDefaultFuneralClubCount() {
		return defaultFuneralClubCount;
	}

	public void setDefaultFuneralClubCount(int defaultFuneralClubCount) {
		this.defaultFuneralClubCount = defaultFuneralClubCount;
	}

	public int getCurrentMembersOnLabor() {
//		return currentMembersOnLabor;
		int count = 0;
		for (Agent adult : this.adults) {
			if (adult.isRemittance()) {
				count++;
			}
		}
		return count;
	}	
	
	public int getCurrentAdultMales() {
		int count = 0;
		for (Agent adult : this.adults) {
			if (adult.getGender() == Settings.GENDER.MALE) {
				count++;
			}
		}
		return count;
	}
	
	//this is redundant method ... 
	public void accomodate(Household household) {
		int count=0;
		ArrayList<Agent> agents = new ArrayList<Agent>(household.getMembers());		
//		for (Agent agent : household.getMembers()) {			
		for (Agent agent : agents) {
			agent.setHouseHoldID(this.id);
			this.addAgent(agent);
			count++;
		}				
	}
	
	//###
	public void die(){
		// remove all references to members
		this.members = null;
		this.adults = null;
		this.children = null;
		this.seniors = null;
		// dissolve farm --> problem: who inherits the farm in reality??
		if (this.isFarm) farm.dissolve();
	}
	
	public void addCouple(Agent husband, Agent wife) {		
		husband.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		husband.setSpouseID(wife.getId());
		wife.setMaritalStatus(Settings.MARITAL_STATUS.MARRIED);
		wife.setSpouseID(husband.getId()); 
		Couple couple = new Couple(husband, wife);
		this.coupleList.add(couple);
	}

	public ArrayList<Couple> getCoupleList() {
		return coupleList;
	}

	public void setCoupleList(ArrayList<Couple> coupleList) {
		this.coupleList = coupleList;
	}	
	
	//### for Jess
	public void addPropertyChangeListener(PropertyChangeListener p){
		pcs.addPropertyChangeListener(p);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener p){
		pcs.removePropertyChangeListener(p);
	}
	
}