package units;

import gui.Model;
import java.util.ArrayList;
import uchicago.src.sim.util.SimUtilities;

public class HouseholdCluster {
	private int id = -1;
	/*houses which are members of household clusters*/
	private ArrayList <Household> housesholds = new ArrayList<Household>();
	/*It is assumed that the land parcels are jointly shared by the cluster and not
	 * just one household, which is stylized though, but let's go on with it 
	 */	
	private int members = 0;
	//a reference to the model
	private Model model;
	/* this is the money earned from cultivation
	 * to be distributed among the household members
	 * at some stage, we might need a pooling system as well 
	 */
	private int moneyEarned = 0;
	
	public HouseholdCluster() {		
	}
	
	public HouseholdCluster(int _id, Model _model) {
		this.id = _id;
		this.model = _model;
		/*actually, the household's property and 
		 * we see the aggregate to decide if the cluster is poort or not. */							
	}
	
	public int getMembers() {
		return this.members;
	}	
	
	public void upadateMembers() {
		int sum = 0;
		for (Household household : housesholds) {			
			sum += household.getSize();
		}	    			
		this.members = sum;
	}
	
	public int getId() {
		return this.id;
	}
	
	public void addHousehold(Household household) {
		this.housesholds.add(household);
		household.setClusterID(this.id);		
	}
	
	/*need to write better code */
	public void removeHousehold(Household household) {
		/*if there're any surviving members of this household
		 * then assign them to some other household in the cluster */				
		housesholds.remove(household);				
	}
	
	
	public ArrayList<Household> getHouseholds() {
		return this.housesholds;
	}
	
	public Model getModel() {
		return this.model;
	}	
	
	public int getMoneyEarned() {
		return this.moneyEarned;
	}
	
	public void setMoneyEarned(int _moneyEarned) {
		this.moneyEarned = _moneyEarned;
	}
	
	public void step() {
//		distributeWealth();		
	}
	
	public void distributeWealth() {		
		int share = housesholds.size();
		for (Household household : housesholds) {
			household.addWealth((double) moneyEarned / share);
		}		
	}
	
	//there is a catch ... there's an assumption that we'll always find an household to accommodate 
	//in the cluster ... 
	public Household accomodate(Household household) {
		if (this.housesholds.size() <= 1) {
			return null;
		}
		Household newHousehold = new Household();		
		/*
		 * check for the grandparents ...assuming the children are all
		 * siblings, meaning they either have same grandparents, or none
		 *  ... so picking up the first child only  
		 */
		System.out.println("Household to be purged: " + household.getId() + " -- having remaining members: " + household.getSize());
		if (!household.getChildren().isEmpty()) {			
			Agent child = household.getChildren().get(0);
			//retieve child's grandparent's (paternal or maternal) household, if alive
			newHousehold = child.findGPHousehold();
		}

		if ((newHousehold == null) || newHousehold.getId() == -1) {
			newHousehold = (Household) chooseHousehold(household);	
		}
		
		if (newHousehold == null) {
			System.out.println("Cant Purge household");
			return null;
		}

		int newHouseID = newHousehold.getId(), count = 0;
		
		ArrayList<Agent> agents = new ArrayList<Agent>(household.getMembers());
		//shah: feb 16: testing 
		if (newHouseID == household.getId()) {
			return null;
		}
		
//		for (Agent agent : household.getMembers()) {			
		for (Agent agent : agents) {
			agent.setHouseHoldID(newHouseID);
			newHousehold.addAgent(agent);
			count++;
		}
		this.removeHousehold(household);
		this.members += count;
		
		if (newHousehold == null) {
			System.out.println("Cant Purge household");
			return null;
		}
		else {
			return newHousehold;			
		}

	}
	
	/**choose the household which is best suited to accomodate this household's
	 * ... currently, we choose randomly, but then we need better logics here ...
	 * depedent upon the purged household ... and the status of the rest of the
	 * clusters ... we'll have to return to the case-studies and profiles from SEI
	 */
	public Household chooseHousehold(Household household){
		ArrayList<Household> temp = new ArrayList<Household>(housesholds);
		temp.remove(household);
		if (temp.size() <= 1) {
			return null;
		}		
		SimUtilities.shuffle(temp);
		//return the first house, after shuffling ... 
		return temp.get(0);
	}	
}